@@ -20,6 +20,13 @@
 #include "gwmdesktop.h"
 
 
+#define CONFIG_OBOX_BORDER 6
+#define CONFIG_OBOX_SPACING 8
+#define CONFIG_IBOX_BORDER 6
+#define CONFIG_IBOX_SPACING 4
+#define CONFIG_ITEM_BORDER 0
+#define CONFIG_ITEM_SPACING 4
+  
 
 /* --- prototypes --- */
 static void	gp_init_gui		(void);
@@ -34,7 +41,8 @@ static gboolean	gp_task_notifier	(gpointer	    func_data,
 					 GwmhTaskInfoMask   imask);
 static void	gp_about		(void);
 static void	gp_config_popup		(void);
-static gpointer	gp_config_find_value	(const gchar *path);
+static gpointer	gp_config_find_value	(const gchar       *path,
+					 gboolean           tmp_value);
 static void	gp_load_config		(const gchar	   *privcfgpath);
 static gboolean	gp_save_session		(gpointer	    func_data,
 					 const gchar	   *privcfgpath,
@@ -52,84 +60,90 @@ static GtkWidget	*gp_desk_widget[MAX_DESKTOPS] = { NULL, };
 static guint             gp_n_desk_widgets = 0;
 static GdkWindow	*gp_atom_window = NULL;
 static GtkOrientation    gp_orientation = GTK_ORIENTATION_HORIZONTAL;
+static guint             gp_panel_size = 12;
 static guint		 GP_TYPE_HBOX = 0;
 static guint		 GP_TYPE_VBOX = 0;
 static guint		 GP_ARROW_DIR = 0;
 static gchar		*DESK_GUIDE_NAME = NULL;
 
 static ConfigItem gp_config_items[] = {
-  CONFIG_SECTION (tooltip_settings,			 	N_ ("Tooltips")),
-  CONFIG_BOOL (desktips,	TRUE,
-	       N_ ("Enable Desktop Names")),
-  CONFIG_RANGE (desktips_delay,	10,	1,	5000,
-		N_ ("Desktop Name Popup Delay [ms]")),
-  CONFIG_BOOL (tooltips,	TRUE,
-	       N_ ("Enable Tooltips")),
-  CONFIG_RANGE (tooltips_delay,	500,	1,	5000,
-		N_ ("Tooltips Delay [ms]")),
-  CONFIG_SECTION (layout,				  	N_ ("Layout")),
+  CONFIG_PAGE (N_ ("Display")),
+  CONFIG_SECTION (sect_layout,				  	N_ ("Layout")),
   CONFIG_BOOL (switch_arrow,	FALSE,
-	       N_ ("Switch Arrow")),
-  CONFIG_BOOL (current_only,	FALSE,
-	       N_ ("Show only current Desktop")),
+	       N_ ("Switch tasklist arrow")),
+  CONFIG_BOOL (show_properties_button,	FALSE,
+	       N_ ("Show properties `?' button")),
   CONFIG_BOOL (show_pager,	TRUE,
-	       N_ ("Show Desktop Pager")),
+	       N_ ("Show desktop pager")),
+  CONFIG_BOOL (current_only,	FALSE,
+	       N_ ("Only show current desktop in pager")),
+  CONFIG_BOOL (raise_grid,		FALSE,
+	       N_ ("Raise area grid over tasks")),
+  CONFIG_SECTION (sect_tooltips,			 	N_ ("Tooltips")),
+  CONFIG_BOOL (tooltips,	TRUE,
+	       N_ ("Show Desk-Guide tooltips")),
+  CONFIG_RANGE (tooltips_delay,	500,	1,	5000,
+		N_ ("Desk-Guide tooltip delay [ms]")),
+  CONFIG_BOOL (desktips,	TRUE,
+	       N_ ("Show desktop name tooltips")),
+  CONFIG_RANGE (desktips_delay,	10,	1,	5000,
+		N_ ("Desktop name tooltip delay [ms]")),
   
   CONFIG_PAGE (N_ ("Tasks")),
-  CONFIG_SECTION (task_visibility,			  	N_ ("Visibility")),
+  CONFIG_SECTION (sect_task_visibility,			  	N_ ("Visibility")),
   CONFIG_BOOL (show_hidden_tasks,	TRUE,
-	       N_ ("Show `HIDDEN' Tasks")),
+	       N_ ("Show hidden tasks (HIDDEN)")),
   CONFIG_BOOL (show_shaded_tasks,	TRUE,
-	       N_ ("Show `SHADED' Tasks")),
+	       N_ ("Show shaded tasks (SHADED)")),
   CONFIG_BOOL (show_skip_winlist,	FALSE,
-	       N_ ("Show `SKIP-WINLIST' Tasks")),
+	       N_ ("Show tasks which hide from window list (SKIP-WINLIST)")),
   CONFIG_BOOL (show_skip_taskbar,	FALSE,
-	       N_ ("Show `SKIP-TASKBAR' Tasks")),
-  CONFIG_SECTION (cosmetics,				  	N_ ("Cosmetics")),
-  CONFIG_BOOL (raise_grid,		FALSE,
-	       N_ ("Raise Area Grid Above Tasks")),
-  /*  CONFIG_SECTION ("", NULL), */
+	       N_ ("Show tasks which hide from taskbar (SKIP-TASKBAR)")),
+  /*  CONFIG_SECTION (sect_null_1, NULL), */
   
   CONFIG_PAGE (N_ ("Geometry")),
-  CONFIG_SECTION (horizontal,                        	     	N_ ("Horizontal Layout")),
+  CONFIG_SECTION (sect_horizontal,                     	     	N_ ("Horizontal Layout")),
   CONFIG_RANGE (area_height,	44,	4,	1024,
 		N_ ("Desktop Height [pixels]")),
+  CONFIG_BOOL (abandon_area_height,		TRUE,
+	       N_ ("Override desktop height with panel size")),
   CONFIG_BOOL (div_by_vareas,		TRUE,
-	       N_ ("Divide Height By Number Of Vertical Areas")),
+	       N_ ("Divide height by number of vertical areas")),
   CONFIG_RANGE (row_stackup,	1,	1,	64,
-		N_ ("Number of Rows to pack Desktops in")),
-  CONFIG_SECTION (vertical,                        	     	N_ ("Vertical Layout")),
+		N_ ("Rows of Desktops")),
+  CONFIG_BOOL (div_by_nrows,		TRUE,
+	       N_ ("Divide height by number of rows")),
+  CONFIG_SECTION (sect_vertical,                       	     	N_ ("Vertical Layout")),
   CONFIG_RANGE (area_width,	44,	4,	1024,
-		N_ ("Desktop width [pixels]")),
+		N_ ("Desktop Width [pixels]")),
+  CONFIG_BOOL (abandon_area_width,		TRUE,
+	       N_ ("Override desktop width with panel size")),
   CONFIG_BOOL (div_by_hareas,		TRUE,
-	       N_ ("Divide Width By Number Of Horizontal Areas")),
+	       N_ ("Divide width by number of horizontal areas")),
   CONFIG_RANGE (col_stackup,	1,	1,	64,
-		N_ ("Number of Columns to pack Desktops in")),
+		N_ ("Columns of Desktops")),
+  CONFIG_BOOL (div_by_ncols,		TRUE,
+	       N_ ("Divide width by number of columns")),
 
-  CONFIG_PAGE (N_ ("Advanced Options")),
+  CONFIG_PAGE (N_ ("Advanced")),
+  CONFIG_SECTION (sect_drawing,                   	     	N_ ("Drawing")),
   CONFIG_BOOL (double_buffer,	TRUE,
-	       N_ ("Draw Desktops Double Buffered")),
-  CONFIG_SECTION (bug_fixes,                        	     	N_ ("Bug Fixes")),
+	       N_ ("Draw desktops double-buffered (recommended)")),
+  CONFIG_SECTION (sect_workarounds,         	     	N_ ("Window Manager Workarounds")),
   CONFIG_BOOL (skip_movement_offset,		TRUE,
-	       N_ ("Window Manager Moves Decoration Window Instead\n"
+	       N_ ("Window manager moves decoration window instead\n"
 		   "(AfterStep, Enlightenment, FVWM, IceWM, SawMill)")),
   CONFIG_BOOL (unified_areas,			TRUE,
-	       N_ ("Window Manager Changes Active Area On All Desktops\n"
+	       N_ ("Window manager changes active area on all desktops\n"
 		   "(FVWM, SawMill)")),
+  CONFIG_BOOL (violate_client_msg,			TRUE,
+	       N_ ("Window manager expects pager to modify area+desktop properties directly\n"
+		   "(Enlightenment, FVWM, SawMill)")),
 };
 static guint  gp_n_config_items = (sizeof (gp_config_items) /
 				   sizeof (gp_config_items[0]));
 
 
-/* --- FIXME: bug workarounds --- */
-GNOME_Panel_OrientType fixme_panel_orient = 0;
-/* #define applet_widget_get_panel_orient(x)	(fixme_panel_orient) */
-static void fixme_applet_widget_get_panel_orient (gpointer               dummy,
-						  GNOME_Panel_OrientType orient)
-{
-  fixme_panel_orient = orient;
-}
-
 /* --- main ---*/
 gint 
 main (gint   argc,
@@ -196,9 +210,10 @@ main (gint   argc,
   gwmh_desk_notifier_add (gp_desk_notifier, NULL);
   gwmh_task_notifier_add (gp_task_notifier, NULL);
   gtk_widget_set (gp_applet,
-		  "signal::change-orient", fixme_applet_widget_get_panel_orient, NULL,
 		  "signal::change-orient", gp_destroy_gui, NULL,
 		  "signal::change-orient", gp_init_gui, NULL,
+		  "signal::change-pixel-size", gp_destroy_gui, NULL,
+		  "signal::change-pixel-size", gp_init_gui, NULL,
 		  "object_signal::save-session", gp_save_session, NULL,
 		  "signal::destroy", gtk_main_quit, NULL,
 		  NULL);
@@ -246,7 +261,7 @@ gp_load_config (const gchar *privcfgpath)
 {
   static guint loaded = FALSE;
   guint i;
-  gchar *section = "general";
+  gchar *section = "sect_general";
   
   if (loaded)
     return;
@@ -266,9 +281,7 @@ gp_load_config (const gchar *privcfgpath)
 	{
 	  gchar *path = g_strconcat (section, "/",
 				     item->path, "=",
-				     GPOINTER_TO_INT (item->value)
-				     ? "true" :
-				     "false",
+				     GPOINTER_TO_INT (item->value) ? "true" : "false",
 				     NULL);
 	  
 	  item->value = GINT_TO_POINTER (gnome_config_get_bool (path));
@@ -290,7 +303,8 @@ gp_load_config (const gchar *privcfgpath)
 }
 
 static gpointer
-gp_config_find_value (const gchar *path)
+gp_config_find_value (const gchar *path,
+		      gboolean     tmp_value)
 {
   guint i;
   
@@ -298,8 +312,8 @@ gp_config_find_value (const gchar *path)
     {
       ConfigItem *item = gp_config_items + i;
       
-      if (path == item->path)
-	return item->value;
+      if ((path && item->path && strcmp (path, item->path) == 0) || path == item->path)
+	return tmp_value ? item->tmp_value : item->value;
     }
   
   g_warning (G_GNUC_PRETTY_FUNCTION "(): unable to find config value for <%s>", path);
@@ -313,7 +327,7 @@ gp_save_session (gpointer     func_data,
 		 const gchar *globcfgpath)
 {
   guint i;
-  gchar *section = "general";
+  gchar *section = "sect_general";
   
   gnome_config_push_prefix (privcfgpath);
   
@@ -441,19 +455,45 @@ gp_create_desk_widgets (void)
     g_error ("MAX_DESKTOPS limit reached, adjust source code");
 
   /* some gwmh configuration hacks ;( */
-  gwmh_desk_set_hack_values (BOOL_CONFIG (unified_areas));
+  gwmh_desk_set_hack_values (BOOL_CONFIG (unified_areas),
+			     BOOL_CONFIG (violate_client_msg));
 
   if (!gp_desk_box)
     return;
 
+  gtk_widget_ensure_style (gp_desk_box);
+
   /* configure Desktop widget class for us */
-  area_size = (gp_orientation == GTK_ORIENTATION_HORIZONTAL
-	       ? RANGE_CONFIG (area_height)
-	       : RANGE_CONFIG (area_width));
-  if (gp_orientation == GTK_ORIENTATION_HORIZONTAL && BOOL_CONFIG (div_by_vareas))
-    area_size /= (gdouble) N_VAREAS;
-  if (gp_orientation == GTK_ORIENTATION_VERTICAL && BOOL_CONFIG (div_by_hareas))
-    area_size /= (gdouble) N_HAREAS;
+  if (gp_orientation == GTK_ORIENTATION_HORIZONTAL)
+    {
+      if (BOOL_CONFIG (abandon_area_height))
+	area_size = gp_panel_size - 2 * gp_desk_box->style->klass->ythickness;
+      else
+	area_size = RANGE_CONFIG (area_height);
+      if (BOOL_CONFIG (div_by_vareas))
+	area_size /= (gdouble) N_VAREAS;
+      if (BOOL_CONFIG (div_by_nrows) && !BOOL_CONFIG (current_only))
+	{
+	  guint n = RANGE_CONFIG (row_stackup);
+
+	  area_size /= (gdouble) MIN (n, N_DESKTOPS);
+	}
+    }
+  else /* gp_orientation == GTK_ORIENTATION_VERTICAL */
+    {
+      if (BOOL_CONFIG (abandon_area_width))
+	area_size = gp_panel_size - 2 * gp_desk_box->style->klass->xthickness;
+      else
+	area_size = RANGE_CONFIG (area_width);
+      if (BOOL_CONFIG (div_by_hareas))
+	area_size /= (gdouble) N_HAREAS;
+      if (BOOL_CONFIG (div_by_ncols) && !BOOL_CONFIG (current_only))
+	{
+	  guint n = (gdouble) RANGE_CONFIG (col_stackup);
+
+	  area_size /= (gdouble) MIN (n, N_DESKTOPS);
+	}
+    }
   gwm_desktop_class_config (gtk_type_class (GWM_TYPE_DESKTOP),
 			    BOOL_CONFIG (double_buffer),
 			    gp_orientation,
@@ -472,11 +512,16 @@ gp_create_desk_widgets (void)
       if (gp_orientation == GTK_ORIENTATION_HORIZONTAL)
 	{
 	  n_desks_per_column = RANGE_CONFIG (row_stackup);
+	  n_desks_per_column = MIN (n_desks_per_column, N_DESKTOPS);
 	  n_desks_per_column = (N_DESKTOPS + n_desks_per_column - 1) / n_desks_per_column;
 	}
       else
-	n_desks_per_column = RANGE_CONFIG (col_stackup);
-      
+	{
+	  guint n = RANGE_CONFIG (col_stackup);
+
+	  n_desks_per_column = MIN (n, N_DESKTOPS);
+	}
+
       for (i = 0; i < max; i++)
 	{
 	  GtkWidget *alignment;
@@ -522,66 +567,50 @@ gp_check_task_visible (GwmDesktop *desktop,
     return TRUE;
 }
 
-static inline gboolean
-gp_widget_button_popup_task_editor (GtkWidget      *widget,
-				    GdkEventButton *event,
-				    gpointer        data)
+static void
+gp_widget_button_popup_task_editor (GtkWidget *widget,
+				    gpointer   data)
 {
-  guint button = GPOINTER_TO_UINT (data);
-
-
-  if (event->button == button)
-    {
-      g_message ("Task Editor unimplemented yet!");
-      gdk_beep ();
-
-      return TRUE;
-    }
-  else
-    return FALSE;
+  g_message ("Task Editor unimplemented yet!");
+  gdk_beep ();
 }
 
 static inline gboolean
-gp_widget_button_popup_config (GtkWidget      *widget,
-			       GdkEventButton *event,
-			       gpointer        data)
+gp_widget_ignore_button (GtkWidget *widget,
+			 GdkEvent  *event,
+			 gpointer   data)
 {
   guint button = GPOINTER_TO_UINT (data);
 
-
-  if (event->button == button)
+  if (event->type == GDK_BUTTON_PRESS ||
+      event->type == GDK_2BUTTON_PRESS ||
+      event->type == GDK_3BUTTON_PRESS ||
+      event->type == GDK_BUTTON_RELEASE)
     {
-      gp_config_popup ();
-
-      return TRUE;
-    }
-  else
-    return FALSE;
-}
-
-static inline gboolean
-gp_widget_ignore_button (GtkWidget      *widget,
-			 GdkEventButton *event,
-			 gpointer        data)
-{
-  guint button = GPOINTER_TO_UINT (data);
+      GdkEventButton *bevent = &event->button;
   
-  if (event->button == button)
-    gtk_signal_emit_stop_by_name (GTK_OBJECT (widget),
-				  event->type == GDK_BUTTON_PRESS
-				  ? "button_press_event"
-				  : "button_release_event");
+      if (bevent->button == button)
+	{
+	  if (widget->parent)
+	    gtk_propagate_event (widget->parent, event);
+
+	  return TRUE;
+	}
+    }
+
   return FALSE;
 }
 
 static void 
 gp_init_gui (void)
 {
-  GtkWidget *button, *abox;
+  GtkWidget *button, *abox, *arrow;
   gboolean arrow_at_end = FALSE;
   GtkWidget *main_box;
   
   gtk_widget_set_usize (gp_container, 0, 0);
+  gp_panel_size = applet_widget_get_panel_pixel_size (APPLET_WIDGET (gp_applet));
+  gp_panel_size = MAX (gp_panel_size, 12);
   switch (applet_widget_get_panel_orient (APPLET_WIDGET (gp_applet)))
     {
     case ORIENT_UP:
@@ -667,18 +696,17 @@ gp_init_gui (void)
   
   /* add arrow and button
    */
+  arrow = gtk_widget_new (GTK_TYPE_ARROW,
+			  "arrow_type", GP_ARROW_DIR,
+			  "visible", TRUE,
+			  NULL);
   button = gtk_widget_new (GTK_TYPE_BUTTON,
 			   "visible", TRUE,
 			   "can_focus", FALSE,
-			   "child", gtk_widget_new (GTK_TYPE_ARROW,
-						    "arrow_type", GP_ARROW_DIR,
-						    "visible", TRUE,
-						    NULL),
-			   "signal::button_press_event", gp_widget_button_popup_task_editor, GUINT_TO_POINTER (1),
-			   "signal::button_press_event", gp_widget_ignore_button, GUINT_TO_POINTER (2),
-			   "signal::button_release_event", gp_widget_ignore_button, GUINT_TO_POINTER (2),
-			   "signal::button_press_event", gp_widget_ignore_button, GUINT_TO_POINTER (3),
-			   "signal::button_release_event", gp_widget_ignore_button, GUINT_TO_POINTER (3),
+			   "child", arrow,
+			   "signal::clicked", gp_widget_button_popup_task_editor, NULL,
+			   "signal::event", gp_widget_ignore_button, GUINT_TO_POINTER (2),
+			   "signal::event", gp_widget_ignore_button, GUINT_TO_POINTER (3),
 			   NULL);
   gtk_tooltips_set_tip (gp_tooltips,
 			button,
@@ -686,16 +714,14 @@ gp_init_gui (void)
 			NULL);
   (arrow_at_end
    ? gtk_box_pack_end
-   : gtk_box_pack_start) (GTK_BOX (abox), button, FALSE, TRUE, 0);
+   : gtk_box_pack_start) (GTK_BOX (abox), button, !BOOL_CONFIG (show_properties_button), TRUE, 0);
   button = gtk_widget_new (GTK_TYPE_BUTTON,
-			   "visible", TRUE,
+			   "visible", BOOL_CONFIG (show_properties_button),
 			   "can_focus", FALSE,
 			   "label", "?",
-			   "signal::button_press_event", gp_widget_button_popup_config, GUINT_TO_POINTER (1),
-			   "signal::button_press_event", gp_widget_ignore_button, GUINT_TO_POINTER (2),
-			   "signal::button_release_event", gp_widget_ignore_button, GUINT_TO_POINTER (2),
-			   "signal::button_press_event", gp_widget_ignore_button, GUINT_TO_POINTER (3),
-			   "signal::button_release_event", gp_widget_ignore_button, GUINT_TO_POINTER (3),
+			   "signal::clicked", gp_config_popup, NULL,
+			   "signal::event", gp_widget_ignore_button, GUINT_TO_POINTER (2),
+			   "signal::event", gp_widget_ignore_button, GUINT_TO_POINTER (3),
 			   NULL);
   gtk_tooltips_set_tip (gp_tooltips,
 			button,
@@ -742,6 +768,17 @@ gp_about (void)
   gdk_window_raise (dialog->window);
 }
 
+static void
+gp_config_check (GtkWidget *widget)
+{
+  GtkWidget *toplevel = gtk_widget_get_toplevel (widget);
+
+  gtk_widget_set_sensitive (CONFIG_WIDGET (toplevel, area_height),
+			    !BOOL_TMP_CONFIG (abandon_area_height));
+  gtk_widget_set_sensitive (CONFIG_WIDGET (toplevel, area_width),
+			    !BOOL_TMP_CONFIG (abandon_area_width));
+}
+
 static void
 gp_config_toggled (GtkToggleButton *button,
 		   ConfigItem      *item)
@@ -754,18 +791,21 @@ static inline GtkWidget*
 gp_config_add_boolean (GtkWidget  *vbox,
 		       ConfigItem *item)
 {
-  GtkWidget *widget;
+  GtkWidget *widget, *toplevel = gtk_widget_get_toplevel (vbox);
   
   widget = gtk_widget_new (GTK_TYPE_CHECK_BUTTON,
 			   "visible", TRUE,
 			   "label", _ (item->name),
 			   "active", GPOINTER_TO_INT (item->value),
+			   "border_width", CONFIG_ITEM_BORDER,
 			   "signal::toggled", gp_config_toggled, item,
+			   "object_signal_after::toggled", gp_config_check, toplevel,
 			   NULL);
   gtk_box_pack_start (GTK_BOX (vbox), widget, FALSE, TRUE, 0);
   gtk_widget_set (GTK_BIN (widget)->child,
 		  "xalign", 0.0,
 		  NULL);
+  gtk_object_set_data (GTK_OBJECT (toplevel), item->path, widget);
   
   return widget;
 }
@@ -783,7 +823,7 @@ gp_config_add_range (GtkWidget  *vbox,
 		     ConfigItem *item)
 {
   GtkObject *adjustment;
-  GtkWidget *hbox, *label, *spinner;
+  GtkWidget *hbox, *label, *spinner, *toplevel = gtk_widget_get_toplevel (vbox);
   
   adjustment = gtk_adjustment_new (GPOINTER_TO_UINT (item->tmp_value),
 				   item->min,
@@ -791,10 +831,10 @@ gp_config_add_range (GtkWidget  *vbox,
 				   1, 1,
 				   (item->max - item->min) / 10);
   hbox = gtk_widget_new (GTK_TYPE_HBOX,
-			 "homogeneous", TRUE,
+			 "homogeneous", FALSE,
 			 "visible", TRUE,
-			 "spacing", GNOME_PAD_SMALL,
-			 "border_width", GNOME_PAD_SMALL,
+			 "spacing", CONFIG_ITEM_SPACING,
+			 "border_width", CONFIG_ITEM_BORDER,
 			 NULL);
   label = gtk_widget_new (GTK_TYPE_LABEL,
 			  "visible", TRUE,
@@ -803,14 +843,18 @@ gp_config_add_range (GtkWidget  *vbox,
 			  "parent", hbox,
 			  NULL);
   spinner = gtk_spin_button_new (GTK_ADJUSTMENT (adjustment), 0, 0);
-  gtk_widget_show (spinner);
+  gtk_widget_set (spinner,
+		  "visible", TRUE,
+		  "width", 80,
+		  NULL);
   gtk_object_set_user_data (GTK_OBJECT (adjustment), spinner);
   gtk_box_pack_end (GTK_BOX (hbox), spinner, FALSE, TRUE, 0);
-  gtk_signal_connect (adjustment,
-		      "value_changed",
-		      GTK_SIGNAL_FUNC (gp_config_value_changed),
-		      item);
+  gtk_object_set (adjustment,
+		  "signal::value_changed", gp_config_value_changed, item,
+		  "object_signal_after::value_changed", gp_config_check, toplevel,
+		  NULL);
   gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, TRUE, 0);
+  gtk_object_set_data (GTK_OBJECT (toplevel), item->path, hbox);
   
   return hbox;
 }
@@ -831,17 +875,21 @@ gp_config_add_section (GtkBox      *parent,
       box = gtk_widget_new (GTK_TYPE_VBOX,
 			    "visible", TRUE,
 			    "parent", widget,
+			    "border_width", CONFIG_IBOX_BORDER,
+			    "spacing", CONFIG_IBOX_SPACING,
 			    NULL);
     }
   else
     {
       box = gtk_widget_new (GTK_TYPE_VBOX,
 			    "visible", TRUE,
+			    "border_width", 0,
+			    "spacing", CONFIG_OBOX_SPACING,
 			    NULL);
       widget = box;
     }
 
-  gtk_box_pack_start (parent, widget, TRUE, TRUE, 0);
+  gtk_box_pack_start (parent, widget, FALSE, TRUE, 0);
   
   return box;
 }
@@ -862,7 +910,6 @@ gp_config_create_page (GSList		*item_slist,
       GSList *node = item_slist;
       
       item_slist = node->next;
-      g_slist_free_1 (node);
       
       page_name = _ (item->name);
     }
@@ -871,19 +918,18 @@ gp_config_create_page (GSList		*item_slist,
   
   page = gtk_widget_new (GTK_TYPE_VBOX,
 			 "visible", TRUE,
-			 "border_width", GNOME_PAD_SMALL,
-			 "spacing", GNOME_PAD_SMALL,
+			 "border_width", CONFIG_OBOX_BORDER,
+			 "spacing", CONFIG_OBOX_SPACING,
 			 NULL);
+  gnome_property_box_append_page (pbox, page, gtk_label_new (_ (page_name)));
   
   while (item_slist)
     {
-      GSList *node = item_slist;
-      ConfigItem *item = node->data;
+      ConfigItem *item = item_slist->data;
       
       if (!item->path)						/* page */
 	break;
-      item_slist = node->next;
-      g_slist_free_1 (node);
+      item_slist = item_slist->next;
       
       if (item->min == -2 && item->max == -2)			/* section */
 	vbox = gp_config_add_section (GTK_BOX (page), _ (item->name));
@@ -901,8 +947,6 @@ gp_config_create_page (GSList		*item_slist,
 	}
     }
   
-  gnome_property_box_append_page (pbox, page, gtk_label_new (_ (page_name)));
-  
   return item_slist;
 }
 
@@ -915,12 +959,14 @@ gp_config_popup (void)
   
   if (!dialog)
     {
-      GSList *slist = NULL;
+      GSList *fslist, *slist = NULL;
       guint i;
       
+      gp_config_reset_tmp_values ();
       for (i = 0; i < gp_n_config_items; i++)
 	slist = g_slist_prepend (slist, gp_config_items + i);
       slist = g_slist_reverse (slist);
+      fslist = slist;
       
       dialog = gnome_property_box_new ();
       gtk_widget_set (dialog,
@@ -929,17 +975,17 @@ gp_config_popup (void)
 		      "signal::apply", gp_config_apply_tmp_values, NULL,
 		      "signal::apply", gp_init_gui, NULL,
 		      "signal::destroy", gp_config_reset_tmp_values, NULL,
+		      "signal::destroy", gtk_widget_destroyed, &dialog,
 		      NULL);
       
       while (slist)
 	slist = gp_config_create_page (slist, GNOME_PROPERTY_BOX (dialog));
-      
-      gtk_widget_set (dialog,
-		      "signal::destroy", gtk_widget_destroyed, &dialog,
-		      NULL);
+      g_slist_free (fslist);
+
       gtk_quit_add_destroy (1, GTK_OBJECT (dialog));
     }
   
+  gp_config_check (dialog);
   gtk_widget_show (dialog);
   gdk_window_raise (dialog->window);
 }
