@@ -88,7 +88,6 @@ drag_data_received (GtkWidget        *widget,
 		    guint             time,
 		    gpointer          data)
 {
-	gchar *str;
 	Printer *pr = data;
 	GList *names;
 
@@ -109,7 +108,6 @@ printer_widget (Printer *pr)
 {
 	GtkWidget *fixed;
 	GtkWidget *printer;
-	int height;
 	static GtkTargetEntry drop_types [] = { 
 		{ "text/uri-list", 0, TARGET_URI_LIST }
 	};
@@ -147,6 +145,8 @@ printer_widget (Printer *pr)
 static void
 applet_set_default_back(Printer *pr)
 {
+	GtkStyle *ns;
+	g_return_if_fail (pr != NULL);
 	g_return_if_fail (pr->printer != NULL);
 
 	gtk_widget_set_rc_style(pr->printer);
