@@ -1,4 +1,11 @@
 #include <config.h>
+#include <gnome.h>
+
+#include "gstc.h"
+#include "gwmh.h"
+
+#include <gdk-pixbuf/gdk-pixbuf.h>
+
 #include "tasklist_applet.h"
 #include "unknown.xpm"
 
@@ -8,7 +15,6 @@ static void cb_about (void);
 gchar *fixup_task_label (TasklistTask *task);
 gboolean is_task_visible (TasklistTask *task);
 void draw_task (TasklistTask *task);
-void layout_tasklist (void);
 TasklistTask *find_gwmh_task (GwmhTask *gwmh_task);
 gboolean desk_notifier (gpointer func_data, GwmhDesk *desk, GwmhDeskInfoMask change_mask);
 gboolean task_notifier (gpointer func_data, GwmhTask *gwmh_task, GwmhTaskNotifyType ntype, GwmhTaskInfoMask imask);
@@ -17,14 +23,15 @@ gboolean cb_expose_event (GtkWidget *widget, GdkEventExpose *event);
 void create_applet (void);
 TasklistTask *task_get_xy (gint x, gint y);
 GList *get_visible_tasks (void);
+GdkPixbuf *create_minimized_icon (GdkPixbuf *pixbuf);
 
 GNOME_Panel_OrientType tasklist_orient; /* Tasklist orient */
 GtkWidget *handle; /* The handle box */
 GtkWidget *applet; /* The applet */
 GtkWidget *area; /* The drawing area used to display tasks */
 GList *tasks; /* The list of tasks used */
-GdkPixmap *unknown_icon; /* Unknown icon */
-GdkBitmap *unknown_mask; /* Unknown mask */
+
+TasklistIcon *unknown_icon; /* The unknown icon */
 
 gint vert_height=0; /* Vertical height, used for resizing */
 gint horz_width=0;  /* Horizontal width, used for resizing */
@@ -34,6 +41,53 @@ extern TasklistConfig Config;
 /* from gtkhandlebox.c */
 #define DRAG_HANDLE_SIZE 10
 
+/* Create a minimized version of a mini icon.
+   This code is stolen from gnome-pixmap2.
+ */
+GdkPixbuf *
+create_minimized_icon (GdkPixbuf *pixbuf)
+{
+	GdkPixbuf *minimized;
+	GdkColor color;
+	gint w, h, x, y;
+	gint32 red, green, blue;
+
+	/* Get colors */
+	color = area->style->bg[0];
+	red = color.red / 255;
+	blue = color.blue / 255;
+	green = color.green / 255;
+
+	w = gdk_pixbuf_get_width (pixbuf);
+	h = gdk_pixbuf_get_height (pixbuf);
+
+	minimized = gdk_pixbuf_new (gdk_pixbuf_get_format (pixbuf),
+				    gdk_pixbuf_get_has_alpha (pixbuf),
+				    gdk_pixbuf_get_bits_per_sample (pixbuf),
+				    w, h);
+
+	/* Fade each pixel */
+	for (y = 0; y < h; y++) {
+		for (x = 0; x < w; x++) {
+			guchar *src_pixel;
+			guchar *dest_pixel;
+			
+			src_pixel = pixbuf->art_pixbuf->pixels +
+				y * pixbuf->art_pixbuf->rowstride +
+				x * (pixbuf->art_pixbuf->has_alpha ? 4 : 3);
+
+			dest_pixel = minimized->art_pixbuf->pixels +
+				y * minimized->art_pixbuf->rowstride +
+				x * (minimized->art_pixbuf->has_alpha ? 4 : 3);
+			dest_pixel [0] = ((src_pixel [0] - red) >> 2) + red;
+			dest_pixel [1] = ((src_pixel [1] - green) >> 2) + green;
+			dest_pixel [2] = ((src_pixel [2] - blue) >> 2) + blue;
+		}
+	}
+
+	return minimized;
+}
+
 /* Shorten a label that is too long */
 gchar *
 fixup_task_label (TasklistTask *task)
@@ -171,7 +225,11 @@ draw_task (TasklistTask *task)
 {
 	gchar *tempstr;
 	gint text_height, text_width;
-	
+
+	/* For mini icons */
+	TasklistIcon *icon;
+	GdkPixbuf *pixbuf;
+
 	if (!is_task_visible (task))
 		return;
 
@@ -196,55 +254,45 @@ draw_task (TasklistTask *task)
 				 task->y + ((task->height - text_height) / 2) + text_height,
 				 tempstr);
 
-#if 0
-		gdk_draw_rectangle (area->window,
-				    area->style->black_gc,
-				    FALSE,
-				    task->x + 
-				    (Config.show_mini_icons ? 8 : 0) +
-				    ((task->width - text_width) / 2),
-				    task->y + ((task->height - text_height) / 2),
-				    text_width,
-				    text_height);
-#endif 
 		g_free (tempstr);
 	}
 
 	if (Config.show_mini_icons) {
-		if (task->pixmap) {
-			if (task->mask) {
-				gdk_gc_set_clip_mask (area->style->black_gc, 
-						      task->mask);
+		icon = task->icon;
+		
+		if (GWMH_TASK_ICONIFIED (task->gwmh_task))
+			pixbuf = icon->minimized;
+		else
+			pixbuf = icon->normal;
 
-				gdk_gc_set_clip_origin (area->style->black_gc,
-							task->x + 3,
-							task->y + (task->height - 16) / 2);
-			}
-			
-			gdk_draw_pixmap (area->window,
-					 area->style->black_gc,
-					 task->pixmap,
-					 0, 0,
-					 task->x + 3, 
-					 task->y + (task->height - 16) / 2,
-					 16, 16);
-			if (task->mask)
-				gdk_gc_set_clip_mask (area->style->black_gc, NULL);
+		gdk_gc_set_clip_mask (area->style->black_gc, icon->mask);
+		gdk_gc_set_clip_origin (area->style->black_gc,
+					task->x + 3,
+					task->y + (task->height - 16) / 2);
+		
+		if (pixbuf->art_pixbuf->has_alpha) {
+			gdk_draw_rgb_32_image (area->window,
+					       area->style->black_gc,
+					       task->x + 3, 
+					       task->y + (task->height - 16) / 2,
+					       16, 16,
+					       GDK_RGB_DITHER_NORMAL,
+					       pixbuf->art_pixbuf->pixels,
+					       pixbuf->art_pixbuf->rowstride);
 		}
 		else {
-			gdk_gc_set_clip_mask (area->style->black_gc, unknown_mask);
-			gdk_gc_set_clip_origin (area->style->black_gc,
-						task->x + 3,
-						task->y + (task->height - 16) / 2);
-			
-			gdk_draw_pixmap (area->window,
-					 area->style->black_gc,
-					 unknown_icon,
-					 0, 0,
-					 task->x + 3, task->y + (task->height - 16) / 2,
-					 16, 16);
-			gdk_gc_set_clip_mask (area->style->black_gc, NULL);
+			gdk_draw_rgb_image (area->window,
+					    area->style->black_gc,
+					    task->x + 3, 
+					    task->y + (task->height - 16) / 2,
+					    16, 16,
+					    GDK_RGB_DITHER_NORMAL,
+					    pixbuf->art_pixbuf->pixels,
+					    pixbuf->art_pixbuf->rowstride);
 		}
+
+		gdk_gc_set_clip_mask (area->style->black_gc, NULL);
+
 	}
 }
 
@@ -255,7 +303,7 @@ layout_tasklist (void)
 	gint j = 0, k = 0, num = 0, p = 0;
 	GList *temp_tasks;
 	TasklistTask *task;
-	gint extra_space;
+	/* gint extra_space; */
 	gint num_rows = 0, num_cols = 0;
 	gint curx = 0, cury = 0, curwidth = 0, curheight = 0;
 	
@@ -454,6 +502,8 @@ task_notifier (gpointer func_data, GwmhTask *gwmh_task,
 	       GwmhTaskInfoMask imask)
 {
 	TasklistTask *task;
+	GdkPixmap *pixmap = NULL;
+	GdkBitmap *mask = NULL;
 	
 	switch (ntype)
 	{
@@ -478,13 +528,31 @@ task_notifier (gpointer func_data, GwmhTask *gwmh_task,
 	case GWMH_NOTIFY_NEW:
 		task = g_malloc0 (sizeof (TasklistTask));
 		task->gwmh_task = gwmh_task;
+		task->icon = g_new (TasklistIcon, 1);
 		gwmh_task_get_mini_icon (task->gwmh_task, 
-					 &task->pixmap, &task->mask);
+					 &pixmap, &mask);
+		if (pixmap) {
+			task->icon->mask = mask;
+			task->icon->normal = gdk_pixbuf_get_from_drawable (NULL,
+									   pixmap,
+									   gtk_widget_get_colormap (area),
+									   0, 0,
+									   0, 0,
+									   16, 16);
+									   
+			task->icon->minimized = create_minimized_icon (task->icon->normal);
+		}
+		else {
 
+			task->icon->mask = unknown_icon->mask;
+			task->icon->normal = unknown_icon->normal;
+			task->icon->minimized = unknown_icon->minimized;
+		}
 		tasks = g_list_append (tasks, task);
 	        layout_tasklist ();
 		break;
 	case GWMH_NOTIFY_DESTROY:
+		g_free (find_gwmh_task (gwmh_task));
 		tasks = g_list_remove (tasks, find_gwmh_task (gwmh_task));
 		layout_tasklist ();
 		break;
@@ -507,10 +575,15 @@ cb_button_press_event (GtkWidget *widget, GdkEventButton *event)
 		return FALSE;
 
 	if (event->button == 1) {
-		gwmh_desk_set_current_area (task->gwmh_task->desktop,
-					    task->gwmh_task->harea,
-					    task->gwmh_task->varea);
-		gwmh_task_show (task->gwmh_task);
+		if (!Config.all_desks_minimized)
+			gwmh_desk_set_current_area (task->gwmh_task->desktop,
+						    task->gwmh_task->harea,
+						    task->gwmh_task->varea);
+
+		if (GWMH_TASK_ICONIFIED (task->gwmh_task) || !GWMH_TASK_FOCUSED (task->gwmh_task))
+			gwmh_task_show (task->gwmh_task);
+		else
+		  gwmh_task_iconify (task->gwmh_task);
 	}
 
 	if (event->button == 3) {
@@ -699,11 +772,17 @@ main (gint argc, gchar *argv[])
 	/* Initialize i18n */
 	bindtextdomain (PACKAGE, GNOMELOCALEDIR);
 	textdomain (PACKAGE);
-	
+
 	applet_widget_init ("tasklist_applet",
 			    VERSION,
 			    argc, argv,
 			    NULL, 0, NULL);
+
+	gdk_rgb_init ();
+
+	gtk_widget_set_default_colormap (gdk_rgb_get_cmap ());
+	gtk_widget_set_default_visual (gdk_rgb_get_visual ());
+
 	gwmh_init ();
 	gwmh_task_notifier_add (task_notifier, NULL);
 	gwmh_desk_notifier_add (desk_notifier, NULL);
@@ -712,17 +791,24 @@ main (gint argc, gchar *argv[])
 
 	read_config ();
 
-	/* FIXME: Move this elsewhere */
-	unknown_icon = gdk_pixmap_create_from_xpm_d (area->window, &unknown_mask,
-						     NULL, unknown_xpm);
+	unknown_icon = g_new (TasklistIcon, 1);
+	/* XXX: why do we need to cast unknown_xpm here,
+	   I don't get it -George*/
+	unknown_icon->normal = gdk_pixbuf_new_from_xpm_data ((const gchar **)unknown_xpm);
+	unknown_icon->mask = gdk_pixmap_new (NULL, 16, 16, 1);
+
+	gdk_pixbuf_render_threshold_alpha (unknown_icon->normal,
+					   unknown_icon->mask,
+					   0, 0, 0, 0, 16, 16, 128);
+
+	unknown_icon->minimized = create_minimized_icon (unknown_icon->normal);
 
 	applet_widget_gtk_main ();
 
 	return 0;
 }
 
-#include "gwmh.c"
-#include "gstc.c"
+
 
 
 
