@@ -2,11 +2,24 @@
 #include "authors.h"
 #include "logo.xpm"
 
+gboolean cb_quit      (GtkWidget *widget, gpointer data);
+gboolean cb_exposed   (GtkWidget *widget, GdkEventExpose *event);
+gboolean cb_configure (GtkWidget *widget, GdkEventConfigure *event);
+gboolean cb_keypress (GtkWidget *widget, GdkEventKey *event);
+gboolean cb_clicked (GtkWidget *widget, GdkEvent *event);
+gint     scroll       (gpointer data);
+
 GtkWidget *area;
 GdkPixmap *pixmap=NULL;
 GdkFont *font=NULL;
 GdkFont *italicfont=NULL;
-gint y;
+gint y, y_to_wrap_at;
+gint howmuch=0;
+GdkImlibImage *im;
+GtkWidget *canvas;
+GnomeCanvasItem *image;
+	
+
 
 /* Sparkles */
 #define NUM_FRAMES 8
@@ -189,8 +202,11 @@ sparkle_new (GnomeCanvas *canvas, double x, double y)
 static gint 
 new_sparkles_timeout(GnomeCanvas* canvas)
 {
-
 	static gint which_sparkle = 0;
+
+	if (howmuch >= 5)
+	        return TRUE;
+
 	switch (which_sparkle) {
 	case 0:
 		sparkle_new(canvas,50.0,70.0);
@@ -225,6 +241,77 @@ free_imlib_image (GtkObject *object, gpointer data)
 	gdk_imlib_destroy_image (data);
 }
 
+gboolean
+cb_clicked (GtkWidget *widget, GdkEvent *event)
+{
+	if (event->type == GDK_BUTTON_PRESS) {
+		if (howmuch >= 5) {
+			gchar *filename = gnome_datadir_file ("gnome-about/authors.dat");
+			if (filename)
+				gnome_sound_play (filename);
+
+			g_free (filename);
+		}
+
+	}
+
+	return FALSE;
+}
+
+gboolean
+cb_keypress (GtkWidget *widget, GdkEventKey *event)
+{
+	if (howmuch >= 5)
+		return FALSE;
+
+	switch (event->keyval) {
+	case GDK_e:
+	case GDK_E:
+		if (howmuch == 4) {
+			howmuch++;
+			im = gdk_imlib_create_image_from_xpm_data (magick);
+			gnome_canvas_item_set (image,
+					       "image", im,
+					       NULL);
+		}
+		else
+			howmuch = 0;
+		break;
+	case GDK_g:
+	case GDK_G:
+		if (howmuch == 0)
+			howmuch++;
+		else
+			howmuch = 0;
+		break;
+	case GDK_m:
+	case GDK_M:
+		if (howmuch == 3)
+			howmuch ++;
+		else
+			howmuch = 0;
+		break;
+	case GDK_n:
+	case GDK_N:
+		if (howmuch == 1)
+			howmuch++;
+		else
+			howmuch = 0;
+		break;
+	case GDK_o:
+	case GDK_O:
+		if (howmuch == 2)
+			howmuch++;
+		else
+			howmuch = 0;
+		break;
+	default:
+		howmuch = 0;
+	}
+	
+	return FALSE;
+}
+
 gboolean
 cb_quit (GtkWidget *widget, gpointer data)
 {
@@ -252,29 +339,35 @@ scroll (gpointer data)
 		if (authors[i].email)
 			totalwidth += 4 + gdk_string_width (italicfont, authors[i].email);
 
-		gdk_draw_string (pixmap,
-				 font,
-				 area->style->white_gc,
-				 (area->allocation.width - 
-				  totalwidth) / 2,
-				 cury,
-				 _(authors[i].name));
-		gdk_draw_string (pixmap,
-				 italicfont,
-				 area->style->white_gc,
-				 (area->allocation.width -
-				  totalwidth) / 2  + 
-				 (authors[i].email ? 4 :0) +
-				 gdk_string_width (font, authors[i].name),
-				 cury,
-				 _(authors[i].email));
-				  
+		if(cury > -font->descent &&
+		   cury < area->allocation.height + font->ascent) {
+		        gdk_draw_string (pixmap,
+					 font,
+					 area->style->white_gc,
+					 (area->allocation.width - 
+					  totalwidth) / 2,
+					 cury,
+					 _(authors[i].name));
+			gdk_draw_string (pixmap,
+					 italicfont,
+					 area->style->white_gc,
+					 (area->allocation.width -
+					  totalwidth) / 2  + 
+					 (authors[i].email ? 4 :0) +
+					 gdk_string_width (font, authors[i].name),
+					 cury,
+					 _(authors[i].email));
+		}
+
 		i++;
 		cury += font->ascent + font->descent;
 
 	}
 
 	y --;
+	if(y < y_to_wrap_at)
+	        y = area->allocation.height + font->ascent;
+
 	update_rect.x = 0;
 	update_rect.y = 0;
 	update_rect.width = area->allocation.width;
@@ -300,7 +393,7 @@ cb_exposed (GtkWidget *widget, GdkEventExpose *event)
 gboolean
 cb_configure (GtkWidget *widget, GdkEventConfigure *event)
 {
-	y = widget->allocation.height;
+	y = widget->allocation.height + font->ascent;
 
 	if (pixmap)
 		gdk_pixmap_unref (pixmap);
@@ -324,11 +417,9 @@ main (gint argc, gchar *argv[])
 	GdkPixmap *logo_pixmap;
 	GdkBitmap *logo_mask;
 	GtkWidget *frame;
-	GnomeCanvasItem *image;
 	GtkWidget *gtkpixmap;
-	GtkWidget *canvas;
-	GdkImlibImage *im;
-
+	GtkWidget *href;
+	
 	gnome_init ("gnome-about","1.0", argc, argv);
 	
 	window = gtk_window_new (GTK_WINDOW_DIALOG);
@@ -336,7 +427,8 @@ main (gint argc, gchar *argv[])
 
 	gtk_signal_connect (GTK_OBJECT (window), "delete_event",
 			    GTK_SIGNAL_FUNC (cb_quit), NULL);
-
+	gtk_signal_connect (GTK_OBJECT (window), "key_press_event",
+			    GTK_SIGNAL_FUNC (cb_keypress), NULL);
 	gtk_window_set_title (GTK_WINDOW (window), "About GNOME");
 	gtk_widget_realize (window);
 
@@ -349,6 +441,9 @@ main (gint argc, gchar *argv[])
 	if (!italicfont)
 	        italicfont = window->style->font;
 
+	y_to_wrap_at = -(sizeof(authors)/sizeof(authors[0]))*
+	                (font->ascent+font->descent);
+
 	logo_pixmap = gdk_pixmap_create_from_xpm_d (window->window, &logo_mask, NULL,
 					       logo_xpm);
 	gtkpixmap = gtk_pixmap_new (logo_pixmap, logo_mask);
@@ -370,6 +465,8 @@ main (gint argc, gchar *argv[])
 				       NULL);
 	gtk_signal_connect (GTK_OBJECT (image), "destroy",
 			    GTK_SIGNAL_FUNC (free_imlib_image), im);
+	gtk_signal_connect (GTK_OBJECT (image), "event",
+			    GTK_SIGNAL_FUNC (cb_clicked), NULL);
 	gtk_timeout_add (1300, (GtkFunction) new_sparkles_timeout, canvas);
 
 	gtk_container_border_width (GTK_CONTAINER (window), 10);
@@ -394,6 +491,26 @@ main (gint argc, gchar *argv[])
 	gtk_signal_connect (GTK_OBJECT (area), "configure_event",
 			    GTK_SIGNAL_FUNC (cb_configure), NULL);
 
+	hbox = gtk_hbox_new (TRUE, 10);
+
+	href = gnome_href_new ("http://gnotices.gnome.org/gnome-news/",
+			       _("GNOME News Site"));
+	gtk_box_pack_start (GTK_BOX (hbox), href,
+			    TRUE, FALSE, 0);
+
+	href = gnome_href_new ("http://www.gnome.org/",
+			       _("GNOME Main Site"));
+	gtk_box_pack_start (GTK_BOX (hbox), href,
+			    TRUE, FALSE, 0);
+
+	href = gnome_href_new ("http://developer.gnome.org/",
+			       _("GNOME Developers' Site"));
+	gtk_box_pack_start (GTK_BOX (hbox), href,
+			    TRUE, FALSE, 0);
+	
+	gtk_box_pack_start (GTK_BOX (vbox), hbox,
+			    FALSE, TRUE, 0);
+
 	button_box = gtk_hbutton_box_new ();
 	gtk_box_pack_end (GTK_BOX (vbox), button_box,
 			  FALSE, TRUE, 0);
