@@ -106,7 +106,6 @@ static void pageUp(GtkWidget *w, HelpWindow win);
 static void pageDown(GtkWidget *w, HelpWindow win);
 static void spaceUp(GtkWidget *w, HelpWindow win);
 static void spaceDown(GtkWidget *w, HelpWindow win);
-static void focusEnter(GtkWidget *w, HelpWindow win);
 
 static void dndDrop(GtkWidget *widget, GdkDragContext *context, gint x, gint y,
 		    GtkSelectionData *data, guint info,
@@ -149,6 +148,31 @@ GnomeUIInfo filemenu[] = {
 	GNOMEUIINFO_END
 };
 
+GnomeUIInfo viewmenu[] = {
+	{ GNOME_APP_UI_ITEM, N_("_Back"), NULL, help_backward, NULL, NULL,
+	  GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_BACK, 'B',
+	  GDK_SHIFT_MASK, NULL },
+	{ GNOME_APP_UI_ITEM, N_("_Forward"), NULL, help_forward, NULL, NULL,
+	  GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_FORWARD, 'F',
+	  GDK_SHIFT_MASK, NULL },
+	GNOMEUIINFO_SEPARATOR,
+
+	{ GNOME_APP_UI_ITEM, N_("_Reload"), NULL, reload_page, NULL, NULL,
+	  GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_REFRESH, 'R',
+	  0, NULL },
+
+	GNOMEUIINFO_SEPARATOR,
+	{ GNOME_APP_UI_ITEM, N_("_Index"), NULL, help_gotoindex, NULL, NULL,
+	  GNOME_APP_PIXMAP_NONE, NULL, 'I',
+	  0, NULL },
+
+	GNOMEUIINFO_END
+};
+
+/* Add this to gnome-convert 
++    s# (/[a-z]+/[a-z.-/]+)# <A HREF="file:$1">$1</A>#g;
+ */
+
 GnomeUIInfo helpmenu[] = {
   
     GNOMEUIINFO_HELP("help-browser"),
@@ -180,6 +204,7 @@ GnomeUIInfo settingsmenu[] = {
 GnomeUIInfo mainmenu[] = {
     GNOMEUIINFO_MENU_FILE_TREE(filemenu),
     GNOMEUIINFO_SUBTREE(N_("_Window"), windowmenu),
+    GNOMEUIINFO_SUBTREE(N_("_View"), viewmenu),
     GNOMEUIINFO_MENU_SETTINGS_TREE(settingsmenu),
     GNOMEUIINFO_MENU_HELP_TREE(helpmenu),
     GNOMEUIINFO_END
@@ -530,8 +555,6 @@ makeEntryArea(HelpWindow w)
     gnome_dock_add_item (GNOME_DOCK (GNOME_APP (w->app)->dock),
 			 GNOME_DOCK_ITEM (dock), GNOME_DOCK_TOP, 1, 0, 0, FALSE);
     w->entryBox = GTK_COMBO(entry)->entry;
-    
-    return dock;
 }
 
 
@@ -651,8 +674,7 @@ static void init_accel(HelpWindow win)
     static gint page_up_signal = 0;
     static gint page_down_signal = 0;
     static gint up_signal = 0;
-    static down_signal = 0;
-    static gint grab_focus_signal = 0;
+    static gint down_signal = 0;
 
     GtkAccelGroup *accel_group = gtk_accel_group_get_default();
 
@@ -754,8 +776,6 @@ static void dndDrop(GtkWidget *widget, GdkDragContext *context, gint x, gint y,
 		    GtkSelectionData *data, guint info,
 		    guint time, HelpWindow win)
 {
-    GList *urls;
-    
     if (data->data) {
 	GList *urls = gnome_uri_list_extract_uris ((gchar *)data->data);
 
@@ -776,7 +796,6 @@ helpWindowNew(gchar *name,
 	      HelpWindowCB config_callback)
 {
         HelpWindow w;
-	GtkWidget *entryArea;
 	GtkWidget *vbox;
 	static GtkTargetEntry target_table[] = {
 		{ "text/uri-list", 0, 0 },
@@ -1001,3 +1020,41 @@ load_image(GtkWidget *html_widget, gchar *ref)
 	docObjFree(obj);
 	return XmHTMLImageDefaultProc(html_widget, tmpnam, NULL, 0);
 }
+
+void
+statusMsg(gchar *msg)
+{
+    HelpWindow win;
+    extern GList *windowList;
+
+    if (!windowList) 
+      puts(msg);
+    else {
+      win = (HelpWindow)g_list_first(windowList)->data;
+      gnome_appbar_pop(GNOME_APPBAR(win->appBar));
+      gnome_appbar_push(GNOME_APPBAR(win->appBar), msg);
+      gnome_appbar_refresh(GNOME_APPBAR(win->appBar));
+      while (gtk_events_pending ())
+        gtk_main_iteration ();
+      gdk_flush();
+    }
+}
+
+/* Change first parameter of gnome_appbar_new for this to work */
+void
+statusPerc(gfloat percent)
+{
+    HelpWindow win;
+    extern GList *windowList;
+
+    if (!windowList) 
+      ;
+    else {
+      win = (HelpWindow)g_list_first(windowList)->data;
+      gnome_appbar_set_progress(GNOME_APPBAR(win->appBar), percent);
+      gnome_appbar_refresh(GNOME_APPBAR(win->appBar));
+      while (gtk_events_pending ())
+        gtk_main_iteration ();
+      gdk_flush();
+    }
+}
