@@ -55,7 +55,7 @@ server_applet_change_orient(CustomAppletServant *servant,
 
 static void
 server_applet_change_size(CustomAppletServant *servant,
-			  GNOME_Panel_SizeType size,
+			  CORBA_short size,
 			  CORBA_Environment *ev);
 
 static void
@@ -64,17 +64,27 @@ server_applet_do_callback(CustomAppletServant *servant,
 			  CORBA_Environment *ev);
 
 static void
-server_applet_session_save(CustomAppletServant *servant,
+server_applet_save_session(CustomAppletServant *servant,
 			   CORBA_char * cfgpath,
 			   CORBA_char * globcfgpath,
 			   CORBA_unsigned_long cookie,
 			   CORBA_Environment *ev);
 
+static CORBA_boolean
+server_applet_session_save(CustomAppletServant *servant,
+			   CORBA_char * cfgpath,
+			   CORBA_char * globcfgpath,
+			   CORBA_Environment *ev);
+
 static void
 server_applet_back_change(CustomAppletServant *servant,
 			  GNOME_Panel_BackInfoType *backing,
 			  CORBA_Environment *ev);
 
+static void
+server_applet_draw(CustomAppletServant *servant,
+		   CORBA_Environment *ev);
+
 static void
 server_applet_set_tooltips_state(CustomAppletServant *servant,
 				 CORBA_boolean enabled,
@@ -101,13 +111,15 @@ server_applet_thaw_changes(CustomAppletServant *servant,
 static POA_GNOME_Applet__epv applet_epv = {
   NULL,
   (gpointer)&server_applet_change_orient,
-  (gpointer)&server_applet_change_size,
   (gpointer)&server_applet_do_callback,
   (gpointer)&server_applet_session_save,
   (gpointer)&server_applet_back_change,
   (gpointer)&server_applet_set_tooltips_state,
-  (gpointer)&server_applet_change_position,
   (gpointer)&server_applet__get_goad_id,
+  (gpointer)&server_applet_draw,
+  (gpointer)&server_applet_save_session,
+  (gpointer)&server_applet_change_size,
+  (gpointer)&server_applet_change_position,
   (gpointer)&server_applet_freeze_changes,
   (gpointer)&server_applet_thaw_changes
 };
@@ -137,6 +149,10 @@ typedef void (*PositionSignal) (GtkObject * object,
 				int y,
 				gpointer data);
 
+typedef void (*DeprecSizeSignal) (GtkObject * object,
+				  GNOME_Panel_SizeType type,
+				  gpointer data);
+
 static int applet_count = 0;
 
 static int die_on_last = FALSE;
@@ -171,9 +187,11 @@ applet_widget_get_type ()
 
 enum {
 	CHANGE_ORIENT_SIGNAL,
+	CHANGE_PIXEL_SIZE_SIGNAL,
 	CHANGE_SIZE_SIGNAL,
 	SAVE_SESSION_SIGNAL,
 	BACK_CHANGE_SIGNAL,
+	DO_DRAW_SIGNAL,
 	TOOLTIP_STATE_SIGNAL,
 	CHANGE_POSITION_SIGNAL,
 	LAST_SIGNAL
@@ -235,6 +253,24 @@ marshal_signal_position (GtkObject * object,
 		  func_data);
 }
 
+static void
+marshal_signal_deprec_size (GtkObject * object,
+			    GtkSignalFunc func,
+			    gpointer func_data,
+			    GtkArg * args)
+{
+	DeprecSizeSignal rfunc;
+
+	g_warning("change_size signal is deprecated, use the new "
+		  "change_pixel_size signal");
+
+	rfunc = (DeprecSizeSignal) func;
+
+	(*rfunc) (object, GTK_VALUE_ENUM (args[0]),
+		  func_data);
+}
+
+
 static void
 applet_widget_class_init (AppletWidgetClass *class)
 {
@@ -254,13 +290,23 @@ applet_widget_class_init (AppletWidgetClass *class)
 			       GTK_TYPE_NONE,
 			       1,
 			       GTK_TYPE_ENUM);
+	applet_widget_signals[CHANGE_PIXEL_SIZE_SIGNAL] =
+		gtk_signal_new("change_pixel_size",
+			       GTK_RUN_FIRST,
+			       object_class->type,
+			       GTK_SIGNAL_OFFSET(AppletWidgetClass,
+			       			 change_pixel_size),
+			       gtk_marshal_NONE__INT,
+			       GTK_TYPE_NONE,
+			       1,
+			       GTK_TYPE_INT);
 	applet_widget_signals[CHANGE_SIZE_SIGNAL] =
 		gtk_signal_new("change_size",
 			       GTK_RUN_FIRST,
 			       object_class->type,
 			       GTK_SIGNAL_OFFSET(AppletWidgetClass,
 			       			 change_size),
-			       gtk_marshal_NONE__ENUM,
+			       marshal_signal_deprec_size,
 			       GTK_TYPE_NONE,
 			       1,
 			       GTK_TYPE_ENUM);
@@ -287,6 +333,15 @@ applet_widget_class_init (AppletWidgetClass *class)
 			       GTK_TYPE_ENUM,
 			       GTK_TYPE_POINTER,
 			       GTK_TYPE_POINTER);
+	applet_widget_signals[DO_DRAW_SIGNAL] =
+		gtk_signal_new("do_draw",
+			       GTK_RUN_LAST,
+			       object_class->type,
+			       GTK_SIGNAL_OFFSET(AppletWidgetClass,
+			       			 do_draw),
+			       gtk_signal_default_marshaller,
+			       GTK_TYPE_NONE,
+			       0);
 	applet_widget_signals[TOOLTIP_STATE_SIGNAL] =
 		gtk_signal_new("tooltip_state",
 			       GTK_RUN_LAST,
@@ -316,6 +371,7 @@ applet_widget_class_init (AppletWidgetClass *class)
 	class->change_size = NULL;
 	class->save_session = NULL;
 	class->back_change = NULL;
+	class->do_draw = NULL;
 	class->tooltip_state = NULL;
 	class->change_position = NULL;
 }
@@ -327,6 +383,8 @@ wapplet_widget_init (AppletWidget *applet)
 	g_return_if_fail(IS_APPLET_WIDGET(applet));
 	
 	applet->corbadat = NULL;
+	applet->orient = ORIENT_UP;
+	applet->size = PIXEL_SIZE_STANDARD;
 }
 
 static void
@@ -429,8 +487,8 @@ applet_widget_sync_config(AppletWidget *applet)
 	CORBA_exception_free(&ev);
 }
 
-static char*
-make_sane_name(char *name)
+static const char*
+make_sane_name(const char *name)
 {
 	if(!name)
 		return NULL;
@@ -442,7 +500,7 @@ make_sane_name(char *name)
 }
 
 static CallbackInfo *
-get_callback_info(GtkWidget *applet, char *name)
+get_callback_info(GtkWidget *applet, const char *name)
 {
 	GSList *list;
 	CallbackInfo *info;
@@ -462,9 +520,9 @@ get_callback_info(GtkWidget *applet, char *name)
 /*adds a callback to the callback hash*/
 static void
 gnome_panel_applet_register_callback(GtkWidget *applet,
-				     char *name,
-				     char *stock_item,
-				     char *menutext,
+				     const char *name,
+				     const char *stock_item,
+				     const char *menutext,
 				     AppletCallbackFunc func,
 				     gpointer data)
 {
@@ -498,8 +556,8 @@ gnome_panel_applet_register_callback(GtkWidget *applet,
 
 void
 applet_widget_register_callback(AppletWidget *applet,
-				char *name,
-				char *menutext,
+				const char *name,
+				const char *menutext,
 				AppletCallbackFunc func,
 				gpointer data)
 {
@@ -515,9 +573,9 @@ applet_widget_register_callback(AppletWidget *applet,
 
 void
 applet_widget_register_stock_callback(AppletWidget *applet,
-				      char *name,
-				      char *stock_type,
-				      char *menutext,
+				      const char *name,
+				      const char *stock_type,
+				      const char *menutext,
 				      AppletCallbackFunc func,
 				      gpointer data)
 {
@@ -534,7 +592,7 @@ applet_widget_register_stock_callback(AppletWidget *applet,
 
 void
 applet_widget_unregister_callback(AppletWidget *applet,
-				  char *name)
+				  const char *name)
 {
 	GSList *ltmp;
 	CallbackInfo *cbi = NULL;
@@ -565,9 +623,9 @@ applet_widget_unregister_callback(AppletWidget *applet,
 
 static void
 gnome_panel_applet_register_callback_dir(GtkWidget *applet,
-					 char *name,
-					 char *stock_item,
-					 char *menutext)
+					 const char *name,
+					 const char *stock_item,
+					 const char *menutext)
 {
 	char *n;
 	CORBA_Environment ev;
@@ -591,8 +649,8 @@ gnome_panel_applet_register_callback_dir(GtkWidget *applet,
 
 void
 applet_widget_register_callback_dir(AppletWidget *applet,
-				    char *name,
-				    char *menutext)
+				    const char *name,
+				    const char *menutext)
 {
 	g_return_if_fail(applet != NULL);
 	g_return_if_fail(IS_APPLET_WIDGET(applet));
@@ -604,9 +662,9 @@ applet_widget_register_callback_dir(AppletWidget *applet,
 }
 void
 applet_widget_register_stock_callback_dir(AppletWidget *applet,
-					  char *name,
-					  char *stock_type,
-					  char *menutext)
+					  const char *name,
+					  const char *stock_type,
+					  const char *menutext)
 {
 	g_return_if_fail(applet != NULL);
 	g_return_if_fail(IS_APPLET_WIDGET(applet));
@@ -619,7 +677,7 @@ applet_widget_register_stock_callback_dir(AppletWidget *applet,
 }
 
 void
-applet_widget_unregister_callback_dir(AppletWidget *applet, char *name)
+applet_widget_unregister_callback_dir(AppletWidget *applet, const char *name)
 {
 	CORBA_Environment ev;
 	char *n;
@@ -644,7 +702,7 @@ applet_widget_unregister_callback_dir(AppletWidget *applet, char *name)
 
 void
 applet_widget_callback_set_sensitive(AppletWidget *applet,
-				     char *name,
+				     const char *name,
 				     int sensitive)
 {
 	CORBA_Environment ev;
@@ -718,6 +776,13 @@ gnome_panel_applet_corba_init(AppletWidget *applet, const char *goad_id)
 	/*{  static volatile int stop_here = 0;
 		while(stop_here);}*/
 
+	/* we need to do this as 1.0 panel will crap out otherwise, it NEEDS
+	   to know the applet as it's doing orient change signals during
+	   this */
+	applet_servant->appwidget = applet;
+	/* this is just for consistency with the above */
+	applet_servant->goad_id = g_strdup(goad_id);
+
 	applet_servant->pspot = GNOME_Panel_add_applet(panel_client,
 						       applet_obj,
 						       (char *)goad_id,
@@ -737,6 +802,23 @@ gnome_panel_applet_corba_init(AppletWidget *applet, const char *goad_id)
 		applet->globcfgpath = NULL;
 	CORBA_free(globcfg);
 
+	/* initialize orient and size correctly */
+	applet->orient =
+		GNOME_PanelSpot__get_parent_orient(applet_servant->pspot,&ev);
+	if(ev._major) {
+		g_warning("CORBA Exception, can't get orient");
+		/* just recycle the exception */
+		CORBA_exception_free(&ev);
+		CORBA_exception_init(&ev);
+		applet->size = ORIENT_UP;
+	}
+	applet->size =
+		GNOME_PanelSpot__get_parent_size(applet_servant->pspot,&ev);
+	if(ev._major) {
+		g_warning("CORBA Exception, can't get size");
+		applet->size = PIXEL_SIZE_STANDARD;
+	}
+
 	CORBA_exception_free(&ev);
 
 	return applet_servant;
@@ -747,29 +829,31 @@ GtkWidget *
 applet_widget_new(const char *goad_id)
 {
 	AppletWidget *applet;
+
+	applet = APPLET_WIDGET (gtk_type_new (applet_widget_get_type()));
+	applet_widget_construct(applet, goad_id);
+
+	return GTK_WIDGET(applet);
+}
+
+void
+applet_widget_construct(AppletWidget* applet, const char *goad_id)
+{
 	CustomAppletServant *corbadat;
 	
-	g_return_val_if_fail(goad_id != NULL,NULL);
-
-	applet = APPLET_WIDGET (gtk_type_new (applet_widget_get_type ()));
+	g_return_if_fail(goad_id != NULL);
 
 	CD(applet) = corbadat = gnome_panel_applet_corba_init(applet,goad_id);
 
-	g_return_val_if_fail(corbadat!=NULL,NULL);
-
-	corbadat->appwidget = applet;
+	g_return_if_fail(corbadat!=NULL);
 
 	gtk_plug_construct(GTK_PLUG(applet), corbadat->winid);
 	
-	corbadat->goad_id = g_strdup(goad_id);
-
 	gtk_signal_connect(GTK_OBJECT(applet),"destroy",
 			   GTK_SIGNAL_FUNC(applet_widget_destroy),
 			   NULL);
 
 	applet_count++;
-
-	return GTK_WIDGET(applet);
 }
 
 int
@@ -865,7 +949,7 @@ applet_widget_bind_events(AppletWidget *applet, GtkWidget *widget)
 void
 applet_widget_set_widget_tooltip(AppletWidget *applet,
 				 GtkWidget *widget,
-				 char *text)
+				 const char *text)
 {
 	g_return_if_fail(applet != NULL);
 	g_return_if_fail(IS_APPLET_WIDGET(applet));
@@ -879,7 +963,7 @@ applet_widget_set_widget_tooltip(AppletWidget *applet,
 }
 
 void
-applet_widget_set_tooltip(AppletWidget *applet, char *text)
+applet_widget_set_tooltip(AppletWidget *applet, const char *text)
 {
 	CORBA_Environment ev;
 	g_return_if_fail(applet != NULL);
@@ -904,8 +988,30 @@ applet_widget_get_panel_orient(AppletWidget *applet)
 GNOME_Panel_SizeType
 applet_widget_get_panel_size(AppletWidget *applet)
 {
-	g_return_val_if_fail(applet != NULL,SIZE_STANDARD);
-	g_return_val_if_fail(IS_APPLET_WIDGET(applet), SIZE_STANDARD);
+	g_return_val_if_fail(applet != NULL,GNOME_Panel_SIZE_STANDARD);
+	g_return_val_if_fail(IS_APPLET_WIDGET(applet), GNOME_Panel_SIZE_STANDARD);
+	
+	g_warning("applet_widget_get_panel_size is deprecated, use the new pixel_size methods");
+
+	switch(applet->size) {
+	case 24:
+		return GNOME_Panel_SIZE_TINY;
+	default:
+	case 48:
+		return GNOME_Panel_SIZE_STANDARD;
+	case 64:
+		return GNOME_Panel_SIZE_LARGE;
+	case 80:
+		return GNOME_Panel_SIZE_HUGE;
+	}
+}
+
+/* Get the size the applet should use */
+int
+applet_widget_get_panel_pixel_size(AppletWidget *applet)
+{
+	g_return_val_if_fail(applet != NULL,PIXEL_SIZE_STANDARD);
+	g_return_val_if_fail(IS_APPLET_WIDGET(applet), PIXEL_SIZE_STANDARD);
 
 	return applet->size;
 }
@@ -939,6 +1045,84 @@ applet_widget_send_position(AppletWidget *applet, int enable)
 	CORBA_exception_free(&ev);
 }
 
+/* sets if the do_draw signal is sent*/
+void
+applet_widget_send_draw(AppletWidget *applet, int enable)
+{
+	CORBA_Environment ev;
+	g_return_if_fail(applet != NULL);
+	g_return_if_fail(IS_APPLET_WIDGET(applet));
+	
+	CORBA_exception_init(&ev);
+	GNOME_PanelSpot__set_send_draw(CD(applet)->pspot, enable, &ev);
+	CORBA_exception_free(&ev);
+}
+
+/* gets the rgb background, useful in conjunction with the draw signal */
+void
+applet_widget_get_rgb_bg(AppletWidget *applet, guchar **rgb,
+			 int *w, int *h, int *rowstride)
+{
+	CORBA_Environment ev;
+	GNOME_Panel_RgbImage *image;
+
+	g_return_if_fail(applet!=NULL);
+	g_return_if_fail(IS_APPLET_WIDGET(applet));
+	g_return_if_fail(rgb!=NULL);
+	g_return_if_fail(w!=NULL);
+	g_return_if_fail(h!=NULL);
+	g_return_if_fail(rowstride!=NULL);
+	
+	CORBA_exception_init(&ev);
+	image = GNOME_PanelSpot__get_rgb_background(CD(applet)->pspot, &ev);
+	if(ev._major) {
+		g_warning("CORBA Exception");
+		CORBA_exception_free(&ev);
+		return;
+	}
+	CORBA_exception_free(&ev);
+
+	*w = image->width;
+	*h = image->width;
+	if(!image->color_only)
+		*rowstride = image->rowstride;
+	else
+		*rowstride = (*w)*3;
+
+	if(image->data._buffer) {
+		*rgb = g_new(guchar, (*h)*(*rowstride));
+		if(!image->color_only) {
+			int size = (*h)*(*rowstride);
+			if(image->data._length<size)
+				size = image->data._length;
+			memcpy(*rgb,image->data._buffer,
+			       sizeof(guchar)*size);
+		} else {
+			int i;
+			int r;
+			int g;
+			int b;
+			guchar *p;
+
+			r = *(image->data._buffer);
+			g = *(image->data._buffer+1);
+			b = *(image->data._buffer+2);
+
+			p = *rgb;
+			for(i=0;i<(*w)*(*h);i++) {
+				*(p++) = r;
+				*(p++) = g;
+				*(p++) = b;
+			}
+		}
+	} else {
+		/* this will make a black background */
+		*rgb = g_new0(guchar, (*h)*(*rowstride));
+	}
+	
+	CORBA_free(image);
+}
+
 int	
 applet_widget_init(const char *app_id,
 		   const char *app_version,
@@ -1012,17 +1196,40 @@ server_applet_change_orient(CustomAppletServant *servant,
 
 static void
 server_applet_change_size(CustomAppletServant *servant,
-			  GNOME_Panel_SizeType size,
+			  CORBA_short size,
 			  CORBA_Environment *ev)
 {
 	servant->appwidget->size = size;
 	if(servant->appwidget->frozen_level>0) {
 		servant->appwidget->frozen_got_size = TRUE;
-		servant->appwidget->frozen_size = (GNOME_Panel_SizeType)size;
+		servant->appwidget->frozen_size = size;
 	} else {
+		switch(size) {
+		case 24:
+			gtk_signal_emit(GTK_OBJECT(servant->appwidget),
+					applet_widget_signals[CHANGE_SIZE_SIGNAL],
+					GNOME_Panel_SIZE_TINY);
+			break;
+		default:
+		case 48:
+			gtk_signal_emit(GTK_OBJECT(servant->appwidget),
+					applet_widget_signals[CHANGE_SIZE_SIGNAL],
+					GNOME_Panel_SIZE_STANDARD);
+			break;
+		case 64:
+			gtk_signal_emit(GTK_OBJECT(servant->appwidget),
+					applet_widget_signals[CHANGE_SIZE_SIGNAL],
+					GNOME_Panel_SIZE_LARGE);
+			break;
+		case 80:
+			gtk_signal_emit(GTK_OBJECT(servant->appwidget),
+					applet_widget_signals[CHANGE_SIZE_SIGNAL],
+					GNOME_Panel_SIZE_HUGE);
+			break;
+		}
 		gtk_signal_emit(GTK_OBJECT(servant->appwidget),
-				applet_widget_signals[CHANGE_SIZE_SIGNAL],
-				(GNOME_Panel_SizeType)size);
+				applet_widget_signals[CHANGE_PIXEL_SIZE_SIGNAL],
+				size);
 	}
 }
 
@@ -1045,8 +1252,9 @@ server_applet_do_callback(CustomAppletServant *servant,
 	}
 }
 
+/* this is the new session saving call and the one which should be used */
 static void
-server_applet_session_save(CustomAppletServant *servant,
+server_applet_save_session(CustomAppletServant *servant,
 			   CORBA_char * cfgpath,
 			   CORBA_char * globcfgpath,
 			   CORBA_unsigned_long cookie,
@@ -1072,6 +1280,30 @@ server_applet_session_save(CustomAppletServant *servant,
 					  !return_val, cookie, ev);
 }
 
+/* this is here just that if an applet uses the new lib with the old
+   panel it will still work */
+static CORBA_boolean
+server_applet_session_save(CustomAppletServant *servant,
+			   CORBA_char * cfgpath,
+			   CORBA_char * globcfgpath,
+			   CORBA_Environment *ev)
+{
+	AppletWidget *applet;
+	char *cfg = g_strdup(cfgpath);
+	char *globcfg = g_strdup(globcfgpath);
+
+	int return_val = FALSE;
+
+	applet = servant->appwidget;
+	gtk_signal_emit(GTK_OBJECT(applet),
+			applet_widget_signals[SAVE_SESSION_SIGNAL],
+			cfg, globcfg, &return_val);
+	g_free(cfg);
+	g_free(globcfg);
+
+	return !return_val;
+}
+
 static void
 server_applet_back_change(CustomAppletServant *servant,
 			  GNOME_Panel_BackInfoType *backing,
@@ -1111,6 +1343,14 @@ server_applet_back_change(CustomAppletServant *servant,
 	}
 }
 
+static void
+server_applet_draw(CustomAppletServant *servant,
+		   CORBA_Environment *ev)
+{
+	gtk_signal_emit(GTK_OBJECT(servant->appwidget),
+			applet_widget_signals[DO_DRAW_SIGNAL]);
+}
+
 static void
 server_applet_set_tooltips_state(CustomAppletServant *servant,
 				 CORBA_boolean enabled,
@@ -1179,8 +1419,31 @@ server_applet_thaw_changes(CustomAppletServant *servant,
 	}
 	if(servant->appwidget->frozen_got_size) {
 		servant->appwidget->frozen_got_size = FALSE;
+		switch(servant->appwidget->frozen_size) {
+		case 24:
+			gtk_signal_emit(GTK_OBJECT(servant->appwidget),
+					applet_widget_signals[CHANGE_SIZE_SIGNAL],
+					GNOME_Panel_SIZE_TINY);
+			break;
+		default:
+		case 48:
+			gtk_signal_emit(GTK_OBJECT(servant->appwidget),
+					applet_widget_signals[CHANGE_SIZE_SIGNAL],
+					GNOME_Panel_SIZE_STANDARD);
+			break;
+		case 64:
+			gtk_signal_emit(GTK_OBJECT(servant->appwidget),
+					applet_widget_signals[CHANGE_SIZE_SIGNAL],
+					GNOME_Panel_SIZE_LARGE);
+			break;
+		case 80:
+			gtk_signal_emit(GTK_OBJECT(servant->appwidget),
+					applet_widget_signals[CHANGE_SIZE_SIGNAL],
+					GNOME_Panel_SIZE_HUGE);
+			break;
+		}
 		gtk_signal_emit(GTK_OBJECT(servant->appwidget),
-				applet_widget_signals[CHANGE_SIZE_SIGNAL],
+				applet_widget_signals[CHANGE_PIXEL_SIZE_SIGNAL],
 				servant->appwidget->frozen_size);
 	}
 	if(servant->appwidget->frozen_got_back) {
