@@ -76,6 +76,8 @@ struct _BasePWidget
 	gboolean                keep_in_screen;
 
 	guint32                 autohide_complete;
+
+	gboolean		compliant_wm;
 };
 
 struct _BasePWidgetClass
@@ -157,31 +159,35 @@ GtkWidget*	basep_widget_construct		(BasePWidget *basep,
 						 int packed,
 						 int reverse_arrows,
 						 PanelOrientation orient,
-						 PanelSizeType sz,
+						 int sz,
 						 BasePMode mode,
 						 BasePState state,
 						 int hidebuttons_enabled,
 						 int hidebutton_pixmaps_enabled,
 						 PanelBackType back_type,
 						 char *back_pixmap,
-						 int fit_pixmap_bg,
+						 gboolean fit_pixmap_bg,
+						 gboolean strech_pixmap_bg,
+						 gboolean rotate_pixmap_bg,
 						 GdkColor *back_color);
 
 /* changing parameters */
 void		basep_widget_change_params	(BasePWidget *basep,
 						 PanelOrientation orient,
-						 PanelSizeType sz,
+						 int sz,
 						 BasePMode mode,
 						 BasePState state,
 						 int hidebuttons_enabled,
 						 int hidebutton_pixmaps_enabled,
 						 PanelBackType back_type,
 						 char *pixmap_name,
-						 int fit_pixmap_bg,
+						 gboolean fit_pixmap_bg,
+						 gboolean strech_pixmap_bg,
+						 gboolean rotate_pixmap_bg,
 						 GdkColor *back_color);
 
-gboolean       basep_widget_convert_to         (BasePWidget *basep,
-						 PanelType type);
+/*gboolean       basep_widget_convert_to         (BasePWidget *basep,
+						 PanelType type);*/
 
 void            _basep_widget_enable_buttons (BasePWidget *basep,
 					      gboolean enabled);
@@ -257,6 +263,9 @@ void            basep_widget_set_state         (BasePWidget *basep,
 
 void            basep_update_frame             (BasePWidget *basep);
 
+/* redo the widget->window based on new compliancy setting */
+void		basep_widget_redo_window	(BasePWidget *basep);
+
 #define GNOME_PANEL_HINTS (WIN_HINTS_SKIP_FOCUS|WIN_HINTS_SKIP_WINLIST|WIN_HINTS_SKIP_TASKBAR)
 
 END_GNOME_DECLS
