@@ -37,16 +37,15 @@ static int  button_widget_leave_notify	(GtkWidget         *widget,
 static void button_widget_pressed	(ButtonWidget *button);
 static void button_widget_unpressed	(ButtonWidget *button);
 
-typedef void (*VoidSignal) (GtkObject * object,
-			    gpointer data);
-
 /*list of all the button widgets*/
 static GList *buttons=NULL;
 
 /*the tiles go here*/
 struct {
 	GdkPixbuf *tiles_up[LAST_TILE];
+	GdkPixbuf *tiles_up_lc[LAST_TILE];
 	GdkPixbuf *tiles_down[LAST_TILE];
+	GdkPixbuf *tiles_down_lc[LAST_TILE];
 } tiles = {{NULL}}; /*ansi C trick to make it all 0*/
 
 static int tile_border[LAST_TILE]={0,0,0,0};
@@ -92,20 +91,6 @@ enum {
 
 static int button_widget_signals[LAST_SIGNAL] = {0};
 
-static void
-marshal_signal_void (GtkObject * object,
-		     GtkSignalFunc func,
-		     gpointer func_data,
-		     GtkArg * args)
-{
-	VoidSignal rfunc;
-
-	rfunc = (VoidSignal) func;
-
-	(*rfunc) (object, func_data);
-}
-
-
 static void
 button_widget_class_init (ButtonWidgetClass *class)
 {
@@ -120,7 +105,7 @@ button_widget_class_init (ButtonWidgetClass *class)
 			       object_class->type,
 			       GTK_SIGNAL_OFFSET(ButtonWidgetClass,
 			       			 clicked),
-			       marshal_signal_void,
+			       gtk_signal_default_marshaller,
 			       GTK_TYPE_NONE,
 			       0);
 	button_widget_signals[PRESSED_SIGNAL] =
@@ -129,7 +114,7 @@ button_widget_class_init (ButtonWidgetClass *class)
 			       object_class->type,
 			       GTK_SIGNAL_OFFSET(ButtonWidgetClass,
 			       			 pressed),
-			       marshal_signal_void,
+			       gtk_signal_default_marshaller,
 			       GTK_TYPE_NONE,
 			       0);
 	button_widget_signals[UNPRESSED_SIGNAL] =
@@ -138,7 +123,7 @@ button_widget_class_init (ButtonWidgetClass *class)
 			       object_class->type,
 			       GTK_SIGNAL_OFFSET(ButtonWidgetClass,
 			       			 unpressed),
-			       marshal_signal_void,
+			       gtk_signal_default_marshaller,
 			       GTK_TYPE_NONE,
 			       0);
 	gtk_object_class_add_signals(object_class,button_widget_signals,
@@ -238,25 +223,25 @@ button_widget_unrealize (GtkWidget *widget)
 static void
 button_widget_map (GtkWidget *widget)
 {
-  g_return_if_fail (widget != NULL);
-  g_return_if_fail (IS_BUTTON_WIDGET (widget));
+	g_return_if_fail (widget != NULL);
+	g_return_if_fail (IS_BUTTON_WIDGET (widget));
 
-  if (GTK_WIDGET_REALIZED (widget) && !GTK_WIDGET_MAPPED (widget))
-      gdk_window_show (BUTTON_WIDGET (widget)->event_window);
+	if (GTK_WIDGET_REALIZED (widget) && !GTK_WIDGET_MAPPED (widget))
+		gdk_window_show (BUTTON_WIDGET (widget)->event_window);
 
-  GTK_WIDGET_CLASS (parent_class)->map (widget);
+	GTK_WIDGET_CLASS (parent_class)->map (widget);
 }
 
 static void
 button_widget_unmap (GtkWidget *widget)
 {
-  g_return_if_fail (widget != NULL);
-  g_return_if_fail (IS_BUTTON_WIDGET (widget));
+	g_return_if_fail (widget != NULL);
+	g_return_if_fail (IS_BUTTON_WIDGET (widget));
 
-  if (GTK_WIDGET_MAPPED (widget))
-      gdk_window_hide (BUTTON_WIDGET (widget)->event_window);
+	if (GTK_WIDGET_MAPPED (widget))
+		gdk_window_hide (BUTTON_WIDGET (widget)->event_window);
 
-  GTK_WIDGET_CLASS (parent_class)->unmap (widget);
+	GTK_WIDGET_CLASS (parent_class)->unmap (widget);
 }
 
 static void
@@ -267,6 +252,9 @@ button_widget_destroy(GtkWidget *w, gpointer data)
 	if(button->pixbuf)
 		gdk_pixbuf_unref(button->pixbuf);
 	button->pixbuf = NULL;
+	if(button->pixbuf_lc)
+		gdk_pixbuf_unref(button->pixbuf_lc);
+	button->pixbuf_lc = NULL;
 	if(button->cache)
 		gdk_pixmap_unref(button->cache);
 	button->cache = NULL;
@@ -297,6 +285,54 @@ loadup_file(char *file)
 	return pb;
 }
 
+/*#define INTENSITY(r, g, b) ((r) * 0.30 + (g) * 0.59 + (b) * 0.11)*/
+#define INTENSITY(r, g, b) (((r)*77 + (g)*150 + (b)*28)>>8)
+
+/* saturation is 0-255, darken is 0-255 */
+static void
+do_saturate_darken (GdkPixbuf *dest, GdkPixbuf *src,
+		    int saturation, int darken)
+{
+	gint i, j;
+	gint width, height, has_alpha, srcrowstride, destrowstride;
+	guchar *target_pixels;
+	guchar *original_pixels;
+	guchar *pixsrc;
+	guchar *pixdest;
+	guchar intensity;
+	guchar alpha;
+	guchar negalpha;
+	guchar r,g,b;
+
+	has_alpha = gdk_pixbuf_get_has_alpha (src);
+	width = gdk_pixbuf_get_width (src);
+	height = gdk_pixbuf_get_height (src);
+	srcrowstride = gdk_pixbuf_get_rowstride (src);
+	destrowstride = gdk_pixbuf_get_rowstride (dest);
+	target_pixels = gdk_pixbuf_get_pixels (dest);
+	original_pixels = gdk_pixbuf_get_pixels (src);
+
+	for (i = 0; i < height; i++) {
+		pixdest = target_pixels + i*destrowstride;
+		pixsrc = original_pixels + i*srcrowstride;
+		for (j = 0; j < width; j++) {
+			r = *(pixsrc++);
+			g = *(pixsrc++);
+			b = *(pixsrc++);
+			intensity = INTENSITY(r,g,b);
+			negalpha = ((255 - saturation)*darken)>>8;
+			alpha = (saturation*darken)>>8;
+			*(pixdest++) = (negalpha * intensity + alpha * r) >> 8;
+			*(pixdest++) = (negalpha * intensity + alpha * g) >> 8;
+			*(pixdest++) = (negalpha * intensity + alpha * b) >> 8;
+			if (has_alpha)
+				*(pixdest++) = *(pixsrc++);
+		}
+	}
+}
+#undef INTENSITY
+
+
 
 #define SCALE(x) (((x)*size)/48.0)
 
@@ -352,7 +388,7 @@ button_widget_draw(ButtonWidget *button, guchar *rgb, int rowstride)
 	GtkWidget *widget = GTK_WIDGET(button);
 	GtkWidget *pwidget;
 	PanelWidget *panel = PANEL_WIDGET(widget->parent);
-	int size = panel_widget_get_pixel_size(panel);
+	int size = panel->sz;
 	/*offset for pressed buttons*/
 	int off = button->in_button&&button->pressed?SCALE(tile_depth[button->tile]):0;
 	/*border to not draw when drawing a tile*/
@@ -364,46 +400,67 @@ button_widget_draw(ButtonWidget *button, guchar *rgb, int rowstride)
 	
 	if(tiles_enabled[button->tile]) {
 		if(button->pressed && button->in_button) {
-			if(tiles.tiles_down[button->tile]) {
+			GdkPixbuf *pb;
+			if(global_config.saturate_when_over && !button->in_button) {
+				pb = tiles.tiles_down_lc[button->tile];
+			} else {
+				pb = tiles.tiles_down[button->tile];
+			}
+			if(pb) {
 				double affine[6];
 				make_scale_affine(affine,
-						  tiles.tiles_down[button->tile]->art_pixbuf->width,
-						  tiles.tiles_down[button->tile]->art_pixbuf->height,
-						  size);
+						  pb->art_pixbuf->width,
+						  pb->art_pixbuf->height,
+						  size, NULL, NULL);
 				art_rgb_pixbuf_affine (rgb, 0, 0, size, size, rowstride,
-						       tiles.tiles_down[button->tile]->art_pixbuf,
+						       pb->art_pixbuf,
 						       affine, ART_FILTER_NEAREST, NULL);
 			}
 		} else if (!global_config.tile_when_over || button->in_button) {
-			if(tiles.tiles_up[button->tile]) {
+			GdkPixbuf *pb;
+			if(global_config.saturate_when_over && !button->in_button) {
+				pb = tiles.tiles_up_lc[button->tile];
+			} else {
+				pb = tiles.tiles_up[button->tile];
+			}
+			if(pb) {
 				double affine[6];
 				make_scale_affine(affine,
-						  tiles.tiles_up[button->tile]->art_pixbuf->width,
-						  tiles.tiles_up[button->tile]->art_pixbuf->height,
-						  size);
+						  pb->art_pixbuf->width,
+						  pb->art_pixbuf->height,
+						  size, NULL, NULL);
 				art_rgb_pixbuf_affine (rgb, 0, 0, size, size, rowstride,
-						       tiles.tiles_up[button->tile]->art_pixbuf,
+						       pb->art_pixbuf,
 						       affine, ART_FILTER_NEAREST, NULL);
 			}
 		}
 	}
 
 	if (pixmaps_enabled[button->tile]) {
-		if(button->pixbuf) {
+		GdkPixbuf *pb;
+		if(global_config.saturate_when_over && !button->in_button) {
+			pb = button->pixbuf_lc;
+		} else {
+			pb = button->pixbuf;
+		}
+		if(pb) {
 			double affine[6];
 			double transl[6];
+			int w,h;
 			make_scale_affine(affine,
-					  button->pixbuf->art_pixbuf->width,
-					  button->pixbuf->art_pixbuf->height,
-					  size);
-			art_affine_translate(transl,-border+off,-border+off);
+					  pb->art_pixbuf->width,
+					  pb->art_pixbuf->height,
+					  size, &w, &h);
+			art_affine_translate(transl,
+					     -border+off + (size-w)/2,
+					     -border+off + (size-h)/2);
 			art_affine_multiply(affine,affine,transl);
 
 			art_rgb_pixbuf_affine((rgb+border*rowstride+border*3),
 					      0, 0,
 					      size-2*border, size-2*border,
 					      rowstride,
-					      button->pixbuf->art_pixbuf,
+					      pb->art_pixbuf,
 					      affine, ART_FILTER_NEAREST, NULL);
 		}
 	}
@@ -494,8 +551,7 @@ static void
 button_widget_size_request(GtkWidget *widget, GtkRequisition *requisition)
 {
 	PanelWidget *panel = PANEL_WIDGET(widget->parent);
-	requisition->width = requisition->height = 
-		panel_widget_get_pixel_size(panel);
+	requisition->width = requisition->height = panel->sz;
 }
 static void
 button_widget_size_allocate(GtkWidget *widget, GtkAllocation *allocation)
@@ -524,6 +580,7 @@ button_widget_init (ButtonWidget *button)
 	GTK_WIDGET_SET_FLAGS (button, GTK_NO_WINDOW);
 
 	button->pixbuf = NULL;
+	button->pixbuf_lc = NULL;
 	
 	button->tile = 0;
 	button->arrow = 0;
@@ -609,7 +666,8 @@ button_widget_enter_notify (GtkWidget *widget, GdkEventCrossing *event)
 	    (event->detail != GDK_NOTIFY_INFERIOR)) {
 		ButtonWidget *button = BUTTON_WIDGET (widget);
 		button->in_button = TRUE;
-		if(global_config.tile_when_over) {
+		if(global_config.tile_when_over ||
+		   global_config.saturate_when_over) {
 			if(button->cache)
 				gdk_pixmap_unref(button->cache);
 			button->cache = NULL;
@@ -639,7 +697,8 @@ button_widget_leave_notify (GtkWidget *widget, GdkEventCrossing *event)
 	    (event->detail != GDK_NOTIFY_INFERIOR) &&
 	    (!button->ignore_leave)) {
 		button->in_button = FALSE;
-		if(global_config.tile_when_over) {
+		if(global_config.tile_when_over ||
+		   global_config.saturate_when_over) {
 			if(button->cache)
 				gdk_pixmap_unref(button->cache);
 			button->cache = NULL;
@@ -699,6 +758,24 @@ button_widget_up(ButtonWidget *button)
 				button_widget_signals[UNPRESSED_SIGNAL]);
 }
 
+static GdkPixbuf *
+make_lc_pixbuf(GdkPixbuf *pb)
+{
+	GdkPixbuf *new;
+	if(!pb)
+		return NULL;
+
+	new = gdk_pixbuf_new(gdk_pixbuf_get_format(pb),
+			     gdk_pixbuf_get_has_alpha(pb),
+			     gdk_pixbuf_get_bits_per_sample(pb),
+			     gdk_pixbuf_get_width(pb),
+			     gdk_pixbuf_get_height(pb));
+	do_saturate_darken (new, pb, (int)(0.85*255), (int)(0.85*255));
+
+	return new;
+}
+
+
 
 GtkWidget*
 button_widget_new(char *filename,
@@ -713,6 +790,7 @@ button_widget_new(char *filename,
 	button = BUTTON_WIDGET (gtk_type_new (button_widget_get_type ()));
 	
 	button->pixbuf = loadup_file(filename);
+	button->pixbuf_lc = make_lc_pixbuf(button->pixbuf);
 	button->filename = g_strdup(filename);
 	button->size = size;
 	button->tile = tile;
@@ -729,9 +807,10 @@ int
 button_widget_set_pixmap(ButtonWidget *button, char *pixmap, int size)
 {
 	if(size<0)
-		size = panel_widget_get_pixel_size(PANEL_WIDGET(GTK_WIDGET(button)->parent));
+		size = PANEL_WIDGET(GTK_WIDGET(button)->parent)->sz;
 	
 	button->pixbuf = loadup_file(pixmap);
+	button->pixbuf_lc = make_lc_pixbuf(button->pixbuf);
 	g_free(button->filename);
 	button->filename = g_strdup(pixmap);
 	button->size = size;
@@ -789,11 +868,17 @@ button_widget_load_tile(int tile, char *tile_up, char *tile_down,
 
 	if(tiles.tiles_up[tile])
 		gdk_pixbuf_unref(tiles.tiles_up[tile]);
+	if(tiles.tiles_up_lc[tile])
+		gdk_pixbuf_unref(tiles.tiles_up_lc[tile]);
 	tiles.tiles_up[tile] = loadup_file(tile_up);
+	tiles.tiles_up_lc[tile] = make_lc_pixbuf(tiles.tiles_up[tile]);
 
 	if(tiles.tiles_down[tile])
 		gdk_pixbuf_unref(tiles.tiles_down[tile]);
+	if(tiles.tiles_down_lc[tile])
+		gdk_pixbuf_unref(tiles.tiles_down_lc[tile]);
 	tiles.tiles_down[tile] = loadup_file(tile_down);
+	tiles.tiles_down_lc[tile] = make_lc_pixbuf(tiles.tiles_down[tile]);
 
 	tile_border[tile] = border;
 	tile_depth[tile] = depth;
