@@ -8,6 +8,7 @@
 
 #include "drawer-widget.h"
 #include "border-widget.h"
+#include "floating-widget.h"
 #include "panel_config_global.h"
 
 extern GlobalConfig global_config;
@@ -24,6 +25,9 @@ static void drawer_pos_get_hide_pos (BasePWidget *basep,
 				     PanelOrientType hide_orient,
 				     gint16 *x, gint16 *y,
 				     guint16 w, guint16 h);
+static void drawer_pos_get_hide_size (BasePWidget *basep, 
+				      PanelOrientType hide_orient,
+				      guint16 *w, guint16 *h);
 
 static void drawer_pos_get_pos(BasePWidget *basep,
 			       gint16 *x, gint16 *y,
@@ -78,6 +82,7 @@ drawer_pos_class_init (DrawerPosClass *klass)
 	pos_class->get_size = NULL; /* the default is ok */
 	pos_class->get_hide_orient = drawer_pos_get_hide_orient;
 	pos_class->get_hide_pos = drawer_pos_get_hide_pos;
+	pos_class->get_hide_size = drawer_pos_get_hide_size;
 	pos_class->get_pos = drawer_pos_get_pos;
 	pos_class->get_menu_pos = drawer_pos_get_menu_pos;
 
@@ -123,56 +128,40 @@ drawer_pos_set_hidebuttons (BasePWidget *basep)
 	}
 }
 
-static PanelData *
-get_lowest_level_master_pd(PanelWidget *panel)
+static void
+get_widget_pos(GtkWidget *widget, int *x, int *y)
 {
-	GtkObject *parent;
-	PanelData *pd;
-
-	while(panel->master_widget)
-		panel = PANEL_WIDGET(panel->master_widget->parent);
-	parent = GTK_OBJECT(panel->panel_parent);
-	g_return_val_if_fail(parent!=NULL,NULL);
-	
-	pd = gtk_object_get_user_data(parent);
-	g_return_val_if_fail(pd!=NULL,NULL);
-	
-	return pd;
+       if(!widget->window) {
+               *x = *y = 0;
+               return;
+       }
+       gdk_window_get_origin (widget->window, x, y);
+       if(GTK_WIDGET_NO_WINDOW(widget)) {
+               *x += widget->allocation.x;
+               *y += widget->allocation.y;
+       }
 }
 
 static PanelOrientType
 drawer_pos_get_applet_orient (BasePWidget *basep)
 {
 	PanelWidget *panel = PANEL_WIDGET (basep->panel);
-	PanelData *tpd = get_lowest_level_master_pd (panel);
-	PanelOrientType orient = ORIENT_UP;
 	PanelOrientation porient = panel->orient;
+	int x,y;
 
-	/* unfortunately we must do this */
-	if (IS_BORDER_WIDGET (tpd->panel)) {
-		switch (BORDER_POS (BASEP_WIDGET (tpd->panel)->pos)->edge) {
-		case BORDER_TOP:
-			orient = (porient == PANEL_VERTICAL)
-				? ORIENT_RIGHT : ORIENT_DOWN;
-			break;
-		case BORDER_BOTTOM:
-		case BORDER_LEFT:
-			orient = (porient == PANEL_VERTICAL)
-				? ORIENT_RIGHT : ORIENT_UP;
-			break;
-		case BORDER_RIGHT:
-			orient = (porient == PANEL_VERTICAL)
-				? ORIENT_LEFT : ORIENT_UP;
-			break;
-		}
-	} else if (IS_DRAWER_WIDGET (tpd->panel)) {
-		orient = (porient == PANEL_VERTICAL)
-			? ORIENT_RIGHT : ORIENT_UP;
+	get_widget_pos(GTK_WIDGET(basep),&x,&y);
+
+	if(porient == PANEL_VERTICAL) {
+		if(x > (gdk_screen_width()/2))
+			return ORIENT_LEFT;
+		else
+			return ORIENT_RIGHT;
 	} else {
-		g_warning (_("Don't know about base panel type: %d\n"), tpd->type);
+		if(y > (gdk_screen_height()/2))
+			return ORIENT_UP;
+		else
+			return ORIENT_DOWN;
 	}
-	
-	return orient;
 }
 
 static PanelOrientType
@@ -338,6 +327,24 @@ drawer_pos_get_hide_pos (BasePWidget *basep,
 	}
 }
 
+static void
+drawer_pos_get_hide_size (BasePWidget *basep, 
+			  PanelOrientType hide_orient,
+			  guint16 *w, guint16 *h)
+{
+	switch (hide_orient) {
+	case ORIENT_UP:
+	case ORIENT_DOWN:
+		*h = 1;
+		break;
+	case ORIENT_RIGHT:
+	case ORIENT_LEFT:
+		*w = 1;
+		break;
+	}
+}
+
+
 static void
 drawer_pos_pre_convert_hook (BasePWidget *basep)
 {
@@ -349,12 +356,14 @@ void drawer_widget_change_params (DrawerWidget *drawer,
 				   PanelOrientType orient,
 				  BasePMode mode,
 				  BasePState state,
-				  PanelSizeType sz,
+				  int sz,
 				  int hidebuttons_enabled,
 				  int hidebutton_pixmap_enabled,
 				  PanelBackType back_type,
 				  char *back_pixmap,
-				  int fit_pixmap_bg,
+				  gboolean fit_pixmap_bg,
+				  gboolean strech_pixmap_bg,
+				  gboolean rotate_pixmap_bg,
 				  GdkColor *back_color)
 {
 	PanelOrientation porient;
@@ -375,6 +384,18 @@ void drawer_widget_change_params (DrawerWidget *drawer,
 	if (PANEL_WIDGET (BASEP_WIDGET (drawer)->panel)->orient != porient)
 		BASEP_WIDGET (drawer)->request_cube = TRUE;
 
+	if (state != BASEP_WIDGET (drawer)->state ||
+	    pos->orient != orient) {
+		if (state == BASEP_HIDDEN_LEFT &&
+		    (orient == ORIENT_LEFT ||
+		     orient == ORIENT_UP))
+			state = BASEP_HIDDEN_RIGHT;
+		else if (state == BASEP_HIDDEN_RIGHT &&
+			 (orient == ORIENT_RIGHT ||
+			  orient == ORIENT_DOWN))
+			 state = BASEP_HIDDEN_LEFT;
+	}
+
 	if (pos->orient != orient) {
 		pos->orient = orient;
 #if 0
@@ -389,7 +410,9 @@ void drawer_widget_change_params (DrawerWidget *drawer,
 				    hidebuttons_enabled,
 				    hidebutton_pixmap_enabled,
 				    back_type, back_pixmap,
-				    fit_pixmap_bg, back_color);
+				    fit_pixmap_bg, strech_pixmap_bg,
+				    rotate_pixmap_bg,
+				    back_color);
 				    
 }
 
@@ -410,6 +433,8 @@ drawer_widget_change_orient (DrawerWidget *drawer,
 					     panel->back_type,
 					     panel->back_pixmap,
 					     panel->fit_pixmap_bg,
+					     panel->strech_pixmap_bg,
+					     panel->rotate_pixmap_bg,
 					     &panel->back_color);
 	}
 }
@@ -418,12 +443,14 @@ GtkWidget *
 drawer_widget_new (PanelOrientType orient,
 		   BasePMode mode,
 		   BasePState state,
-		   PanelSizeType sz,
+		   int sz,
 		   int hidebuttons_enabled,
 		   int hidebutton_pixmap_enabled,
 		   PanelBackType back_type,
 		   char *back_pixmap,
-		   int fit_pixmap_bg,
+		   gboolean fit_pixmap_bg,
+		   gboolean strech_pixmap_bg,
+		   gboolean rotate_pixmap_bg,
 		   GdkColor *back_color)
 {
 	DrawerWidget *drawer;
@@ -454,6 +481,8 @@ drawer_widget_new (PanelOrientType orient,
 				back_type,
 				back_pixmap,
 				fit_pixmap_bg,
+				strech_pixmap_bg,
+				rotate_pixmap_bg,
 				back_color);
 
 	return GTK_WIDGET (drawer);
