@@ -18,7 +18,7 @@
 
 #include "panel-include.h"
 
-#define DRAWER_PROPERTIES "drawer_properties"
+#include "xstuff.h"
 
 extern GSList *applets;
 extern GSList *applets_last;
@@ -75,8 +75,7 @@ static void
 properties_close_callback(GtkWidget *widget, gpointer data)
 {
 	Drawer *drawer = data;
-	gtk_object_set_data(GTK_OBJECT(drawer->button),
-			    DRAWER_PROPERTIES,NULL);
+	drawer->properties = NULL;
 }
 
 static void
@@ -168,8 +167,7 @@ add_drawer_properties_page(PerPanelConfig *ppc, Drawer *drawer)
 			   GTK_SIGNAL_FUNC(properties_apply_callback),
 			   drawer);
 
-	gtk_object_set_data(GTK_OBJECT(drawer->button),
-			    DRAWER_PROPERTIES,dialog);
+	drawer->properties = dialog;
 }
 
 static void
@@ -198,8 +196,8 @@ static void
 destroy_drawer(GtkWidget *widget, gpointer data)
 {
 	Drawer *drawer = data;
-	GtkWidget *prop_dialog = gtk_object_get_data(GTK_OBJECT(drawer->button),
-						     DRAWER_PROPERTIES);
+	GtkWidget *prop_dialog = drawer->properties;
+
 	if(prop_dialog)
 		gtk_widget_destroy(prop_dialog);
 	g_free(drawer);
@@ -211,7 +209,7 @@ enter_notify_drawer(GtkWidget *widget, GdkEventCrossing *event, gpointer data)
 	Drawer *drawer = data;
 	BasePWidget *basep = BASEP_WIDGET (drawer->drawer);
 
-	if (!gnome_win_hints_wm_exists())
+	if (!xstuff_is_compliant_wm())
 		gdk_window_raise(drawer->drawer->window);
 
 	if (basep->state == BASEP_MOVING)
@@ -253,7 +251,9 @@ create_drawer_applet(GtkWidget * drawer_panel, char *tooltip, char *pixmap,
 {
 	Drawer *drawer;
 	
-	drawer = g_new(Drawer,1);
+	drawer = g_new0(Drawer,1);
+	
+	drawer->properties = NULL;
 
 	if(!tooltip ||
 	   !*tooltip)
@@ -299,7 +299,7 @@ create_empty_drawer_applet(char *tooltip, char *pixmap,
 					  SIZE_STANDARD,
 					  TRUE, TRUE,
 					  PANEL_BACK_NONE, NULL,
-					  TRUE, NULL);
+					  TRUE, FALSE, TRUE, NULL);
 	return create_drawer_applet(dw, tooltip,pixmap,orient);
 }
 
@@ -332,29 +332,40 @@ drawer_setup(Drawer *drawer)
 	gtk_widget_show(drawer->drawer);
 }
 
-static void
-drawer_move_foreach(GtkWidget *w, gpointer user_data)
-{
-	AppletInfo *info = gtk_object_get_data(GTK_OBJECT(w), "applet_info");
-	
-	if(info->type == APPLET_DRAWER) {
-		Drawer *drawer = info->data;
-		gtk_widget_queue_resize(drawer->drawer);
-	}
-}
-
 static void
 button_size_alloc(GtkWidget *widget, GtkAllocation *alloc, Drawer *drawer)
 {
 	if(!GTK_WIDGET_REALIZED(widget))
 		return;
-	drawer_move_foreach(widget,NULL);
+
+	gtk_widget_queue_resize(drawer->drawer);
+
 	gtk_object_set_data(GTK_OBJECT(widget),"allocated",GINT_TO_POINTER(1));
 }
 
+static void
+drawer_orient_change_foreach(GtkWidget *w, gpointer data)
+{
+	AppletInfo *info = gtk_object_get_data(GTK_OBJECT(w), "applet_info");
+	PanelWidget *panel = data;
+	
+	orientation_change(info,panel);
+}
+
+static void
+drawer_size_alloc(BasePWidget *basep, GtkAllocation *alloc, gpointer data)
+{
+	if(!GTK_WIDGET_REALIZED(basep))
+		return;
+
+	gtk_container_foreach(GTK_CONTAINER(basep->panel),
+			      drawer_orient_change_foreach,
+			      basep->panel);
+}
+
 void
 load_drawer_applet(int mypanel, char *pixmap, char *tooltip,
-		   PanelWidget *panel, int pos)
+		   PanelWidget *panel, int pos, gboolean exactpos)
 {
 	Drawer *drawer;
 	PanelOrientType orient = get_applet_orient(panel);
@@ -370,8 +381,8 @@ load_drawer_applet(int mypanel, char *pixmap, char *tooltip,
 		g_return_if_fail(li != NULL);
 		dr_pd = li->data;
 
-		drawer=create_drawer_applet(dr_pd->panel, tooltip,
-					    pixmap, orient);
+		drawer = create_drawer_applet(dr_pd->panel, tooltip,
+					      pixmap, orient);
 
 		drawer_widget_change_orient(DRAWER_WIDGET(dr_pd->panel),
 					    orient);
@@ -380,7 +391,22 @@ load_drawer_applet(int mypanel, char *pixmap, char *tooltip,
 	if(!drawer)
 		return;
 
-	register_toy(drawer->button,drawer, panel, pos, APPLET_DRAWER);
+	{
+		GtkWidget *dw = drawer->drawer;
+
+		if(!register_toy(drawer->button,drawer,
+				 panel, pos, exactpos, APPLET_DRAWER)) {
+			/* by this time drawer has been freed as register_toy
+			   has destroyed drawer->button */
+			gtk_widget_destroy(dw);
+			return;
+		}
+	}
+
+	gtk_signal_connect_after(GTK_OBJECT(drawer->drawer),
+				 "size_allocate",
+				 GTK_SIGNAL_FUNC(drawer_size_alloc),
+				 NULL);
 
 	gtk_signal_connect_after(GTK_OBJECT(drawer->button),
 				 "size_allocate",
