@@ -13,10 +13,11 @@ typedef struct {
 	char *tooltip;
 	GtkWidget *button;
 	GtkWidget *drawer;
+	GtkWidget *properties;
 } Drawer;
 
 void load_drawer_applet(int mypanel, char *pixmap, char *tooltip,
-			PanelWidget *panel, int pos);
+			PanelWidget *panel, int pos, gboolean exactpos);
 
 void set_drawer_applet_orient(Drawer *drawer, PanelOrientType orient);
 
