@@ -125,6 +125,18 @@ s_panelspot_set_send_position(POA_GNOME_PanelSpot *servant,
 			      CORBA_boolean,
 			      CORBA_Environment *ev);
 
+static CORBA_boolean
+s_panelspot_get_send_draw(POA_GNOME_PanelSpot *servant,
+			  CORBA_Environment *ev);
+static void
+s_panelspot_set_send_draw(POA_GNOME_PanelSpot *servant,
+			  CORBA_boolean,
+			  CORBA_Environment *ev);
+
+static GNOME_Panel_RgbImage *
+s_panelspot_get_rgb_background(POA_GNOME_PanelSpot *servant,
+			       CORBA_Environment *ev);
+
 static void
 s_panelspot_register_us(POA_GNOME_PanelSpot *servant,
 		     CORBA_Environment *ev);
@@ -221,6 +233,9 @@ static POA_GNOME_PanelSpot__epv panelspot_epv = {
   (gpointer)&s_panelspot_get_free_space,
   (gpointer)&s_panelspot_get_send_position,
   (gpointer)&s_panelspot_set_send_position,
+  (gpointer)&s_panelspot_get_send_draw,
+  (gpointer)&s_panelspot_set_send_draw,
+  (gpointer)&s_panelspot_get_rgb_background,
   (gpointer)&s_panelspot_register_us,
   (gpointer)&s_panelspot_unregister_us,
   (gpointer)&s_panelspot_abort_load,
@@ -274,15 +289,17 @@ extern_clean(Extern *ext)
 	PortableServer_POA_deactivate_object(thepoa, id, &ev);
 	CORBA_free (id);
 	POA_GNOME_PanelSpot__fini((PortableServer_Servant) ext, &ev);
+	
+	if(ext->send_draw_timeout) {
+		gtk_timeout_remove(ext->send_draw_timeout);
+		ext->send_draw_timeout = 0;
+	}
 
 	g_free(ext);
 
 	CORBA_exception_free(&ev);
 }
 
-
-
-
 static void
 extern_socket_destroy(GtkWidget *w, gpointer data)
 {
@@ -312,7 +329,6 @@ send_position_change(Extern *ext)
 		
 		CORBA_Environment ev;
 		CORBA_exception_init(&ev);
-		g_warning ("Crazy function!\n");
 		/*go the the toplevel panel widget*/
 		for(;;) {
 			if(!GTK_WIDGET_NO_WINDOW(wid)) {
@@ -324,9 +340,7 @@ send_position_change(Extern *ext)
 			else
 				break;
 		}
-		GNOME_Applet_change_position(ext->applet,
-					     x,y,
-					     &ev);
+		GNOME_Applet_change_position(ext->applet, x, y, &ev);
 		if(ev._major)
 			panel_clean_applet(ext->info);
 		CORBA_exception_free(&ev);
@@ -343,10 +357,59 @@ ebox_size_allocate(GtkWidget *applet, GtkAllocation *alloc, Extern *ext)
 static void
 socket_size_allocate(GtkWidget *applet, GtkAllocation *alloc)
 {
-  if (applet->requisition.width > 0 && applet->requisition.height > 0 &&
-      (applet->allocation.width > applet->requisition.width ||
-       applet->allocation.height > applet->requisition.height))
-    gtk_widget_queue_resize(applet->parent);
+	GtkWidget *plug_widget;
+	GtkSocket *socket = GTK_SOCKET(applet);
+
+	/* perhaps an already unnecessary hack to avoid some missed
+	   reallocations */
+	if (applet->requisition.width > 0 && applet->requisition.height > 0 &&
+	    (applet->allocation.width > applet->requisition.width ||
+	     applet->allocation.height > applet->requisition.height))
+		gtk_widget_queue_resize(applet->parent);
+
+	if (!socket->same_app ||
+	    !socket->plug_window)
+		return;
+    
+	gdk_window_get_user_data (socket->plug_window,
+				  (gpointer *)&plug_widget);
+	if (plug_widget) {
+		GtkAllocation child_allocation;
+		child_allocation.x = child_allocation.y = 0;
+		child_allocation.width = alloc->width;
+		child_allocation.height = alloc->height;
+		gtk_widget_size_allocate (plug_widget, &child_allocation);
+	}
+	gdk_window_move_resize (socket->plug_window,
+				0, 0, alloc->width, alloc->height);
+	if (socket->need_map) {
+		gdk_window_show (socket->plug_window);
+		socket->need_map = FALSE;
+	}
+	gdk_flush ();
+}
+
+static void
+socket_size_request(GtkSocket *socket, GtkRequisition *req)
+{
+	GtkWidget *plug_widget;
+
+	if (!socket->same_app ||
+	    !socket->plug_window)
+		return;
+
+	gdk_window_get_user_data (socket->plug_window,
+				  (gpointer *)&plug_widget);
+	if (plug_widget) {
+		GtkRequisition child_requisition;
+		gtk_widget_size_request (plug_widget, &child_requisition);
+
+		socket->request_width = child_requisition.width;
+		socket->request_height = child_requisition.height;
+	}
+
+	req->width = MAX (socket->request_width, 1);
+	req->height = MAX (socket->request_height, 1);
 }
 
 /*note that type should be APPLET_EXTERN_RESERVED or APPLET_EXTERN_PENDING
@@ -368,32 +431,38 @@ reserve_applet_spot (Extern *ext, PanelWidget *panel, int pos,
 
 	socket = gtk_socket_new();
 
+	gtk_signal_connect_after(GTK_OBJECT(socket),"size_request",
+				 GTK_SIGNAL_FUNC(socket_size_request),
+				 NULL);
 	gtk_signal_connect_after(GTK_OBJECT(socket),"size_allocate",
 				 GTK_SIGNAL_FUNC(socket_size_allocate),
 				 NULL);
 
+	/* here for debugging purposes */
 	/*gtk_signal_connect_after(GTK_OBJECT(socket),"size_allocate",
 				 GTK_SIGNAL_FUNC(sal),NULL);*/
 
 
 	g_return_val_if_fail(socket!=NULL,0);
 
-	gtk_container_add(GTK_CONTAINER(ext->ebox),socket);
+	gtk_container_add(GTK_CONTAINER(ext->ebox), socket);
 
-	gtk_signal_connect(GTK_OBJECT(socket),"destroy",
-			   GTK_SIGNAL_FUNC(extern_socket_destroy),
-			   ext);
-
-	gtk_widget_show_all (ext->ebox);
+	gtk_widget_show_all(ext->ebox);
 	
 	/*we save the obj in the id field of the appletinfo and the 
 	  path in the path field */
 	ext->info = NULL;
-	if(!register_toy(ext->ebox,ext,panel,pos,type)) {
-		g_warning("Couldn't add applet");
+	if(!register_toy(ext->ebox, ext, panel, pos, ext->exactpos, type)) {
+		/* the ebox is destroyed in register_toy */
+		ext->ebox = NULL;
+		g_warning(_("Couldn't add applet"));
 		return 0;
 	}
 	ext->info = applets_last->data;
+
+	gtk_signal_connect(GTK_OBJECT(socket),"destroy",
+			   GTK_SIGNAL_FUNC(extern_socket_destroy),
+			   ext);
 	
 	if(!GTK_WIDGET_REALIZED(socket))
 		gtk_widget_realize(socket);
@@ -402,7 +471,8 @@ reserve_applet_spot (Extern *ext, PanelWidget *panel, int pos,
 }
 
 void
-load_extern_applet(char *goad_id, char *cfgpath, PanelWidget *panel, int pos, int queue)
+load_extern_applet(char *goad_id, char *cfgpath, PanelWidget *panel,
+		   int pos, gboolean exactpos, gboolean queue)
 {
 	Extern *ext;
 	POA_GNOME_PanelSpot *panelspot_servant;
@@ -416,7 +486,14 @@ load_extern_applet(char *goad_id, char *cfgpath, PanelWidget *panel, int pos, in
 	
 	ext = g_new0(Extern,1);
 	ext->started = FALSE;
+	ext->exactpos = exactpos;
 	ext->send_position = FALSE;
+	ext->send_draw = FALSE;
+	ext->orient = -1;
+
+	ext->send_draw_timeout = 0;
+	ext->send_draw_queued = FALSE;
+
 
 	panelspot_servant = (POA_GNOME_PanelSpot *)ext;
 	panelspot_servant->_private = NULL;
@@ -433,8 +510,8 @@ load_extern_applet(char *goad_id, char *cfgpath, PanelWidget *panel, int pos, in
 	ext->cfg = cfgpath;
 
 	if(reserve_applet_spot (ext, panel, pos, APPLET_EXTERN_PENDING)==0) {
-		g_warning("Whoops! for some reason we can't add "
-			  "to the panel");
+		g_warning(_("Whoops! for some reason we can't add "
+			    "to the panel"));
 		extern_clean(ext);
 		return;
 	}
@@ -535,7 +612,10 @@ s_panel_add_applet_full(POA_GNOME_Panel *servant,
 	  have already reserved a spot for it*/
 	ext = g_new0(Extern,1);
 	ext->started = FALSE;
+	ext->exactpos = FALSE;
 	ext->send_position = FALSE;
+	ext->send_draw = FALSE;
+	ext->orient = -1;
 
 	panelspot_servant = (POA_GNOME_PanelSpot *)ext;
 	panelspot_servant->_private = NULL;
@@ -771,6 +851,74 @@ s_panelspot_set_send_position(POA_GNOME_PanelSpot *servant,
 	ext->send_position = enable?TRUE:FALSE;
 }
 
+static CORBA_boolean
+s_panelspot_get_send_draw(POA_GNOME_PanelSpot *servant,
+			  CORBA_Environment *ev)
+{
+	Extern *ext = (Extern *)servant;
+
+	g_assert(ext);
+	
+	return ext->send_draw;
+}
+
+static void
+s_panelspot_set_send_draw(POA_GNOME_PanelSpot *servant,
+			  CORBA_boolean enable,
+			  CORBA_Environment *ev)
+{
+	Extern *ext = (Extern *)servant;
+
+	g_assert(ext);
+	
+	ext->send_draw = enable?TRUE:FALSE;
+}
+
+static GNOME_Panel_RgbImage *
+s_panelspot_get_rgb_background(POA_GNOME_PanelSpot *servant,
+			       CORBA_Environment *ev)
+{
+	Extern *ext = (Extern *)servant;
+	PanelWidget *panel;
+	GNOME_Panel_RgbImage *image;
+	int w, h, rowstride;
+	guchar *rgb;
+	int r,g,b;
+
+	g_assert(ext);
+	g_assert(ext->info);
+
+	panel = PANEL_WIDGET(ext->info->widget->parent);
+
+	panel_widget_get_applet_rgb_bg(panel, ext->ebox,
+				       &rgb,&w,&h,&rowstride,
+				       TRUE,&r,&g,&b);
+
+	image = GNOME_Panel_RgbImage__alloc();
+	image->width = w;
+	image->height = h;
+		
+	/* if we got an rgb */
+	if(rgb) {
+		image->data._buffer = CORBA_sequence_CORBA_octet_allocbuf(h*rowstride);
+		image->data._length = image->data._maximum = h*rowstride;
+		memcpy(image->data._buffer,rgb,sizeof(guchar)*h*rowstride);
+		image->rowstride = rowstride;
+		image->color_only = FALSE;
+	} else { /* we must have gotten a color */
+		image->data._buffer = CORBA_sequence_CORBA_octet_allocbuf(3);
+		image->data._length = image->data._maximum = 3;
+		*(image->data._buffer) = r;
+		*(image->data._buffer+1) = g;
+		*(image->data._buffer+2) = b;
+		image->rowstride = 0;
+		image->color_only = TRUE;
+	}
+	CORBA_sequence_set_release(&image->data, TRUE);
+
+	return image;
+}
+
 static void
 s_panelspot_register_us(POA_GNOME_PanelSpot *servant,
 			CORBA_Environment *ev)
@@ -1122,3 +1270,42 @@ panel_corba_gtk_init(CORBA_ORB panel_orb)
 
   return status;
 }
+
+static void
+send_draw(Extern *ext)
+{
+	CORBA_Environment ev;
+
+	CORBA_exception_init(&ev);
+	GNOME_Applet_draw(ext->applet, &ev);
+	if(ev._major)
+		panel_clean_applet(ext->info);
+	CORBA_exception_free(&ev);
+}
+
+static int
+send_draw_timeout(gpointer data)
+{
+	Extern *ext = data;
+	if(ext->send_draw_queued == TRUE) {
+		ext->send_draw_queued = FALSE;
+		send_draw(ext);
+		return TRUE;
+	}
+	ext->send_draw_timeout = 0;
+	return FALSE;
+}
+
+void
+extern_send_draw(Extern *ext)
+{
+	if(!ext || !ext->applet || !ext->send_draw)
+		return;
+	if(!ext->send_draw_timeout) {
+		ext->send_draw_queued = FALSE;
+		send_draw(ext);
+		ext->send_draw_timeout =
+			gtk_timeout_add(1000,send_draw_timeout,ext);
+	} else 
+		ext->send_draw_queued = TRUE;
+}
