@@ -145,7 +145,7 @@ floating_pos_get_applet_orient (BasePWidget *basep)
 			gdk_screen_height () / 2)
 			? ORIENT_DOWN : ORIENT_UP;
 	else
-		return (FLOATING_POS (basep->pos)->x > 
+		return (FLOATING_POS (basep->pos)->x < 
 			gdk_screen_width () /2)
 			? ORIENT_RIGHT : ORIENT_LEFT;
 }
@@ -153,7 +153,7 @@ floating_pos_get_applet_orient (BasePWidget *basep)
 static PanelOrientType
 floating_pos_get_hide_orient (BasePWidget *basep)
 {
-	FloatingPos *pos = FLOATING_POS (basep->pos);
+	/*FloatingPos *pos = FLOATING_POS (basep->pos);*/
 	PanelWidget *panel = PANEL_WIDGET (basep->panel);
 
 	switch (basep->state) {
@@ -230,7 +230,7 @@ floating_pos_set_pos (BasePWidget *basep,
 				break;
 			}
 		}
-		newx = CLAMP (x < minx ? x2 : x2 - w, 0, gdk_screen_width () - w);
+		newx = CLAMP (x < minx ? x2 : x2 - w2, 0, gdk_screen_width () - w);
 	}
 
 	if (y < miny || y > maxy) {
@@ -370,12 +370,14 @@ floating_widget_change_params (FloatingWidget *floating,
 			       PanelOrientation orient,
 			       BasePMode mode,
 			       BasePState state,
-			       PanelSizeType sz,
+			       int sz,
 			       int hidebuttons_enabled,
 			       int hidebutton_pixmap_enabled,
 			       PanelBackType back_type,
 			       char *back_pixmap,
-			       int fit_pixmap_bg,
+			       gboolean fit_pixmap_bg,
+			       gboolean strech_pixmap_bg,
+			       gboolean rotate_pixmap_bg,
 			       GdkColor *back_color)
 {
 	FloatingPos *pos = FLOATING_POS (BASEP_WIDGET (floating)->pos);
@@ -401,7 +403,9 @@ floating_widget_change_params (FloatingWidget *floating,
 				    hidebuttons_enabled,
 				    hidebutton_pixmap_enabled,
 				    back_type, back_pixmap,
-				    fit_pixmap_bg, back_color);
+				    fit_pixmap_bg, strech_pixmap_bg,
+				    rotate_pixmap_bg,
+				    back_color);
 				    
 }
 
@@ -424,6 +428,8 @@ floating_widget_change_orient (FloatingWidget *floating,
 					       panel->back_type,
 					       panel->back_pixmap,
 					       panel->fit_pixmap_bg,
+					       panel->strech_pixmap_bg,
+					       panel->rotate_pixmap_bg,
 					       &panel->back_color);
 	}
 }
@@ -447,6 +453,8 @@ floating_widget_change_coords (FloatingWidget *floating,
 					       panel->back_type,
 					       panel->back_pixmap,
 					       panel->fit_pixmap_bg,
+					       panel->strech_pixmap_bg,
+					       panel->rotate_pixmap_bg,
 					       &panel->back_color);
 	}
 }
@@ -456,12 +464,14 @@ floating_widget_new (gint16 x, gint16 y,
 		     PanelOrientation orient,
 		     BasePMode mode,
 		     BasePState state,
-		     PanelSizeType sz,
+		     int sz,
 		     int hidebuttons_enabled,
 		     int hidebutton_pixmap_enabled,
 		     PanelBackType back_type,
 		     char *back_pixmap,
-		     int fit_pixmap_bg,
+		     gboolean fit_pixmap_bg,
+		     gboolean strech_pixmap_bg,
+		     gboolean rotate_pixmap_bg,
 		     GdkColor *back_color)
 {
 	FloatingWidget *floating;
@@ -483,6 +493,8 @@ floating_widget_new (gint16 x, gint16 y,
 				back_type,
 				back_pixmap,
 				fit_pixmap_bg,
+				strech_pixmap_bg,
+				rotate_pixmap_bg,
 				back_color);
 
 	return GTK_WIDGET (floating);
