@@ -1,7 +1,9 @@
 /*   gnome-panel-properties: crapplet for global panel properties
  *
  *   Copyright (C) 1999 Free Software Foundation
- *   Author: George Lebl <jirka@5z.com>
+ *   Copyright 2000 Helix Code, Inc.
+ *   Authors: George Lebl <jirka@5z.com>
+ *            Jacob Berkman <jacob@helixcode.com>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -22,8 +24,9 @@
 #include <config.h>
 #include <gnome.h>
 #include <libgnorba/gnorba.h>
+#include <gdk/gdkx.h>
 #include "capplet-widget.h"
-
+#include "global-keys.h"
 /*we just use the GlobalConfig and LAST_TILE definitions, some panel-util stuff,
   and the DEFAULT_* defines from session.h !!!*/
 #include "panel-types.h"
@@ -31,6 +34,17 @@
 #include "panel-util.h"
 #include "session.h"
 
+/* for MAIN_MENU_* */
+#include "menu.h"
+
+#include <gdk-pixbuf/gdk-pixbuf.h>
+#include <libart_lgpl/art_alphagamma.h>
+#include <libart_lgpl/art_filterlevel.h>
+#include <libart_lgpl/art_pixbuf.h>
+#include <libart_lgpl/art_rgb_pixbuf_affine.h>
+#include <libart_lgpl/art_affine.h>
+#include "nothing.cP"
+
 static GlobalConfig global_config;
 static GlobalConfig loaded_config;
 
@@ -57,6 +71,7 @@ static GtkWidget *entry_down[LAST_TILE];
 
 static GtkWidget *hide_panel_frame_cb;
 static GtkWidget *tile_when_over_cb;
+static GtkWidget *saturate_when_over_cb;
 
 
 /* applet page*/
@@ -66,15 +81,45 @@ static GtkWidget *movement_type_push_rb;
 static GtkAdjustment *applet_padding;
 
 
-/* miscellaneous page */
+/* menu page */
 static GtkWidget *show_small_icons_cb;
 static GtkWidget *show_dot_buttons_cb;
 static GtkWidget *off_panel_popups_cb;
 static GtkWidget *hungry_menus_cb;
+static GtkWidget *use_large_icons_cb;
+
+typedef struct {
+	int inline_flag;
+	int submenu_flag;
+	char *label;
+
+	GtkWidget *inline_rb;
+	GtkWidget *submenu_rb;
+	GtkWidget *none_rb;
+} MenuOptions;
+
+/* don't forget to update the table size when changing the num of elements
+   in this array */
+static MenuOptions menu_options[] = {
+	{ MAIN_MENU_SYSTEM,  MAIN_MENU_SYSTEM_SUB,  N_("Programs: ") },
+	{ MAIN_MENU_USER,    MAIN_MENU_USER_SUB,    N_("Favorites: ") },
+	{ MAIN_MENU_APPLETS, MAIN_MENU_APPLETS_SUB, N_("Applets: ") },
+	{ MAIN_MENU_REDHAT,  MAIN_MENU_REDHAT_SUB,  N_("AnotherLevel: ") },
+	{ MAIN_MENU_KDE,     MAIN_MENU_KDE_SUB,     N_("KDE: ") },
+	{ MAIN_MENU_DEBIAN,  MAIN_MENU_DEBIAN_SUB,  N_("Debian: ") },
+	{ MAIN_MENU_PANEL,   MAIN_MENU_PANEL_SUB,   N_("Panel: ") },
+	{ MAIN_MENU_DESKTOP, MAIN_MENU_DESKTOP_SUB, N_("Desktop: ") },
+	{ 0 }
+};
+
+/* miscellaneous page */
 static GtkWidget *tooltips_enabled_cb;
 static GtkWidget *drawer_auto_close_cb;
 static GtkWidget *autoraise_cb;
 static GtkWidget *keep_bottom_cb;
+static GtkWidget *keys_enabled_cb;
+static GtkWidget *menu_key_sb;
+static GtkWidget *run_key_sb;
 
 static gboolean changing = TRUE;
 static GtkWidget *capplet;
@@ -86,57 +131,6 @@ changed_cb(void)
 		capplet_widget_state_changed(CAPPLET_WIDGET(capplet), TRUE);
 }
 
-static void
-destroy_egg(GtkWidget *widget, int **pages)
-{
-	if(pages)
-		*pages = 0;
-}
-
-/*thy evil easter egg*/
-static int
-config_event(GtkWidget *widget,GdkEvent *event,GtkNotebook *nbook)
-{
-	GtkWidget *w;
-	char *file;
-	static int clicks=0;
-	static int pages=0;
-	GdkEventButton *bevent;
-	
-	if(event->type != GDK_BUTTON_PRESS)
-		return FALSE;
-	
-	bevent = (GdkEventButton *)event;
-	if(bevent->button != 3)
-		clicks = 0;
-	else
-		clicks++;
-	
-	if(clicks<3)
-		return FALSE;
-	clicks = 0;
-	
-	if(pages==0) {
-		file = gnome_unconditional_pixmap_file("gnome-gegl.png");
-		if (file && g_file_exists (file))
-			w = gnome_pixmap_new_from_file (file);
-		else
-			w = gtk_label_new("<insert picture of goat here>");
-		g_free(file);
-		gtk_widget_show(w);
-		/*the GEGL shall not be translated*/
-		gtk_notebook_append_page (nbook, w,
-					  gtk_label_new ("GEGL"));
-		gtk_notebook_set_page(nbook,-1);
-		pages = 1;
-		gtk_signal_connect(GTK_OBJECT(widget),"destroy",
-				   GTK_SIGNAL_FUNC(destroy_egg),&pages);
-	} else {
-		gtk_notebook_set_page(nbook,-1);
-	}
-	return FALSE;
-}
-
 static GtkWidget *
 make_int_scale_box (char *title, GtkAdjustment **adj,
 		    double min, double max, double step)
@@ -187,6 +181,11 @@ set_anim_button_value(GtkWidget *w, gpointer data)
 static void
 sync_animation_page_with_config(GlobalConfig *conf)
 {
+	gtk_widget_set_sensitive(simple_movement_cb,
+				 !conf->disable_animations);
+	gtk_widget_set_sensitive(anim_frame,
+				 !conf->disable_animations);
+
 	gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(enable_animations_cb),
 				    /*notice the not*/
 				    !conf->disable_animations);
@@ -303,6 +302,8 @@ sync_buttons_page_with_config(GlobalConfig *conf)
 				    conf->hide_panel_frame);
 	gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(tile_when_over_cb),
 				    conf->tile_when_over);
+	gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(saturate_when_over_cb),
+				    conf->saturate_when_over);
 	
 	for(i=0;i<LAST_TILE;i++) {
 		gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(tile_enable_cb[i]),
@@ -323,6 +324,8 @@ sync_config_with_buttons_page(GlobalConfig *conf)
 		GTK_TOGGLE_BUTTON(hide_panel_frame_cb)->active;
 	conf->tile_when_over =
 		GTK_TOGGLE_BUTTON(tile_when_over_cb)->active;
+	conf->saturate_when_over =
+		GTK_TOGGLE_BUTTON(saturate_when_over_cb)->active;
 
 	for(i=0;i<LAST_TILE;i++) {
 		conf->tiles_enabled[i] =
@@ -509,6 +512,12 @@ buttons_notebook_page (void)
 	  gtk_signal_connect (GTK_OBJECT (tile_when_over_cb), "toggled",
 			      GTK_SIGNAL_FUNC (changed_cb), NULL);
 	  gtk_box_pack_start (GTK_BOX (vbox), tile_when_over_cb, FALSE, FALSE, 0);
+
+	  /* saturate on mouseovers hack */
+	  saturate_when_over_cb = gtk_check_button_new_with_label (_("Keep saturation low when cursor is not on the button"));
+	  gtk_signal_connect (GTK_OBJECT (saturate_when_over_cb), "toggled",
+			      GTK_SIGNAL_FUNC (changed_cb), NULL);
+	  gtk_box_pack_start (GTK_BOX (vbox), saturate_when_over_cb, FALSE, FALSE, 0);
 	  
 	  return (vbox);
 }
@@ -601,8 +610,10 @@ applets_notebook_page (void)
 }
 
 static void
-sync_misc_page_with_config(GlobalConfig *conf)
+sync_menu_page_with_config(GlobalConfig *conf)
 {
+	MenuOptions *opt;
+	GtkWidget *w;
 	gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(show_small_icons_cb),
 				    conf->show_small_icons);
 	gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(show_dot_buttons_cb),
@@ -611,18 +622,25 @@ sync_misc_page_with_config(GlobalConfig *conf)
 				    conf->off_panel_popups);
 	gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(hungry_menus_cb),
 				    conf->hungry_menus);
-	gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(tooltips_enabled_cb),
-				    conf->tooltips_enabled);
-	gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(drawer_auto_close_cb),
-				    conf->drawer_auto_close);
-	gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(autoraise_cb),
-				    conf->autoraise);
-	gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(keep_bottom_cb),
-				    conf->keep_bottom);
+	gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(use_large_icons_cb),
+				    conf->use_large_icons);
+	gtk_widget_set_sensitive(use_large_icons_cb, conf->show_small_icons);
+
+	for (opt = menu_options; opt->inline_flag; ++opt) {
+		if (conf->menu_flags & opt->inline_flag)
+			w = opt->inline_rb;
+		else if (conf->menu_flags & opt->submenu_flag)
+			w = opt->submenu_rb;
+		else
+			w = opt->none_rb;
+		gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON (w), TRUE);
+	}
 }
+
 static void
-sync_config_with_misc_page(GlobalConfig *conf)
+sync_config_with_menu_page(GlobalConfig *conf)
 {
+	MenuOptions *opt;
 	conf->show_small_icons =
 		GTK_TOGGLE_BUTTON(show_small_icons_cb)->active;
 	conf->show_dot_buttons =
@@ -631,23 +649,58 @@ sync_config_with_misc_page(GlobalConfig *conf)
 		GTK_TOGGLE_BUTTON(off_panel_popups_cb)->active;
 	conf->hungry_menus =
 		GTK_TOGGLE_BUTTON(hungry_menus_cb)->active;
-	conf->tooltips_enabled =
-		GTK_TOGGLE_BUTTON(tooltips_enabled_cb)->active;
-	conf->drawer_auto_close =
-		GTK_TOGGLE_BUTTON(drawer_auto_close_cb)->active;
-	conf->autoraise =
-		GTK_TOGGLE_BUTTON(autoraise_cb)->active;
-	conf->keep_bottom =
-		GTK_TOGGLE_BUTTON(keep_bottom_cb)->active;
+	conf->use_large_icons =
+		GTK_TOGGLE_BUTTON(use_large_icons_cb)->active;
+	conf->menu_flags = 0;
+	for (opt = menu_options; opt->inline_flag; ++opt) {
+		if (GTK_TOGGLE_BUTTON (opt->inline_rb)->active)
+			conf->menu_flags |= opt->inline_flag;
+		else if (GTK_TOGGLE_BUTTON (opt->submenu_rb)->active)
+			conf->menu_flags |= opt->submenu_flag;
+	}
+}
+
+static void
+add_menu_options (GtkTable *table, MenuOptions *opt, int row)
+{
+	GtkWidget *w;
+	GtkRadioButton *rb;
+
+	w = gtk_label_new (_(opt->label));
+	gtk_table_attach_defaults (table, w, 0, 1, row, row+1);
+
+	w = opt->none_rb = gtk_radio_button_new_with_label (NULL, _("Off"));
+	gtk_table_attach_defaults (table, w, 3, 4, row, row+1);
+	gtk_signal_connect (GTK_OBJECT (w), "toggled", 
+			    GTK_SIGNAL_FUNC (changed_cb),  NULL);
+	rb = GTK_RADIO_BUTTON (w);
+
+	w = opt->submenu_rb = gtk_radio_button_new_with_label (
+		gtk_radio_button_group (rb), _("In a submenu"));
+	gtk_table_attach_defaults (table, w, 2, 3, row, row+1);
+	gtk_signal_connect (GTK_OBJECT (w), "toggled", 
+			    GTK_SIGNAL_FUNC (changed_cb),  NULL);
+
+	w = opt->inline_rb = gtk_radio_button_new_with_label (
+		gtk_radio_button_group (rb), _("In the menu"));
+	gtk_table_attach_defaults (table, w, 1, 2, row, row+1);
+	gtk_signal_connect (GTK_OBJECT (w), "toggled", 
+			    GTK_SIGNAL_FUNC (changed_cb),  NULL);
+}
+
+static void
+toggle_sensitive (GtkWidget *t1, GtkWidget *t2)
+{
+	gtk_widget_set_sensitive (t2, GTK_TOGGLE_BUTTON (t1)->active);
 }
 
 static GtkWidget *
-misc_notebook_page(void)
+menu_notebook_page(void)
 {
 	GtkWidget *frame;
-	GtkWidget *box;
 	GtkWidget *table;
 	GtkWidget *vbox;
+	int i;
 	
 	/* main vbox */
 	vbox = gtk_vbox_new (FALSE, GNOME_PAD_SMALL);
@@ -659,12 +712,12 @@ misc_notebook_page(void)
 	gtk_box_pack_start (GTK_BOX (vbox), frame, FALSE, FALSE, 0);
 	
 	/* table for frame */
-	table = gtk_table_new(2,2,FALSE);
+	table = gtk_table_new(2,3,FALSE);
 	gtk_container_set_border_width(GTK_CONTAINER (table), GNOME_PAD_SMALL);
 	gtk_container_add (GTK_CONTAINER (frame), table);
 	
 	/* Small Icons */
-	show_small_icons_cb = gtk_check_button_new_with_label (_("Show small icons"));
+	show_small_icons_cb = gtk_check_button_new_with_label (_("Show icons"));
 	gtk_signal_connect (GTK_OBJECT (show_small_icons_cb), "toggled", 
 			    GTK_SIGNAL_FUNC (changed_cb), NULL);
 	gtk_table_attach_defaults(GTK_TABLE(table),show_small_icons_cb, 0,1,0,1);
@@ -679,13 +732,157 @@ misc_notebook_page(void)
 	off_panel_popups_cb = gtk_check_button_new_with_label (_("Show popup menus outside of panels"));
 	gtk_signal_connect (GTK_OBJECT (off_panel_popups_cb), "toggled", 
 			    GTK_SIGNAL_FUNC (changed_cb),  NULL);
-	gtk_table_attach_defaults(GTK_TABLE(table),off_panel_popups_cb, 0,1,1,2);
+	gtk_table_attach_defaults(GTK_TABLE(table),off_panel_popups_cb, 0,1,2,3);
 
 	/* Hungry Menus */
 	hungry_menus_cb = gtk_check_button_new_with_label (_("Keep menus in memory"));
 	gtk_signal_connect (GTK_OBJECT (hungry_menus_cb), "toggled", 
 			    GTK_SIGNAL_FUNC (changed_cb), NULL);
 	gtk_table_attach_defaults(GTK_TABLE(table),hungry_menus_cb, 1,2,1,2);
+
+	/* large icons */
+	use_large_icons_cb = gtk_check_button_new_with_label (_("Use large icons"));
+	gtk_signal_connect (GTK_OBJECT (use_large_icons_cb), "toggled",
+			    GTK_SIGNAL_FUNC (changed_cb), NULL);
+	gtk_signal_connect (GTK_OBJECT (show_small_icons_cb), "toggled",
+			    GTK_SIGNAL_FUNC (toggle_sensitive),
+			    use_large_icons_cb);
+	gtk_table_attach_defaults(GTK_TABLE(table),use_large_icons_cb, 0,1,1,2);
+
+	/* Menu frame */
+	frame = gtk_frame_new (_("Panel menu"));
+	gtk_container_set_border_width(GTK_CONTAINER (frame), GNOME_PAD_SMALL);
+	gtk_box_pack_start (GTK_BOX (vbox), frame, FALSE, FALSE, 0);
+	
+	/* table for frame */
+	table = gtk_table_new(7,4,FALSE);
+	gtk_container_set_border_width(GTK_CONTAINER (table), GNOME_PAD_SMALL);
+	gtk_container_add (GTK_CONTAINER (frame), table);
+	
+	for (i=0; menu_options[i].inline_flag; ++i)
+		add_menu_options (GTK_TABLE (table), menu_options+i, i);
+	
+	return vbox;
+}
+
+static void
+sync_misc_page_with_config(GlobalConfig *conf)
+{
+	gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(tooltips_enabled_cb),
+				    conf->tooltips_enabled);
+	gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(drawer_auto_close_cb),
+				    conf->drawer_auto_close);
+	gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(autoraise_cb),
+				    conf->autoraise);
+	gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(keep_bottom_cb),
+				    conf->keep_bottom);
+	gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(keys_enabled_cb),
+				    conf->keys_enabled);
+	gtk_spin_button_set_value (GTK_SPIN_BUTTON (menu_key_sb),
+				   (float)conf->menu_keycode);
+	gtk_object_set_user_data (GTK_OBJECT (menu_key_sb),
+				  GINT_TO_POINTER (conf->menu_state));
+	gtk_spin_button_set_value (GTK_SPIN_BUTTON (run_key_sb),
+				   (float)conf->run_keycode);
+	gtk_object_set_user_data (GTK_OBJECT (run_key_sb),
+				  GINT_TO_POINTER (conf->run_state));
+}
+static void
+sync_config_with_misc_page(GlobalConfig *conf)
+{
+	conf->tooltips_enabled =
+		GTK_TOGGLE_BUTTON(tooltips_enabled_cb)->active;
+	conf->drawer_auto_close =
+		GTK_TOGGLE_BUTTON(drawer_auto_close_cb)->active;
+	conf->autoraise =
+		GTK_TOGGLE_BUTTON(autoraise_cb)->active;
+	conf->keep_bottom =
+		GTK_TOGGLE_BUTTON(keep_bottom_cb)->active;
+	conf->keys_enabled =
+		GTK_TOGGLE_BUTTON(keys_enabled_cb)->active;
+	conf->menu_keycode = gtk_spin_button_get_value_as_int (
+		GTK_SPIN_BUTTON (menu_key_sb));
+	conf->menu_state = GPOINTER_TO_INT (gtk_object_get_user_data (
+		GTK_OBJECT (menu_key_sb)));		
+	conf->run_keycode = gtk_spin_button_get_value_as_int (
+		GTK_SPIN_BUTTON (run_key_sb));
+	conf->run_state = GPOINTER_TO_INT (gtk_object_get_user_data (
+		GTK_OBJECT (run_key_sb)));		
+}
+
+static GtkWidget *grab_dialog;
+
+static GdkFilterReturn
+grab_key_filter(GdkXEvent *gdk_xevent, GdkEvent *event, gpointer data)
+{
+	XEvent *xevent = (XEvent *)gdk_xevent;
+	XKeyEvent *xkevent;
+	GtkSpinButton *sb;
+
+	if (xevent->type != KeyPress && xevent->type != KeyRelease)
+		return GDK_FILTER_CONTINUE;
+	
+	xkevent = (XKeyEvent *)xevent;
+	sb = GTK_SPIN_BUTTON (data);
+
+	g_message ("keycode: %d\tstate: %d",
+		   xkevent->keycode, xkevent->state);
+
+	gtk_spin_button_set_value (sb, (float)xkevent->keycode);
+	gtk_object_set_user_data (GTK_OBJECT (sb),
+				  GINT_TO_POINTER (xkevent->state));
+
+	gdk_keyboard_ungrab (GDK_CURRENT_TIME);
+	gtk_widget_destroy (grab_dialog);
+	gdk_window_remove_filter (GDK_ROOT_PARENT(),
+				  grab_key_filter, data);
+
+	return GDK_FILTER_REMOVE;
+}
+
+static void
+grab_button_pressed (GtkButton *button, gpointer data)
+{
+	GtkWidget *frame;
+	GtkWidget *box;
+	GtkWidget *label;
+	grab_dialog = gtk_window_new (GTK_WINDOW_POPUP);
+
+
+	gdk_keyboard_grab (GDK_ROOT_PARENT(), TRUE, GDK_CURRENT_TIME);
+	gdk_window_add_filter (GDK_ROOT_PARENT(), grab_key_filter, data);
+
+	gtk_window_set_policy (GTK_WINDOW (grab_dialog), FALSE, FALSE, TRUE);
+	gtk_window_set_position (GTK_WINDOW (grab_dialog), GTK_WIN_POS_CENTER);
+	gtk_window_set_modal (GTK_WINDOW (grab_dialog), TRUE);
+
+	frame = gtk_frame_new (NULL);
+	gtk_container_add (GTK_CONTAINER (grab_dialog), frame);
+
+	box = gtk_hbox_new (0, 0);
+	gtk_container_set_border_width (GTK_CONTAINER (box), 20);
+	gtk_container_add (GTK_CONTAINER (frame), box);
+
+	label = gtk_label_new (_("Press a key..."));
+	gtk_container_add (GTK_CONTAINER (box), label);
+	
+	gtk_widget_show_all (grab_dialog);
+	return;
+}
+
+static GtkWidget *
+misc_notebook_page(void)
+{
+	GtkWidget *frame;
+	GtkWidget *box;
+	GtkWidget *table;
+	GtkWidget *vbox;
+	GtkWidget *w;
+	GtkObject *adj;
+	
+	/* main vbox */
+	vbox = gtk_vbox_new (FALSE, GNOME_PAD_SMALL);
+	gtk_container_set_border_width(GTK_CONTAINER (vbox), GNOME_PAD_SMALL);
 	
 	/* Miscellaneous frame */
 	frame = gtk_frame_new (_("Miscellaneous"));
@@ -721,7 +918,63 @@ misc_notebook_page(void)
 			    GTK_SIGNAL_FUNC (changed_cb), NULL);
 	gtk_box_pack_start (GTK_BOX (box), keep_bottom_cb, FALSE, FALSE, 0);
 
-	return (vbox);
+
+	/* Key Bindings frame */
+	frame = gtk_frame_new (_("Key Bindings"));
+	gtk_container_set_border_width(GTK_CONTAINER (frame), GNOME_PAD_SMALL);
+	gtk_box_pack_start (GTK_BOX (vbox), frame, FALSE, FALSE, 0);
+
+	
+	/* table for frame */
+	table = gtk_table_new (3, 3, FALSE);
+	gtk_table_set_row_spacings (GTK_TABLE (table), GNOME_PAD_SMALL);
+	gtk_table_set_col_spacings (GTK_TABLE (table), GNOME_PAD_SMALL);
+	gtk_container_set_border_width(GTK_CONTAINER (table), GNOME_PAD_SMALL);
+	gtk_container_add (GTK_CONTAINER (frame), table);
+
+	/* enabled */
+	keys_enabled_cb = gtk_check_button_new_with_label (_("Enable panel keybindings"));
+	gtk_signal_connect (GTK_OBJECT (keys_enabled_cb), "toggled",
+			    GTK_SIGNAL_FUNC (changed_cb), NULL);
+	gtk_table_attach_defaults (GTK_TABLE (table), keys_enabled_cb, 
+				   0, 3, 0, 1);
+	
+	/* menu key */
+	w = gtk_label_new (_("Popup menu key"));
+	gtk_table_attach_defaults (GTK_TABLE (table), w, 0, 1, 1, 2);
+	
+	adj = gtk_adjustment_new (0, 0, 255, 1, 10, 10);
+	gtk_signal_connect (GTK_OBJECT (adj), "value_changed",
+			    GTK_SIGNAL_FUNC (changed_cb), NULL);
+	menu_key_sb = gtk_spin_button_new (GTK_ADJUSTMENT (adj), 1, 0);
+	gtk_widget_set_sensitive (menu_key_sb, FALSE);
+	gtk_table_attach_defaults (GTK_TABLE (table), menu_key_sb, 1, 2, 1, 2);
+
+	w = gtk_button_new_with_label (_("Grab key..."));
+	gtk_table_attach_defaults (GTK_TABLE (table), w, 2, 3, 1, 2);
+	gtk_signal_connect (GTK_OBJECT (w), "clicked",
+			    GTK_SIGNAL_FUNC (grab_button_pressed),
+			    menu_key_sb);
+
+	/* run key...*/
+	w = gtk_label_new (_("Run dialog key"));
+	gtk_table_attach_defaults (GTK_TABLE (table), w, 0, 1, 2, 3);
+	
+	adj = gtk_adjustment_new (0, 0, 255, 1, 10, 10);
+	gtk_signal_connect (GTK_OBJECT (adj), "value_changed",
+			    GTK_SIGNAL_FUNC (changed_cb), NULL);
+	run_key_sb = gtk_spin_button_new (GTK_ADJUSTMENT (adj), 1, 0);
+	gtk_widget_set_sensitive (run_key_sb, FALSE);
+	gtk_table_attach_defaults (GTK_TABLE (table), run_key_sb, 1, 2, 2, 3);
+
+	w = gtk_button_new_with_label (_("Grab key..."));
+	gtk_table_attach_defaults (GTK_TABLE (table), w, 2, 3, 2, 3);
+	gtk_signal_connect (GTK_OBJECT (w), "clicked",
+			    GTK_SIGNAL_FUNC (grab_button_pressed),
+			    run_key_sb);
+	
+
+ 	return (vbox);
 }
 
 static void
@@ -769,6 +1022,9 @@ loadup_vals(void)
 	global_config.hungry_menus =
 		gnome_config_get_bool("hungry_menus=TRUE");
 
+	global_config.use_large_icons =
+		gnome_config_get_bool("use_large_icons=TRUE");
+
 	global_config.off_panel_popups =
 		gnome_config_get_bool("off_panel_popups=TRUE");
 		
@@ -796,6 +1052,25 @@ loadup_vals(void)
 	g_string_sprintf(buf,"movement_type=%d", PANEL_SWITCH_MOVE);
 	global_config.movement_type=gnome_config_get_int(buf->str);
 
+	g_string_sprintf(buf,"menu_flags=%d", 
+			 (int)(MAIN_MENU_SYSTEM_SUB | MAIN_MENU_USER_SUB|
+			       MAIN_MENU_APPLETS_SUB | MAIN_MENU_REDHAT_SUB|
+			       MAIN_MENU_DEBIAN_SUB | MAIN_MENU_KDE_SUB|
+			       MAIN_MENU_PANEL | MAIN_MENU_DESKTOP));
+	global_config.menu_flags=gnome_config_get_int(buf->str);
+
+	global_config.keys_enabled=gnome_config_get_bool("keys_enabled=TRUE");
+
+	g_string_sprintf(buf,"menu_keycode=%d", XK_LEFT_WIN);
+	global_config.menu_keycode=gnome_config_get_int(buf->str);
+
+	global_config.menu_state=gnome_config_get_int("menu_state=0");
+
+	g_string_sprintf(buf,"run_keycode=%d", GDK_r);
+	global_config.run_keycode=gnome_config_get_int(buf->str);
+
+	global_config.run_state=gnome_config_get_int("run_state=0");
+
 	global_config.applet_padding=gnome_config_get_int("applet_padding=3");
 
 	global_config.autoraise = gnome_config_get_bool("autoraise=TRUE");
@@ -806,8 +1081,11 @@ loadup_vals(void)
 	global_config.simple_movement = gnome_config_get_bool("simple_movement=FALSE");
 	global_config.hide_panel_frame = gnome_config_get_bool("hide_panel_frame=FALSE");
 	global_config.tile_when_over = gnome_config_get_bool("tile_when_over=FALSE");
+	global_config.saturate_when_over = gnome_config_get_bool("saturate_when_over=TRUE");
+	
+
 	for(i=0;i<LAST_TILE;i++) {
-		g_string_sprintf(buf,"tiles_enabled_%d=TRUE",i);
+		g_string_sprintf(buf,"tiles_enabled_%d=FALSE",i);
 		global_config.tiles_enabled[i] =
 			gnome_config_get_bool(buf->str);
 
@@ -876,6 +1154,8 @@ write_config(GlobalConfig *conf)
 			      conf->show_dot_buttons);
 	gnome_config_set_bool("hungry_menus",
 			      conf->hungry_menus);
+	gnome_config_set_bool("use_large_icons",
+			      conf->use_large_icons);
 	gnome_config_set_bool("off_panel_popups",
 			      conf->off_panel_popups);
 	gnome_config_set_bool("disable_animations",
@@ -894,6 +1174,15 @@ write_config(GlobalConfig *conf)
 			      conf->hide_panel_frame);
 	gnome_config_set_bool("tile_when_over",
 			      conf->tile_when_over);
+	gnome_config_set_bool("saturate_when_over",
+			      conf->saturate_when_over);
+	gnome_config_set_int("menu_flags", conf->menu_flags);
+	gnome_config_set_bool("keys_enabled", conf->keys_enabled);
+	gnome_config_set_int("menu_keycode", conf->menu_keycode);
+	gnome_config_set_int("menu_state", conf->menu_state);
+	gnome_config_set_int("run_keycode", conf->run_keycode);
+	gnome_config_set_int("run_sate", conf->run_state);
+			     
 	buf = g_string_new(NULL);
 	for(i=0;i<LAST_TILE;i++) {
 		g_string_sprintf(buf,"tiles_enabled_%d",i);
@@ -925,6 +1214,7 @@ try(GtkWidget *capplet, gpointer data)
 	sync_config_with_animation_page(&global_config);
 	sync_config_with_buttons_page(&global_config);
 	sync_config_with_applets_page(&global_config);
+	sync_config_with_menu_page(&global_config);
 	sync_config_with_misc_page(&global_config);
 	write_config(&global_config);
 }
@@ -935,6 +1225,7 @@ revert(GtkWidget *capplet, gpointer data)
 	sync_animation_page_with_config(&loaded_config);
 	sync_buttons_page_with_config(&loaded_config);
 	sync_applets_page_with_config(&loaded_config);
+	sync_menu_page_with_config(&loaded_config);
 	sync_misc_page_with_config(&loaded_config);
 	global_config = loaded_config;
 	write_config(&global_config);
@@ -963,6 +1254,11 @@ setup_the_ui(GtkWidget *capplet)
 	page = applets_notebook_page ();
 	gtk_notebook_append_page (GTK_NOTEBOOK (nbook),
 				  page, gtk_label_new (_("Applets")));
+
+	/* Menu notebook page */
+	page = menu_notebook_page ();
+	gtk_notebook_append_page (GTK_NOTEBOOK (nbook),
+				  page, gtk_label_new (_("Menu")));
 		
 	/* Miscellaneous notebook page */
 	page = misc_notebook_page ();
@@ -980,6 +1276,7 @@ setup_the_ui(GtkWidget *capplet)
 	sync_animation_page_with_config(&loaded_config);
 	sync_buttons_page_with_config(&loaded_config);
 	sync_applets_page_with_config(&loaded_config);
+	sync_menu_page_with_config(&loaded_config);
 	sync_misc_page_with_config(&loaded_config);
 
 	/* Finished */
