@@ -31,6 +31,7 @@ static char *default_app_pixmap=NULL;
 
 extern GlobalConfig global_config;
 
+extern GSList *panels;
 
 static void
 launch (Launcher *launcher, int argc, char *argv[])
@@ -41,7 +42,7 @@ launch (Launcher *launcher, int argc, char *argv[])
 	g_return_if_fail(launcher->dentry != NULL);
 
 	item = launcher->dentry;
-
+	
 	if(!item->exec) {
 		GtkWidget *dlg;
 		dlg = gnome_message_box_new(_("This launch icon does not "
@@ -54,12 +55,27 @@ launch (Launcher *launcher, int argc, char *argv[])
 		gtk_widget_show_all(dlg);
 		return;
 	}
+	
+	if(item->type && strcmp(item->type,"URL")==0) {
+		char *s;
+		s = g_strjoinv(" ",item->exec);
+		gnome_url_show(s);
+		g_free(s);
+	} else if(item->type && strcmp(item->type,"PanelApplet")==0) {
+		char *goad_id;
 
-	/*UGLY HACK!*/
-	if (item->exec_length > 0
-	    && strstr(item->exec[0], "://"))
-		gnome_url_show(item->exec[0]);
-	else
+		goad_id = get_applet_goad_id_from_dentry(item);
+
+		if(goad_id) {
+			load_extern_applet(goad_id, NULL,
+					   panels->data,
+					   0, FALSE, FALSE);
+		} else {
+			g_warning(_("Can't get goad_id from desktop entry!"));
+		}
+
+		g_free(goad_id);
+	} else
 		gnome_desktop_entry_launch_with_args (item,argc,argv);
 	
 	if(global_config.drawer_auto_close) {
@@ -98,9 +114,18 @@ drag_data_received_cb (GtkWidget        *widget,
 	int argc;
 	char **argv;
 	int i;
+	GtkWidget *wid;
 
 	g_return_if_fail(launcher!=NULL);
 
+	wid = gtk_drag_get_source_widget(context);
+	if(wid == widget) {
+		gtk_drag_finish(context,FALSE,FALSE,time);
+
+		/*gdk_drag_abort(context,time);*/
+		return;
+	}
+
 	files = gnome_uri_list_extract_filenames(selection_data->data);
 	argc = g_list_length(files);
 	argv = g_new(char *,argc+1);
@@ -126,6 +151,28 @@ destroy_launcher(GtkWidget *widget, gpointer data)
 	g_free(launcher);
 }
 
+#if 0
+static gboolean
+drag_motion_cb(GtkWidget *widget,
+	       GdkDragContext *context,
+	       gint x,
+	       gint y,
+	       guint time,
+	       Launcher *launcher)
+{
+	GtkWidget *wid;
+
+	wid = gtk_drag_get_source_widget(context);
+
+	if(wid == widget) {
+		/* somehow make it not highlight or whatever the widget,
+		   I dunno how to do this though */
+	}
+
+	return TRUE;
+}
+#endif
+
 static void  
 drag_data_get_cb (GtkWidget *widget, GdkDragContext     *context,
 		  GtkSelectionData   *selection_data, guint info,
@@ -226,16 +273,21 @@ create_launcher (char *parameters, GnomeDesktopEntry *dentry)
 	GTK_WIDGET_SET_FLAGS(launcher->button,GTK_NO_WINDOW);
 	
 	gtk_drag_dest_set (GTK_WIDGET (launcher->button),
-			   GTK_DEST_DEFAULT_MOTION |
-			   GTK_DEST_DEFAULT_HIGHLIGHT |
-			   GTK_DEST_DEFAULT_DROP,
+			   GTK_DEST_DEFAULT_ALL,
 			   dnd_targets, 1,
 			   GDK_ACTION_COPY);
 
 	gtk_signal_connect(GTK_OBJECT(launcher->button), "drag_data_get",
-			   drag_data_get_cb, launcher);
+			   GTK_SIGNAL_FUNC(drag_data_get_cb),
+			   launcher);
 	gtk_signal_connect(GTK_OBJECT(launcher->button), "drag_data_received",
-			   drag_data_received_cb, launcher);
+			   GTK_SIGNAL_FUNC(drag_data_received_cb),
+			   launcher);
+#if 0
+	gtk_signal_connect(GTK_OBJECT(launcher->button), "drag_motion",
+			   GTK_SIGNAL_FUNC(drag_motion_cb),
+			   launcher);
+#endif
 
 	gtk_signal_connect (GTK_OBJECT(launcher->button), "clicked",
 			    (GtkSignalFunc) launch_cb,
@@ -311,6 +363,7 @@ static GtkWidget *
 create_properties_dialog(Launcher *launcher)
 {
 	GtkWidget  *dialog;
+	GList *types = NULL;
 
 	dialog = gnome_property_box_new();
 	gtk_window_set_wmclass(GTK_WINDOW(dialog),
@@ -321,6 +374,12 @@ create_properties_dialog(Launcher *launcher)
 	launcher->dedit =
 		gnome_dentry_edit_new_notebook(
 		      GTK_NOTEBOOK(GNOME_PROPERTY_BOX(dialog)->notebook));
+	
+	types = g_list_append(types, "Application");
+	types = g_list_append(types, "URL");
+	types = g_list_append(types, "PanelApplet");
+	gtk_combo_set_popdown_strings(GTK_COMBO(GNOME_DENTRY_EDIT(launcher->dedit)->type_combo), types);
+	g_list_free(types);
 
 	gnome_dentry_edit_set_dentry(GNOME_DENTRY_EDIT(launcher->dedit),
 				     launcher->dentry);
@@ -378,31 +437,34 @@ launcher_properties(Launcher *launcher)
 
 void
 load_launcher_applet_full (char *params, GnomeDesktopEntry *dentry,
-			   PanelWidget *panel, int pos)
+			   PanelWidget *panel, int pos, gboolean exactpos)
 {
 	Launcher *launcher;
 
 	launcher = create_launcher(params,dentry);
 
-	if(launcher) {
-		register_toy(launcher->button,launcher, panel, pos,
-			     APPLET_LAUNCHER);
+	if(!launcher)
+		return;
 
-		gtk_tooltips_set_tip (panel_tooltips,
-				      launcher->button,
-				      launcher->dentry->comment,NULL);
+	if(!register_toy(launcher->button, launcher, panel, pos, exactpos,
+			 APPLET_LAUNCHER))
+		return;
 
-		applet_add_callback(applets_last->data,"properties",
-				    GNOME_STOCK_MENU_PROP,
-				    _("Properties..."));
-	}
+	gtk_tooltips_set_tip (panel_tooltips,
+			      launcher->button,
+			      launcher->dentry->comment,NULL);
+
+	applet_add_callback(applets_last->data,"properties",
+			    GNOME_STOCK_MENU_PROP,
+			    _("Properties..."));
 }
 
 static void
-really_add_launcher(GtkWidget *d,int button, gpointer data)
+really_add_launcher(GtkWidget *d, int button, gpointer data)
 {
 	GnomeDEntryEdit *dedit = GNOME_DENTRY_EDIT(data);
 	int pos = GPOINTER_TO_INT(gtk_object_get_data(GTK_OBJECT(d),"pos"));
+	gboolean exactpos = GPOINTER_TO_INT(gtk_object_get_data(GTK_OBJECT(d),"exactpos"));
 	PanelWidget *panel = gtk_object_get_data(GTK_OBJECT(d),"panel");
 	GnomeDesktopEntry *dentry;
 	
@@ -413,18 +475,19 @@ really_add_launcher(GtkWidget *d,int button, gpointer data)
 			g_free(dentry->name);
 			dentry->name=g_strdup("???");
 		}
-		load_launcher_applet_full (NULL, dentry, panel, pos);
+		load_launcher_applet_full (NULL, dentry, panel, pos, exactpos);
 		panel_config_sync();
 	}
 	gtk_widget_destroy(d);
 }
 
 void
-ask_about_launcher(char *file, PanelWidget *panel, int pos)
+ask_about_launcher(char *file, PanelWidget *panel, int pos, gboolean exactpos)
 {
 	GtkWidget *d;
 	GtkWidget *notebook;
 	GnomeDEntryEdit *dee;
+	GList *types = NULL;
 
 	d = gnome_dialog_new(_("Create launcher applet"),
 			     GNOME_STOCK_BUTTON_OK,
@@ -438,6 +501,12 @@ ask_about_launcher(char *file, PanelWidget *panel, int pos)
 	gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(d)->vbox),notebook,
 			   TRUE,TRUE,GNOME_PAD_SMALL);
 	dee = GNOME_DENTRY_EDIT(gnome_dentry_edit_new_notebook(GTK_NOTEBOOK(notebook)));
+
+	types = g_list_append(types, "Application");
+	types = g_list_append(types, "URL");
+	types = g_list_append(types, "PanelApplet");
+	gtk_combo_set_popdown_strings(GTK_COMBO(dee->type_combo), types);
+	g_list_free(types);
 	
 	if(file)
 		gtk_entry_set_text(GTK_ENTRY(dee->exec_entry), file);
@@ -446,6 +515,8 @@ ask_about_launcher(char *file, PanelWidget *panel, int pos)
 	
 	
 	gtk_object_set_data(GTK_OBJECT(d),"pos", GINT_TO_POINTER(pos));
+	gtk_object_set_data(GTK_OBJECT(d),"exactpos",
+			    GINT_TO_POINTER(exactpos));
 	gtk_object_set_data(GTK_OBJECT(d),"panel",panel);
 
 	gtk_signal_connect(GTK_OBJECT(d),"clicked",
@@ -462,7 +533,8 @@ ask_about_launcher(char *file, PanelWidget *panel, int pos)
 void
 load_launcher_applet_from_info(char *name, char *comment,
 			       char **exec, int execn, char *icon,
-			       PanelWidget *panel, int pos)
+			       PanelWidget *panel, int pos,
+			       gboolean exactpos)
 {
 	GnomeDesktopEntry *dentry = g_new0(GnomeDesktopEntry,1);
 	dentry->name = g_strdup(name);
@@ -473,13 +545,39 @@ load_launcher_applet_from_info(char *name, char *comment,
 		dentry->icon = gnome_pixmap_file(icon);
 	else
 		dentry->icon = g_strdup(icon);
+	
+	dentry->type = g_strdup("Application");
 
-	load_launcher_applet_full (NULL,dentry,panel, pos);
+	load_launcher_applet_full (NULL, dentry, panel, pos, exactpos);
 	panel_config_sync();
 }
 
 void
-load_launcher_applet(char *params, PanelWidget *panel, int pos)
+load_launcher_applet_from_info_url(char *name, char *comment,
+				   char *url, char *icon,
+				   PanelWidget *panel, int pos,
+				   gboolean exactpos)
 {
-	load_launcher_applet_full (params,NULL,panel, pos);
+	char *exec[] = { NULL, NULL };
+	GnomeDesktopEntry *dentry = g_new0(GnomeDesktopEntry,1);
+	dentry->name = g_strdup(name);
+	dentry->comment = g_strdup(comment);
+	dentry->exec_length = 1;
+	exec[0] = url;
+	dentry->exec = g_copy_vector(exec);
+	if(icon && *icon != '/')
+		dentry->icon = gnome_pixmap_file(icon);
+	else
+		dentry->icon = g_strdup(icon);
+	dentry->type = g_strdup("URL");
+
+	load_launcher_applet_full(NULL, dentry, panel, pos, exactpos);
+	panel_config_sync();
+}
+
+void
+load_launcher_applet(char *params, PanelWidget *panel, int pos,
+		     gboolean exactpos)
+{
+	load_launcher_applet_full(params, NULL, panel, pos, exactpos);
 }
