@@ -12,11 +12,11 @@
 #include <sys/wait.h>
 #include <gnome.h>
 #include <libgnorba/gnorba.h>
-#include <gdk/gdkx.h>
-#include <X11/Xlib.h>
 
 #include "panel-include.h"
 
+#include "xstuff.h"
+
 extern int config_sync_timeout;
 extern int applets_to_sync;
 extern int panels_to_sync;
@@ -42,7 +42,9 @@ GnomeClient *client = NULL;
 /*a list of started extern applet child processes*/
 extern GList * children;
 
-GList *check_swallows = NULL;
+char *kde_menudir = NULL;
+char *kde_icondir = NULL;
+char *kde_mini_icondir = NULL;
 
 static int
 menu_age_timeout(gpointer data)
@@ -93,88 +95,52 @@ try_config_sync(gpointer data)
 	return TRUE;
 }
 
-static int
-ignore_x_error(Display* d, XErrorEvent* e)
+static void
+find_kde_directory(void)
 {
-	return 0;
-}
+	int i;
+	char *kdedir = getenv("KDEDIR");
+	char *try_prefixes[] = {
+		"/usr",
+		"/opt/kde",
+		"/usr/local",
+		"/kde",
+		NULL
+	};
+	if(kdedir) {
+		kde_menudir = g_concat_dir_and_file(kdedir,"share/applnk");
+		kde_icondir = g_concat_dir_and_file(kdedir,"share/icons");
+		kde_mini_icondir = g_concat_dir_and_file(kdedir,"share/icons/mini");
+		return;
+	}
 
-static int
-get_window_id(Window win, char *title, guint32 *wid)
-{
-	Window root_return;
-	Window parent_return;
-	Window *children = NULL;
-	unsigned int nchildren;
-	unsigned int i;
-	char *tit;
-	int ret = FALSE;
-	
-	if(XFetchName(GDK_DISPLAY(), win, &tit) && tit) {
-		if(strstr(tit,title)!=NULL) {
-			if(wid) *wid = win;
-			ret = TRUE;
+	/* if what configure gave us works use that */
+	if(g_file_test(KDE_MENUDIR,G_FILE_TEST_ISDIR)) {
+		kde_menudir = g_strdup(KDE_MENUDIR);
+		kde_icondir = g_strdup(KDE_ICONDIR);
+		kde_mini_icondir = g_strdup(KDE_MINI_ICONDIR);
+		return;
+	}
+
+	for(i=0;try_prefixes[i];i++) {
+		char *try;
+		try = g_concat_dir_and_file(try_prefixes[i],"share/applnk");
+		if(g_file_test(try,G_FILE_TEST_ISDIR)) {
+			kde_menudir = try;
+			kde_icondir = g_concat_dir_and_file(try_prefixes[i],"share/icons");
+			kde_mini_icondir = g_concat_dir_and_file(try_prefixes[i],"share/icons/mini");
+			return;
 		}
-		XFree(tit);
+		g_free(try);
 	}
-	
-	if(ret) return TRUE;
 
-	XQueryTree(GDK_DISPLAY(),
-		   win,
-		   &root_return,
-		   &parent_return,
-		   &children,
-		   &nchildren);
-	
-	/*otherwise we got a problem*/
-	if(children) {
-		for(i=0;!ret && i<nchildren;i++)
-			ret=get_window_id(children[i],title,wid);
-		XFree(children);
-	}
-	return ret;
+	/* absolute fallback, these don't exist, but we're out of options
+	   here */
+	kde_menudir = g_strdup(KDE_MENUDIR);
+	kde_icondir = g_strdup(KDE_ICONDIR);
+	kde_mini_icondir = g_strdup(KDE_MINI_ICONDIR);
 }
 
-static GdkFilterReturn
-event_filter(GdkXEvent *gdk_xevent, GdkEvent *event, gpointer data)
-{
-	XEvent *xevent;
-
-	xevent = (XEvent *)gdk_xevent;
-	if (xevent->type == MapNotify && check_swallows) {
-		int (*oldErrorHandler)(Display*, XErrorEvent*);
-		GList *li;
-		int remove;
-
-		oldErrorHandler = XSetErrorHandler(ignore_x_error);
-
-		remove = 0;
-		for(li = check_swallows; li; li = g_list_next(li)) {
-			Swallow *swallow = li->data;
-			if(get_window_id(xevent->xmap.window,swallow->title,
-					 &(swallow->wid))) {
-				gtk_socket_steal(GTK_SOCKET(swallow->socket),swallow->wid);
-				li->data = NULL;
-				remove++;
-			}
-		}
-		while(remove--)
-			check_swallows = g_list_remove(check_swallows,NULL);
-		
-		if(!check_swallows)
-			XSelectInput(GDK_DISPLAY(), GDK_ROOT_WINDOW(), 0);
-		
-		XSetErrorHandler(oldErrorHandler);
-	}
-	if ((event->any.window) &&
-	    (gdk_window_get_type(event->any.window) == GDK_WINDOW_FOREIGN))
-		return GDK_FILTER_REMOVE;
-	else
-		return GDK_FILTER_CONTINUE;
-}
-
-
 int
 main(int argc, char **argv)
 {
@@ -217,6 +183,8 @@ main(int argc, char **argv)
 	}
 	}
 
+	find_kde_directory();
+
 	client = gnome_master_client ();
 
 	gnome_client_set_restart_style (client, duplicate 
@@ -256,13 +224,16 @@ main(int argc, char **argv)
 
 	/*set the globals*/
 	load_up_globals();
+
+	init_fr_chunks ();
 	
 	init_menus();
 	
 	init_user_panels();
-
 	init_user_applets();
 
+	load_tornoff();
+
 	gnome_triggers_do("Session startup", NULL, "gnome", "login", NULL);
 
 	/*add forbidden lists to ALL panels*/
@@ -281,9 +252,9 @@ main(int argc, char **argv)
 	  starting multiple goad_id's at once are libgnorba's problem*/
 	load_queued_externs();
 
-	/* set up a filter on the root window to get map requests */
-	/* we will select the events later when we actually need them */
-	gdk_window_add_filter(GDK_ROOT_PARENT(), event_filter, NULL);
+	xstuff_init();
+
+	status_applet_create_offscreen();
 	
 	/* I use the glue code to avoid making this a C++ file */
 	gtk_main ();
