@@ -25,7 +25,11 @@ enum {
 	MAIN_MENU_DEBIAN = 1<<8,
 	MAIN_MENU_DEBIAN_SUB = 1<<9,
 	MAIN_MENU_APPLETS = 1<<10,
-	MAIN_MENU_APPLETS_SUB = 1<<11
+	MAIN_MENU_APPLETS_SUB = 1<<11,
+	MAIN_MENU_PANEL = 1<<12,
+	MAIN_MENU_PANEL_SUB = 1<<13,
+	MAIN_MENU_DESKTOP = 1<<14,
+	MAIN_MENU_DESKTOP_SUB = 1<<15
 };
 
 typedef struct _Menu Menu;
@@ -39,7 +43,7 @@ struct _Menu {
 };
 
 void load_menu_applet(char *params, int main_menu_flags,
-		      PanelWidget *panel, int pos);
+		      PanelWidget *panel, int pos, gboolean exactpos);
 void add_menu_widget (Menu *menu, GSList *menudirl, int main_menu,
 		      int fake_subs);
 
@@ -59,6 +63,9 @@ void panel_lock (GtkWidget *widget, void *data);
 /*to be called on startup to load in some of the directories*/
 void init_menus(void);
 
+void save_tornoff(void);
+void load_tornoff(void);
+
 #define MENU_PATH "menu_path"
 
 #define MENU_PROPERTIES "menu_properties"
@@ -87,11 +94,13 @@ enum {
 	HIDEBUTTONS_NONE
 };
 
+
 #define MENU_SIZES "sizes_menu"
 #define MENU_SIZE_TINY "Tiny (24 pixels)"
+#define MENU_SIZE_SMALL "Small (36 pixels)"
 #define MENU_SIZE_STANDARD "Standard (48 pixels)"
 #define MENU_SIZE_LARGE "Large (64 pixels)"
-#define MENU_SIZE_HUGE "Huge (80 pixels"
+#define MENU_SIZE_HUGE "Huge (80 pixels)"
 
 #define MENU_BACKS "background_menu"
 #define MENU_BACK_NONE "Standard"
