@@ -5,7 +5,7 @@
 
 BEGIN_GNOME_DECLS
 
-int string_is_in_list(GSList *list,char *text);
+gboolean string_is_in_list(GSList *list,char *text);
 GtkWidget * create_text_entry(GtkWidget *table,
 			      char *history_id,
 			      int row,
@@ -43,7 +43,17 @@ void set_frame_colors(PanelWidget *panel, GtkWidget *frame,
 		      GtkWidget *but1, GtkWidget *but2, GtkWidget *but3,
 		      GtkWidget *but4);
 
-void remove_directory(char *dirname, int just_clean);
+/* recursively remove directory, if just_clean is true, then
+   the directory itself is also removed, otherwise just it's
+   contents, it does not give any error output and will fail
+   silently, but it will remove as much as possible */
+void remove_directory(char *dirname, gboolean just_clean);
+
+/* like strtok, but unescapes string as well and escaped separators
+   don't count.  if empty is true, don't skip empty parts */
+char *strtok_with_escape(char *string, const char *seps, gboolean empty);
+/* return a newly allocated string that escapes / and 'special' */
+char *escape_string(const char *string, const char *special);
 
 
 END_GNOME_DECLS
