@@ -26,7 +26,7 @@ GSList *panels=NULL; /*other panels we might want to move the applet to*/
 
 /*there  can universally be only one applet being dragged since we assume
 we only have one mouse :) */
-int panel_applet_in_drag = FALSE;
+gboolean panel_applet_in_drag = FALSE;
 
 static void panel_widget_class_init	(PanelWidgetClass *klass);
 static void panel_widget_init		(PanelWidget      *panel_widget);
@@ -57,7 +57,7 @@ int pw_drawer_step = 20;
 int pw_auto_step = 10;
 int pw_minimized_size = 6;
 int pw_minimize_delay = 300;
-int pw_disable_animations = FALSE;
+gboolean pw_disable_animations = FALSE;
 PanelMovementType pw_movement_type = PANEL_SWITCH_MOVE;
 int pw_applet_padding = 3;
 
@@ -131,10 +131,11 @@ enum {
 	APPLET_ADDED_SIGNAL,
 	APPLET_REMOVED_SIGNAL,
 	BACK_CHANGE_SIGNAL,
+	APPLET_DRAW_SIGNAL,
 	LAST_SIGNAL
 };
 
-static int panel_widget_signals[LAST_SIGNAL] = {0,0,0,0,0};
+static int panel_widget_signals[LAST_SIGNAL] = {0};
 static GtkFixedClass *parent_class = NULL;
 
 
@@ -179,10 +180,10 @@ panel_widget_class_init (PanelWidgetClass *class)
 			       object_class->type,
 			       GTK_SIGNAL_OFFSET(PanelWidgetClass,
 			       			 size_change),
-			       gtk_marshal_NONE__ENUM,
+			       gtk_marshal_NONE__INT,
 			       GTK_TYPE_NONE,
 			       1,
-			       GTK_TYPE_ENUM);
+			       GTK_TYPE_INT);
 	panel_widget_signals[APPLET_MOVE_SIGNAL] =
 		gtk_signal_new("applet_move",
 			       GTK_RUN_LAST,
@@ -225,6 +226,16 @@ panel_widget_class_init (PanelWidgetClass *class)
 			       GTK_TYPE_ENUM,
 			       GTK_TYPE_POINTER,
 			       GTK_TYPE_POINTER);
+	panel_widget_signals[APPLET_DRAW_SIGNAL] =
+		gtk_signal_new("applet_draw",
+			       GTK_RUN_LAST,
+			       object_class->type,
+			       GTK_SIGNAL_OFFSET(PanelWidgetClass,
+			       			 applet_draw),
+			       gtk_marshal_NONE__POINTER,
+			       GTK_TYPE_NONE,
+			       1,
+			       GTK_TYPE_POINTER);
 	gtk_object_class_add_signals(object_class,panel_widget_signals,
 				     LAST_SIGNAL);
 
@@ -796,10 +807,10 @@ panel_widget_size_request(GtkWidget *widget, GtkRequisition *requisition)
 
 	if(panel->orient == PANEL_HORIZONTAL) {
 		requisition->width = pw_applet_padding;
-		requisition->height = panel_widget_get_pixel_size(panel);
+		requisition->height = panel->sz;
 	} else {
 		requisition->height = pw_applet_padding;
-		requisition->width = panel_widget_get_pixel_size(panel);
+		requisition->width = panel->sz;
 	}
 
 	for(list = panel->applet_list; list!=NULL; list = g_list_next(list)) {
@@ -827,12 +838,15 @@ panel_widget_size_request(GtkWidget *widget, GtkRequisition *requisition)
 			requisition->height = panel->size;
 		}
 	}
+	requisition->width = CLAMP (requisition->width, 12, gdk_screen_width ());
+	requisition->height = CLAMP (requisition->height, 12, gdk_screen_height ());
 }
 
 static void
 make_background(PanelWidget *panel, guchar *rgb_buf,
 		int x, int y, int w, int h,
-		GdkPixbuf *pb, int scale_w, int scale_h)
+		GdkPixbuf *pb, int scale_w, int scale_h,
+		int rotate)
 {
 	if(pb) {
 		if(scale_w == 0 || scale_h == 0) {
@@ -844,7 +858,8 @@ make_background(PanelWidget *panel, guchar *rgb_buf,
 				 pb->art_pixbuf->has_alpha);
 		} else {
 			tile_rgb_pixbuf(rgb_buf, w, h, x, y, w*3,
-					pb->art_pixbuf, scale_w, scale_h);
+					pb->art_pixbuf, scale_w, scale_h,
+					rotate);
 		}
 	} else {
 		int i;
@@ -891,6 +906,57 @@ kill_cache_on_all_buttons(PanelWidget *panel, int even_no_alpha)
 	}
 }
 
+static void
+setup_background(PanelWidget *panel, GdkPixbuf **pb, int *scale_w, int *scale_h,
+		 gboolean *rotate)
+{
+	GtkWidget *widget = GTK_WIDGET(panel);
+	GdkPixmap *bg_pixmap;
+	
+	*pb = NULL;
+	*scale_w = *scale_h = 0;
+	*rotate = FALSE;
+
+	bg_pixmap = widget->style->bg_pixmap[GTK_WIDGET_STATE(widget)];
+
+	if(panel->back_type == PANEL_BACK_NONE) {
+		if(bg_pixmap && !panel->backpix) {
+			if(panel->backpix)
+				gdk_pixbuf_unref(panel->backpix);
+			panel->backpix = my_gdk_pixbuf_rgb_from_drawable(bg_pixmap);
+			kill_cache_on_all_buttons(panel, FALSE);
+		} else if(!bg_pixmap && panel->backpix) {
+			if(panel->backpix)
+				gdk_pixbuf_unref(panel->backpix);
+			panel->backpix = NULL;
+			kill_cache_on_all_buttons(panel, FALSE);
+		}
+		*pb = panel->backpix;
+	} else if(panel->back_type == PANEL_BACK_PIXMAP) {
+		if(!panel->backpixmap)
+			panel_resize_pixmap(panel);
+
+		*pb = panel->backpix;
+
+		if(panel->fit_pixmap_bg ||
+		   panel->strech_pixmap_bg) {
+			*scale_w = panel->scale_w;
+			*scale_h = panel->scale_h;
+			if(panel->orient == PANEL_VERTICAL &&
+			   panel->rotate_pixmap_bg)
+				*rotate = TRUE;
+		} else {
+			if(panel->orient == PANEL_VERTICAL &&
+			   panel->rotate_pixmap_bg) {
+				/* we need to set scales to rotate*/
+				*scale_w = (*pb)->art_pixbuf->width;
+				*scale_h = (*pb)->art_pixbuf->height;
+				*rotate = TRUE;
+			}
+		}
+	} 
+}
+
 void
 panel_widget_draw_all(PanelWidget *panel, GdkRectangle *area)
 {
@@ -898,17 +964,21 @@ panel_widget_draw_all(PanelWidget *panel, GdkRectangle *area)
 	GtkWidget *widget;
 	GdkGC *gc;
 	GdkPixmap *pixmap;
-	GdkPixmap *bg_pixmap;
 	GdkRectangle da;
 	GdkPixbuf *pb = NULL;
 	int size;
 	int scale_w = 0,scale_h = 0;
+	int rotate = FALSE;
 
 	g_return_if_fail(panel!=NULL);
 	g_return_if_fail(IS_PANEL_WIDGET(panel));
 
 	widget = GTK_WIDGET(panel);
 
+#ifdef PANEL_DEBUG
+	puts("PANEL_WIDGET_DRAW_ALL");
+#endif
+
 	if(!GTK_WIDGET_DRAWABLE(widget) ||
 	   widget->allocation.width <= 0 ||
 	   widget->allocation.height <= 0)
@@ -924,29 +994,8 @@ panel_widget_draw_all(PanelWidget *panel, GdkRectangle *area)
 		da.width = widget->allocation.width;
 		da.height = widget->allocation.height;
 	}
-	
-	bg_pixmap = widget->style->bg_pixmap[GTK_WIDGET_STATE(widget)];
 
-	if(panel->back_type == PANEL_BACK_NONE) {
-		if(bg_pixmap && !panel->backpix) {
-			if(panel->backpix)
-				gdk_pixbuf_unref(panel->backpix);
-			panel->backpix = my_gdk_pixbuf_rgb_from_drawable(bg_pixmap);
-			kill_cache_on_all_buttons(panel, FALSE);
-		} else if(!bg_pixmap && panel->backpix) {
-			if(panel->backpix)
-				gdk_pixbuf_unref(panel->backpix);
-			panel->backpix = NULL;
-			kill_cache_on_all_buttons(panel, FALSE);
-		}
-		pb = panel->backpix;
-	} else if(panel->back_type == PANEL_BACK_PIXMAP) {
-		if(panel->fit_pixmap_bg) {
-			scale_w = panel->scale_w;
-			scale_h = panel->scale_h;
-		}
-		pb = panel->backpix;
-	} 
+	setup_background(panel, &pb, &scale_w, &scale_h, &rotate);
 	
 	pixmap = gdk_pixmap_new(widget->window,
 				da.width,da.height,
@@ -976,11 +1025,14 @@ panel_widget_draw_all(PanelWidget *panel, GdkRectangle *area)
 	gdk_draw_rectangle(pixmap,gc, TRUE, 0,0,-1,-1);
 
 	/* get pixel size of an icon */
-	size = panel_widget_get_pixel_size(panel);
+	size = panel->sz;
 
 	for(li = panel->applet_list; li != NULL;
 	    li = g_list_next(li)) {
 		AppletData *ad = li->data;
+		gtk_signal_emit(GTK_OBJECT(panel),
+				panel_widget_signals[APPLET_DRAW_SIGNAL],
+				ad->applet);
 		if(IS_BUTTON_WIDGET(ad->applet) &&
 		   ad->applet->allocation.x>=0 &&
 		   (!area || gtk_widget_intersect(ad->applet, area, NULL))) {
@@ -998,7 +1050,8 @@ panel_widget_draw_all(PanelWidget *panel, GdkRectangle *area)
 					make_background(panel, rgb_buf,
 							ad->applet->allocation.x,
 							ad->applet->allocation.y,
-							size,size, pb, scale_w, scale_h);
+							size,size, pb, scale_w, scale_h,
+							rotate);
 				button_widget_draw(button, rgb_buf, size*3);
 				gdk_draw_rgb_image(button->cache,gc,
 						   0,0, size, size,
@@ -1033,6 +1086,10 @@ panel_widget_draw_icon(PanelWidget *panel, ButtonWidget *button)
 	g_return_if_fail(button != NULL);
 	g_return_if_fail(GTK_IS_WIDGET(button));
 	
+#ifdef PANEL_DEBUG
+	puts("PANEL_WIDGET_DRAW_ICON");
+#endif
+
 	widget = GTK_WIDGET(button);
 	
 	area.x = widget->allocation.x;
@@ -1051,10 +1108,19 @@ panel_widget_draw(GtkWidget *widget, GdkRectangle *area)
 
 	g_return_if_fail(widget!=NULL);
 	g_return_if_fail(IS_PANEL_WIDGET(widget));
+	
+#ifdef PANEL_DEBUG
+	puts("PANEL_WIDGET_DRAW");
+	
+	printf("allocation %d x %d\n",
+	       widget->allocation.width,
+	       widget->allocation.height);
+#endif
 
 	panel = PANEL_WIDGET(widget);
 
-	if(!GTK_WIDGET_DRAWABLE(widget))
+	if(!GTK_WIDGET_DRAWABLE(widget) ||
+	   panel->inhibit_draw)
 		return;
 
 	panel_widget_draw_all(panel,area);
@@ -1084,6 +1150,10 @@ panel_widget_expose(GtkWidget *widget, GdkEventExpose *event)
 
 	if(!GTK_WIDGET_DRAWABLE(widget))
 		return FALSE;
+	
+#ifdef PANEL_DEBUG
+	puts("PANEL_WIDGET_EXPOSE");
+#endif
 
 	panel_widget_draw_all(panel,&event->area);
 
@@ -1103,38 +1173,6 @@ panel_widget_expose(GtkWidget *widget, GdkEventExpose *event)
 	return FALSE;
 }
 
-static int
-fit_pix_timeout(gpointer data)
-{
-	PanelWidget *panel = data;
-	if(panel->pixmap_resize_pending &&
-	   panel->fit_pixmap_bg &&
-	   panel->back_type == PANEL_BACK_PIXMAP) {
-		panel_resize_pixmap(panel);
-		panel->pixmap_resize_pending = FALSE;
-		return TRUE;
-	}
-	panel->pixmap_resize_timeout = 0;
-	panel->pixmap_resize_pending = FALSE;
-	return FALSE;
-}
-
-static int
-fit_pix_timeout_top(gpointer data)
-{
-	PanelWidget *panel = data;
-	panel->pixmap_resize_timeout_top = 0;
-	if(panel->pixmap_resize_timeout ||
-	   !panel->fit_pixmap_bg ||
-	   panel->back_type != PANEL_BACK_PIXMAP)
-		return FALSE;
-	panel_resize_pixmap(panel);
-	panel->pixmap_resize_pending = FALSE;
-	panel->pixmap_resize_timeout =
-		gtk_timeout_add(2000,fit_pix_timeout,panel);
-	return FALSE;
-}
-
 static void
 panel_widget_size_allocate(GtkWidget *widget, GtkAllocation *allocation)
 {
@@ -1145,15 +1183,28 @@ panel_widget_size_allocate(GtkWidget *widget, GtkAllocation *allocation)
 	int i;
 	int old_size;
 	int old_thick;
+	GtkAllocation old_alloc;
 
 	g_return_if_fail(widget!=NULL);
 	g_return_if_fail(IS_PANEL_WIDGET(widget));
 	g_return_if_fail(allocation!=NULL);
-
+	
 	panel = PANEL_WIDGET(widget);
 
+#ifdef PANEL_DEBUG
+	puts("PANEL_WIDGET_SIZE_ALLOCATE");
+	printf("allocation %d x %d\n",
+	       allocation->width,
+	       allocation->height);
+#endif
+
+	/* allow drawing if it was inhibited */
+	panel->inhibit_draw = FALSE;
+	
 	old_size = panel->size;
 	old_thick = panel->thick;
+	
+	old_alloc = widget->allocation;
 
 	if((widget->allocation.width != allocation->width ||
 	    widget->allocation.height != allocation->height) &&
@@ -1267,16 +1318,13 @@ panel_widget_size_allocate(GtkWidget *widget, GtkAllocation *allocation)
 	else
 		panel->thick = allocation->width;
 
-	if(old_thick != panel->thick &&
-	   panel->fit_pixmap_bg &&
-	   panel->back_type == PANEL_BACK_PIXMAP) {
-		if(panel->pixmap_resize_timeout)
-			panel->pixmap_resize_pending = TRUE;
-		else {
-			if(panel->pixmap_resize_timeout_top)
-				gtk_timeout_remove(panel->pixmap_resize_timeout_top);
-			panel->pixmap_resize_timeout_top = 
-				gtk_timeout_add(500,fit_pix_timeout_top,panel);
+	if(panel->back_type == PANEL_BACK_PIXMAP &&
+	   (panel->fit_pixmap_bg || panel->strech_pixmap_bg) &&
+	   (old_alloc.width != allocation->width ||
+	    old_alloc.height != allocation->height)) {
+		if(panel->backpixmap) {
+			gdk_pixmap_unref(panel->backpixmap);
+			panel->backpixmap = NULL;
 		}
 	}
 
@@ -1328,6 +1376,10 @@ panel_widget_set_back_pixmap (PanelWidget *panel, char *file)
 	g_return_if_fail(IS_PANEL_WIDGET(panel));
 	g_return_if_fail(file!=NULL);
 
+#ifdef PANEL_DEBUG
+	puts("PANEL_WIDGET_SET_BACK_PIXMAP");
+#endif
+
 	if (panel_try_to_set_pixmap (panel, file)) {
 		if (panel->back_pixmap)
 			g_free (panel->back_pixmap);
@@ -1344,17 +1396,6 @@ panel_widget_set_back_pixmap (PanelWidget *panel, char *file)
 		panel_widget_draw_all(panel,NULL);
 }
 
-static void
-panel_try_to_set_default_back(PanelWidget *panel)
-{
-	g_return_if_fail(panel!=NULL);
-	g_return_if_fail(IS_PANEL_WIDGET(panel));
-
-	kill_cache_on_all_buttons(panel, FALSE);
-
-	panel_widget_draw_all(panel,NULL);
-}
-
 static void
 panel_try_to_set_back_color(PanelWidget *panel, GdkColor *color)
 {
@@ -1370,8 +1411,6 @@ panel_try_to_set_back_color(PanelWidget *panel, GdkColor *color)
 
 	if(gdk_colormap_alloc_color(cmap,color,FALSE,TRUE)) {
 		panel->back_color = *color;
-
-		panel_widget_draw_all(panel,NULL);
 	}
 }
 
@@ -1384,6 +1423,7 @@ panel_widget_set_back_color(PanelWidget *panel, GdkColor *color)
 
 	panel->back_type = PANEL_BACK_COLOR;
 	panel_try_to_set_back_color(panel, color);
+	panel_widget_draw_all(panel,NULL);
 
 	gtk_signal_emit(GTK_OBJECT(panel),
 			panel_widget_signals[BACK_CHANGE_SIGNAL],
@@ -1393,33 +1433,51 @@ panel_widget_set_back_color(PanelWidget *panel, GdkColor *color)
 }
 
 static GdkPixmap *
-get_pixmap_from_pixbuf(GtkWidget *w, GdkPixbuf *pb, int scale_w, int scale_h)
+get_pixmap_from_pixbuf(GtkWidget *w, GdkPixbuf *pb, int scale_w, int scale_h,
+		       int rotate)
 {
 	GdkGC *gc;
 	GdkPixmap *p;
 	gdouble affine[6];
 	guchar *rgb;
-	p = gdk_pixmap_new(w->window, scale_w,scale_h,
-			   gtk_widget_get_visual(GTK_WIDGET(w))->depth);
-	gc = gdk_gc_new(p);
 	
 	affine[1] = affine[2] = affine[4] = affine[5] = 0;
 
 	affine[0] = scale_w / (double)(pb->art_pixbuf->width);
 	affine[3] = scale_h / (double)(pb->art_pixbuf->height);
+
+	if(rotate) {
+		int tmp;
+		gdouble aff[6];
+
+		art_affine_rotate(aff,270);
+		art_affine_multiply(affine,affine,aff);
+		art_affine_translate(aff,0,scale_w);
+		art_affine_multiply(affine,affine,aff);
+		
+		tmp = scale_h;
+		scale_h = scale_w;
+		scale_w = tmp;
+	}
 	
-	rgb = g_new0(guchar,scale_w*scale_h*3);
+	rgb = g_new0(guchar,scale_h*scale_w*3);
+#ifdef PANEL_DEBUG
+	printf("scale_w %d scale_h %d\n",scale_w,scale_h);
+#endif
 	art_rgb_pixbuf_affine(rgb,
 			      0,0,scale_w,scale_h,scale_w*3,
 			      pb->art_pixbuf,affine,
 			      ART_FILTER_NEAREST,NULL);
+	p = gdk_pixmap_new(w->window, scale_w,scale_h,
+			   gtk_widget_get_visual(GTK_WIDGET(w))->depth);
+	gc = gdk_gc_new(p);
 	gdk_draw_rgb_image(p,gc,0,0,
 			   scale_w,scale_h,
 			   GDK_RGB_DITHER_NORMAL,
 			   rgb, scale_w*3);
 	g_free(rgb);
-
 	gdk_gc_destroy(gc);
+
 	return p;
 }
 
@@ -1428,6 +1486,7 @@ static void
 panel_resize_pixmap(PanelWidget *panel)
 {
 	int w, h;
+	int pw, ph;
 
 	g_return_if_fail(panel!=NULL);
 	g_return_if_fail(IS_PANEL_WIDGET(panel));
@@ -1439,31 +1498,49 @@ panel_resize_pixmap(PanelWidget *panel)
 	panel->backpixmap = NULL;
 
 	panel->scale_w = w = panel->backpix->art_pixbuf->width;
-	panel->scale_w = h = panel->backpix->art_pixbuf->height;
-
-	switch (panel->orient) {
-	case PANEL_HORIZONTAL:
-		panel->scale_w = w * panel->thick / h;
-		panel->scale_h = panel->thick;
-		break;
-
-	case PANEL_VERTICAL:
-		panel->scale_w = panel->thick;
-		panel->scale_h = h * panel->thick / w;
-		break;
-
-	default:
-		g_assert_not_reached ();
-	}
-
-	panel->backpixmap = get_pixmap_from_pixbuf(GTK_WIDGET(panel),
-						   panel->backpix,
-						   panel->scale_w,
-						   panel->scale_h);
+	panel->scale_h = h = panel->backpix->art_pixbuf->height;
 	
-	kill_cache_on_all_buttons(panel, FALSE);
+	pw = GTK_WIDGET(panel)->allocation.width;
+	ph = GTK_WIDGET(panel)->allocation.height;
 
-	panel_widget_draw_all(panel,NULL);
+	if(panel->fit_pixmap_bg) {
+		switch (panel->orient) {
+		case PANEL_HORIZONTAL:
+			panel->scale_w = w * ph / h;
+			panel->scale_h = ph;
+			break;
+
+		case PANEL_VERTICAL:
+			if(panel->rotate_pixmap_bg) {
+				panel->scale_w = w * pw / h;
+				panel->scale_h = pw;
+			} else {
+				panel->scale_w = pw;
+				panel->scale_h = h * pw / w;
+			}
+			break;
+
+		default:
+			g_assert_not_reached ();
+		}
+	} else if(panel->strech_pixmap_bg) {
+		if(panel->orient == PANEL_VERTICAL &&
+		   panel->rotate_pixmap_bg) {
+			panel->scale_w = ph;
+			panel->scale_h = pw;
+		} else {
+			panel->scale_w = pw;
+			panel->scale_h = ph;
+		}
+	}
+	
+	panel->backpixmap = get_pixmap_from_pixbuf(
+					GTK_WIDGET(panel),
+					panel->backpix,
+					panel->scale_w,
+					panel->scale_h,
+					panel->orient == PANEL_VERTICAL &&
+					  panel->rotate_pixmap_bg);
 }
 
 static int
@@ -1491,40 +1568,9 @@ panel_try_to_set_pixmap (PanelWidget *panel, char *pixmap)
 	panel->backpix = gdk_pixbuf_new_from_file (pixmap);
 	if (!panel->backpix)
 		return FALSE;
-	
-	if (panel->fit_pixmap_bg) {
-		int w, h;
 
-		panel->scale_w = w = panel->backpix->art_pixbuf->width;
-		panel->scale_h = h = panel->backpix->art_pixbuf->height;
-
-		switch (panel->orient) {
-		case PANEL_HORIZONTAL:
-			panel->scale_w = w * panel->thick / h;
-			panel->scale_h = panel->thick;
-			break;
-
-		case PANEL_VERTICAL:
-			panel->scale_w = panel->thick;
-			panel->scale_h = h * panel->thick / w;
-			break;
-
-		default:
-			g_assert_not_reached ();
-		}
-	
-		panel->backpixmap =
-			get_pixmap_from_pixbuf(GTK_WIDGET(panel), 
-					       panel->backpix,
-					       panel->scale_w,
-					       panel->scale_h);
-	} else {
-		panel->backpixmap =
-			get_pixmap_from_pixbuf(GTK_WIDGET(panel),
-					       panel->backpix,
-					       panel->backpix->art_pixbuf->width,
-					       panel->backpix->art_pixbuf->height);
-	}
+	panel->scale_w = panel->backpix->art_pixbuf->width;
+	panel->scale_h = panel->backpix->art_pixbuf->height;
 
 	return TRUE;
 }
@@ -1542,11 +1588,13 @@ panel_widget_realize(GtkWidget *w, gpointer data)
 	if(panel->back_type == PANEL_BACK_PIXMAP) {
 		if (!panel_try_to_set_pixmap (panel, panel->back_pixmap))
 			panel->back_type = PANEL_BACK_NONE;
-		panel_widget_draw_all(panel,NULL);
 	} else if(panel->back_type == PANEL_BACK_COLOR) {
 		panel_try_to_set_back_color(panel, &panel->back_color);
 	}
 
+#ifdef PANEL_DEBUG
+	puts("PANEL_WIDGET_REALIZE");
+#endif
 	panel_widget_draw_all(panel,NULL);
 }
 
@@ -1564,11 +1612,6 @@ panel_widget_destroy(GtkWidget *w, gpointer data)
 	
 	/*remove from panels list*/
 	panels = g_slist_remove(panels,w);
-
-	if(panel->pixmap_resize_timeout)
-		gtk_timeout_remove(panel->pixmap_resize_timeout);
-	if(panel->pixmap_resize_timeout_top)
-		gtk_timeout_remove(panel->pixmap_resize_timeout_top);
 }
 
 static void
@@ -1642,9 +1685,7 @@ panel_widget_init (PanelWidget *panel)
 	
 	panel->back_type =PANEL_BACK_NONE;
 	panel->fit_pixmap_bg = FALSE;
-	panel->pixmap_resize_timeout = 0;
-	panel->pixmap_resize_timeout_top = 0;
-	panel->pixmap_resize_pending = FALSE;
+	panel->strech_pixmap_bg = FALSE;
 	panel->back_pixmap = NULL;
 	panel->back_color.red = 0;
 	panel->back_color.green = 0;
@@ -1660,6 +1701,7 @@ panel_widget_init (PanelWidget *panel)
 	panel->drop_widget = GTK_WIDGET(panel);
 	panel->backpix = NULL;
 	panel->backpixmap = NULL;
+	panel->inhibit_draw = FALSE;
 
 	gtk_signal_connect(GTK_OBJECT(panel),
 			   "destroy",
@@ -1680,10 +1722,12 @@ panel_widget_init (PanelWidget *panel)
 GtkWidget *
 panel_widget_new (int packed,
 		  PanelOrientation orient,
-		  PanelSizeType sz,
+		  int sz,
 		  PanelBackType back_type,
 		  char *back_pixmap,
-		  int fit_pixmap_bg,
+		  gboolean fit_pixmap_bg,
+		  gboolean strech_pixmap_bg,
+		  gboolean rotate_pixmap_bg,
 		  GdkColor *back_color)
 {
 	PanelWidget *panel;
@@ -1699,6 +1743,8 @@ panel_widget_new (int packed,
 	panel->back_type = back_type;
 
 	panel->fit_pixmap_bg = fit_pixmap_bg;
+	panel->strech_pixmap_bg = strech_pixmap_bg;
+	panel->rotate_pixmap_bg = rotate_pixmap_bg;
 	panel->back_pixmap = g_strdup (back_pixmap ? back_pixmap : "");
 	
 	if(back_color)
@@ -1713,6 +1759,10 @@ panel_widget_new (int packed,
 	panel->orient = orient;
 	panel->sz = sz;
 
+#ifdef PANEL_DEBUG
+	printf("GOT SIZE OF %d\n",sz);
+#endif
+
 	panel->packed = packed;
 	if(packed)
 		panel->size = 0;
@@ -2375,7 +2425,7 @@ panel_widget_add_forbidden(PanelWidget *panel)
 }
 
 int
-panel_widget_add_full (PanelWidget *panel, GtkWidget *applet, int pos, int bind_lower_events, int insert_at_pos)
+panel_widget_add_full (PanelWidget *panel, GtkWidget *applet, int pos, gboolean bind_lower_events, gboolean insert_at_pos)
 {
 	AppletData *ad = NULL;
 
@@ -2394,15 +2444,22 @@ panel_widget_add_full (PanelWidget *panel, GtkWidget *applet, int pos, int bind_
 		pos = pw_applet_padding;
 	
 	if(!insert_at_pos) {
-		if(pw_movement_type != PANEL_FREE_MOVE ||
-		   panel->packed) {
+		if(panel->packed) {
 			if(get_applet_list_pos(panel,pos)) 
 				/*this is a slight hack so that this applet
 				  is inserted AFTER an applet with this pos
 				  number*/
 				pos++;
-		} else
-			pos = panel_widget_find_empty_pos(panel,pos);
+		} else {
+			int newpos = panel_widget_find_empty_pos(panel,pos);
+			if(newpos>=0)
+				pos = newpos;
+			else if(get_applet_list_pos(panel,pos)) 
+				/*this is a slight hack so that this applet
+				  is inserted AFTER an applet with this pos
+				  number*/
+				pos++;
+		}
 	}
 
 	if(pos==-1) return -1;
@@ -2555,14 +2612,16 @@ panel_widget_get_pos(PanelWidget *panel, GtkWidget *applet)
 void
 panel_widget_change_params(PanelWidget *panel,
 			   PanelOrientation orient,
-			   PanelSizeType sz,
+			   int sz,
 			   PanelBackType back_type,
 			   char *pixmap,
-			   int fit_pixmap_bg,
+			   gboolean fit_pixmap_bg,
+			   gboolean strech_pixmap_bg,
+			   gboolean rotate_pixmap_bg,
 			   GdkColor *back_color)
 {
 	PanelOrientation oldorient;
-	PanelSizeType oldsz;
+	int oldsz;
 	int change_back = FALSE;
 
 	g_return_if_fail(panel!=NULL);
@@ -2574,6 +2633,10 @@ panel_widget_change_params(PanelWidget *panel,
 
 	oldsz = panel->sz;
 	panel->sz = sz;
+	
+#ifdef PANEL_DEBUG
+	printf("GOT SIZE OF %d\n",sz);
+#endif
 
 	kill_cache_on_all_buttons(panel, TRUE);
 
@@ -2605,20 +2668,23 @@ panel_widget_change_params(PanelWidget *panel,
 	}
 
 	/*clearly a signal should be sent*/
-	if(panel->back_type != back_type)
+	if(panel->back_type != back_type ||
+	   panel->fit_pixmap_bg != fit_pixmap_bg ||
+	   panel->strech_pixmap_bg != strech_pixmap_bg ||
+	   panel->rotate_pixmap_bg != rotate_pixmap_bg)
 		change_back = TRUE;
 	
 	/*this bit is not optimal, it allways sets the pixmap etc etc ...
 	  but this function isn't called too often*/
 	panel->back_type = back_type;
 	panel->fit_pixmap_bg = fit_pixmap_bg;
+	panel->strech_pixmap_bg = strech_pixmap_bg;
+	panel->rotate_pixmap_bg = rotate_pixmap_bg;
 	if(back_type == PANEL_BACK_PIXMAP) {
 		panel_try_to_set_pixmap (panel, panel->back_pixmap);
-		panel_widget_draw_all(panel,NULL);
+		/* we will queue resize and redraw anyhow */
 	} else if(back_type == PANEL_BACK_COLOR) {
 		panel_try_to_set_back_color(panel, &panel->back_color);
-	} else {
-		panel_try_to_set_default_back(panel);
 	}
 
 	/* let the applets know we changed the background */
@@ -2629,6 +2695,8 @@ panel_widget_change_params(PanelWidget *panel,
 				panel->back_pixmap,
 				&panel->back_color);
 	}
+	/* inhibit draws until we resize */
+	panel->inhibit_draw = TRUE;
 	gtk_widget_queue_resize(GTK_WIDGET(panel));
 }
 
@@ -2642,6 +2710,8 @@ panel_widget_change_orient(PanelWidget *panel,
 				   panel->back_type,
 				   panel->back_pixmap,
 				   panel->fit_pixmap_bg,
+				   panel->strech_pixmap_bg,
+				   panel->rotate_pixmap_bg,
 				   &panel->back_color);
 }
 
@@ -2654,7 +2724,7 @@ panel_widget_change_global(int explicit_step,
 			   int minimized_size,
 			   int minimize_delay,
 			   PanelMovementType move_type,
-			   int disable_animations,
+			   gboolean disable_animations,
 			   int applet_padding)
 {
 	if(explicit_step>0)
@@ -2679,17 +2749,67 @@ panel_widget_change_global(int explicit_step,
 	}
 }
 
-int
-panel_widget_get_pixel_size(PanelWidget *panel)
+/* when we get color_only, we also optionally set r, g, b to the
+   color and w, and h to the area if the background is one color
+   only, otherwise normally return an rgb and set r, g, b to -1 */
+void
+panel_widget_get_applet_rgb_bg(PanelWidget *panel,
+			       GtkWidget *applet,
+			       guchar **rgb,
+			       int *w, int *h,
+			       int *rowstride,
+			       gboolean color_only,
+			       int *r, int *g, int *b)
 {
-	g_return_val_if_fail(panel!=NULL,48);
-	g_return_val_if_fail(IS_PANEL_WIDGET(panel),48);
+	GtkWidget *widget;
 
-	switch(panel->sz) {
-	case SIZE_TINY: return 24;
-	case SIZE_STANDARD: return 48;
-	case SIZE_LARGE: return 64;
-	case SIZE_HUGE: return 80;
-	default: return 48;
+	GdkPixbuf *pb = NULL;
+
+	int scale_w = 0,scale_h = 0;
+	int rotate = FALSE;
+	
+	*rgb = NULL;
+	*w = *h = *rowstride = 0;
+	*r = *g = *b = -1;
+
+	g_return_if_fail(panel!=NULL);
+	g_return_if_fail(IS_PANEL_WIDGET(panel));
+	g_return_if_fail(applet!=NULL);
+	g_return_if_fail(GTK_IS_WIDGET(applet));
+
+	widget = GTK_WIDGET(panel);
+
+	if(!GTK_WIDGET_DRAWABLE(widget) ||
+	   widget->allocation.width <= 0 ||
+	   widget->allocation.height <= 0)
+		return;
+
+	setup_background(panel, &pb, &scale_w, &scale_h, &rotate);
+	
+	*w = applet->allocation.width;
+	*h = applet->allocation.height;
+	
+	if(color_only && !pb) {
+		if(panel->back_type != PANEL_BACK_COLOR) {
+			GtkWidget *widget = GTK_WIDGET(panel);
+			/* convert to 8 bit per channel */
+			*r = widget->style->bg[GTK_WIDGET_STATE(widget)].red>>8;
+			*g = widget->style->bg[GTK_WIDGET_STATE(widget)].green>>8;
+			*b = widget->style->bg[GTK_WIDGET_STATE(widget)].blue>>8;
+		} else {
+			/* convert to 8 bit per channel */
+			*r = panel->back_color.red>>8;
+			*g = panel->back_color.green>>8;
+			*b = panel->back_color.blue>>8;
+		}
+		return;
 	}
+	*rowstride = applet->allocation.width*3;
+
+	*rgb = g_new0(guchar, (*h)*(*rowstride));
+
+	make_background(panel, *rgb,
+			applet->allocation.x,
+			applet->allocation.y,
+			*w,*h, pb, scale_w, scale_h, rotate);
 }
