@@ -62,13 +62,15 @@ struct _PanelWidget
 
 	int			size;
 	PanelOrientation	orient;
-	PanelSizeType		sz;
-	int			fit_pixmap_bg;
-	int			pixmap_resize_timeout;
-	int			pixmap_resize_timeout_top;
-	int			pixmap_resize_pending;
+	int			sz;
+	gboolean		fit_pixmap_bg;  /* fit pixmap while keeping
+						   ratio*/
+	gboolean		strech_pixmap_bg; /*strech pixmap to the size
+						    of the panel */
+	gboolean		rotate_pixmap_bg; /*rotate pixmap on vertical
+						    panels */
 
-	int			packed;
+	gboolean		packed;
 
 	AppletData		*currently_dragged_applet;
 
@@ -77,6 +79,7 @@ struct _PanelWidget
 	PanelBackType		back_type;
 	char                    *back_pixmap;
 	GdkColor		back_color;
+	gboolean		inhibit_draw;
 	
 	GtkWidget		*master_widget;
 	
@@ -103,7 +106,7 @@ struct _PanelWidgetClass
 	void (* orient_change) (PanelWidget *panel,
 				PanelOrientation orient);
 	void (* size_change) (PanelWidget *panel,
-			      PanelSizeType sz);
+			      int sz);
 	void (* applet_move) (PanelWidget *panel,
 			      GtkWidget *applet);
 	void (* applet_added) (PanelWidget *panel,
@@ -114,23 +117,27 @@ struct _PanelWidgetClass
 			      PanelBackType type,
 			      char *pixmap,
 			      GdkColor *color);
+	void (* applet_draw) (PanelWidget *panel,
+			      GtkWidget *applet);
 };
 
 guint		panel_widget_get_type		(void);
-GtkWidget *	panel_widget_new		(int packed,
+GtkWidget *	panel_widget_new		(gboolean packed,
 						 PanelOrientation orient,
-						 PanelSizeType sz,
+						 int sz,
 						 PanelBackType back_type,
 						 char *back_pixmap,
-						 int fit_pixmap_bg,
+						 gboolean fit_pixmap_bg,
+						 gboolean strech_pixmap_bg,
+						 gboolean rotate_pixmap_bg,
 						 GdkColor *back_color);
 /*add an applet to the panel, preferably at position pos, if insert_at_pos
   is on, we REALLY want to insert at the pos given by pos*/
 int		panel_widget_add_full		(PanelWidget *panel,
 						 GtkWidget *applet,
 						 int pos,
-						 int bind_lower_events,
-						 int insert_at_pos);
+						 gboolean bind_lower_events,
+						 gboolean insert_at_pos);
 #define panel_widget_add(panel,applet,pos) (panel_widget_add_full(panel,applet,pos,TRUE,FALSE))
 
 /*needs to be called for drawers after add*/
@@ -169,10 +176,12 @@ int		panel_widget_applet_move_to_cursor(PanelWidget *panel);
 /* changing parameters */
 void		panel_widget_change_params	(PanelWidget *panel,
 						 PanelOrientation orient,
-						 PanelSizeType sz,
+						 int sz,
 						 PanelBackType back_type,
 						 char *pixmap_name,
-						 int fit_pixmap_bg,
+						 gboolean fit_pixmap_bg,
+						 gboolean strech_pixmap_bg,
+						 gboolean rotate_pixmap_bg,
 						 GdkColor *back_color);
 
 /* changing parameters (orient only) */
@@ -186,7 +195,7 @@ void		panel_widget_change_global	(int explicit_step,
 						 int minimized_size,
 						 int minimize_delay,
 						 PanelMovementType move_type,
-						 int disable_animations,
+						 gboolean disable_animations,
 						 int applet_padding);
 
 void		panel_widget_set_back_pixmap	(PanelWidget *panel,
@@ -215,11 +224,20 @@ int		panel_widget_get_cursorloc	(PanelWidget *panel);
 int		panel_widget_is_cursor		(PanelWidget *panel,
 						 int overlap);
 
-int		panel_widget_get_pixel_size	(PanelWidget *panel);
+/* when we get color_only, we also optionally set r, g, b to the
+   color and w, and h to the area if the background is one color
+   only, otherwise normally return an rgb and set r, g, b to -1 */
+void panel_widget_get_applet_rgb_bg(PanelWidget *panel,
+				    GtkWidget *applet,
+				    guchar **rgb,
+				    int *w, int *h,
+				    int *rowstride,
+				    gboolean color_only,
+				    int *r, int *g, int *b);
 
 /*extern GSList *panels;*/
 
-extern int panel_applet_in_drag;
+extern gboolean panel_applet_in_drag;
 
 
 END_GNOME_DECLS
