@@ -48,6 +48,11 @@ extern GtkTooltips *panel_tooltips;
   opened that menu whilethe user is looking over the choices*/
 extern PanelWidget *current_panel;
 
+extern char *kde_menudir;
+extern char *kde_icondir;
+extern char *kde_mini_icondir;
+
+
 /*the types of stuff we accept*/
 
 enum {
@@ -181,17 +186,25 @@ orientation_change(AppletInfo *info, PanelWidget *panel)
 {
 	if(info->type == APPLET_EXTERN) {
 		Extern *ext = info->data;
+		int orient = get_applet_orient(panel);
 		g_assert(ext);
 		/*ingore this until we get an ior*/
-		if(ext->applet) {
+		if(ext->applet && ext->orient != orient) {
 			CORBA_Environment ev;
+
 			CORBA_exception_init(&ev);
 			GNOME_Applet_change_orient(ext->applet,
-						   get_applet_orient(panel),
+						   orient,
 						   &ev);
 			if(ev._major)
 				panel_clean_applet(ext->info);
 			CORBA_exception_free(&ev);
+
+			/* we have now sent this orientation thus we
+			   save it and don't send it again unless it
+			   changes */
+			ext->orient = orient;
+
 		}
 	} else if(info->type == APPLET_MENU) {
 		Menu *menu = info->data;
@@ -213,8 +226,10 @@ orientation_change(AppletInfo *info, PanelWidget *panel)
 			set_swallow_applet_orient(swallow,SWALLOW_HORIZONTAL);
 	} else if(info->type == APPLET_STATUS) {
 		StatusApplet *status = info->data;
-		status->orient = panel->orient;
-		status_applet_update(status);
+		if(status->orient != panel->orient) {
+			status->orient = panel->orient;
+			status_applet_update(status);
+		}
 	}
 }
 
@@ -329,7 +344,7 @@ size_change_foreach(GtkWidget *w, gpointer data)
 
 static void
 panel_size_change(GtkWidget *widget,
-		  PanelSizeType sz,
+		  int sz,
 		  gpointer data)
 {
 	gtk_container_foreach(GTK_CONTAINER(widget), size_change_foreach,
@@ -561,11 +576,11 @@ static void
 panel_destroy(GtkWidget *widget, gpointer data)
 {
 	PanelData *pd = gtk_object_get_user_data(GTK_OBJECT(widget));
-
+	PanelWidget *panel = PANEL_WIDGET(BASEP_WIDGET(widget)->panel);
+		
 	kill_config_dialog(widget);
 
 	if(IS_DRAWER_WIDGET(widget)) {
-		PanelWidget *panel = PANEL_WIDGET(BASEP_WIDGET(widget)->panel);
 		if(panel->master_widget) {
 			AppletInfo *info = gtk_object_get_data(GTK_OBJECT(panel->master_widget),
 							       "applet_info");
@@ -579,11 +594,15 @@ panel_destroy(GtkWidget *widget, gpointer data)
 		base_panels--;
 	}
 
+	if (current_panel == panel)
+		current_panel = NULL;
+
 	if(pd->menu)
 		gtk_widget_destroy(pd->menu);
 	
 	panel_list = g_slist_remove(panel_list,pd);
 	g_free(pd);
+	
 }
 
 static void
@@ -592,6 +611,17 @@ panel_applet_move(GtkWidget *panel,GtkWidget *widget, gpointer data)
 	applets_to_sync = TRUE;
 }
 
+static void
+panel_applet_draw(GtkWidget *panel, GtkWidget *widget, gpointer data)
+{
+	AppletInfo *info = gtk_object_get_data(GTK_OBJECT(widget), "applet_info");
+
+	g_return_if_fail(info!=NULL);
+
+	if(info->type == APPLET_EXTERN)
+		extern_send_draw(info->data);
+}
+
 static GtkWidget *
 panel_menu_get(PanelData *pd)
 {
@@ -604,6 +634,27 @@ panel_menu_get(PanelData *pd)
 	return pd->menu;
 }
 
+GtkWidget *
+make_popup_panel_menu (void)
+{
+	BasePWidget *basep;
+	PanelData *pd;
+	GtkWidget *menu;
+
+	if (!current_panel) {
+	        basep = BASEP_WIDGET (((PanelData *)panel_list->data)->panel);			
+		current_panel = PANEL_WIDGET (basep->panel);
+	} else
+		basep = BASEP_WIDGET (current_panel->panel_parent);
+	
+	pd = gtk_object_get_user_data (GTK_OBJECT (basep));
+	menu = panel_menu_get (pd);
+	gtk_object_set_data (GTK_OBJECT (menu), "menu_panel",
+			     basep->panel);
+	pd->menu_age = 0;
+	return menu;
+}
+
 static int
 panel_event(GtkWidget *widget, GdkEvent *event, PanelData *pd)
 {
@@ -615,27 +666,19 @@ panel_event(GtkWidget *widget, GdkEvent *event, PanelData *pd)
 		switch(bevent->button) {
 		case 3: /* fall through */
 			if(!panel_applet_in_drag) {
-				GtkWidget *panel_menu;
-				GtkWidget *rem;
+				GtkWidget *menu;
 				current_panel =
 					PANEL_WIDGET(basep->panel);
-				panel_menu = panel_menu_get(pd);
-				rem = gtk_object_get_data(GTK_OBJECT(panel_menu),
-							  "remove_item");
-				
-				if (!IS_DRAWER_WIDGET(basep))
-					gtk_widget_set_sensitive(rem, base_panels > 1);
-
-				basep->autohide_inhibit = TRUE;
-				basep_widget_queue_autohide(basep);
-
-				pd->menu_age = 0;
-				
-				gtk_menu_popup(GTK_MENU(panel_menu), NULL, NULL,
-					       global_config.off_panel_popups?
-					        panel_menu_position:NULL,
-					       widget, bevent->button,
-					       bevent->time);
+				menu = make_popup_panel_menu ();
+				BASEP_WIDGET(basep)->autohide_inhibit 
+					= TRUE;
+				basep_widget_autohide (
+					BASEP_WIDGET (basep));
+				gtk_menu_popup (GTK_MENU (menu), NULL, NULL, 
+						global_config.off_panel_popups
+						? panel_menu_position : NULL,
+						widget, bevent->button,
+						bevent->time);
 				return TRUE;
 			}
 			break;
@@ -732,6 +775,167 @@ panel_sub_event_handler(GtkWidget *widget, GdkEvent *event, gpointer data)
 	return FALSE;
 }
 
+static gchar *
+extract_filename (const gchar* uri)
+{
+	/* file uri with a hostname */
+	if (strncmp(uri, "file://", strlen("file://"))==0) {
+		char *hostname = g_strdup(&uri[strlen("file://")]);
+		char *p = strchr(hostname,'/');
+		char *path;
+		char localhostname[1024];
+		/* if we can't find the '/' this uri is bad */
+		if(!p) {
+			g_free(hostname);
+			return NULL;
+		}
+		/* if no hostname */
+		if(p==hostname)
+			return hostname;
+
+		path = g_strdup(p);
+		*p = '\0';
+
+		/* if really local */
+		if(g_strcasecmp(hostname,"localhost")==0) {
+			g_free(hostname);
+			return path;
+		}
+
+		/* ok get the hostname */
+		if(gethostname(localhostname,
+			       sizeof(localhostname)) < 0) {
+			strcpy(localhostname,"");
+		}
+
+		/* if really local */
+		if(localhostname[0] &&
+		   g_strcasecmp(hostname,localhostname)==0) {
+			g_free(hostname);
+			return path;
+		}
+		
+		g_free(hostname);
+		g_free(path);
+		return NULL;
+
+	/* if the file doesn't have the //, we take it containing 
+	   a local path */
+	} else if (strncmp(uri, "file:", strlen("file:"))==0) {
+		const char *path = &uri[strlen("file:")];
+		/* if empty bad */
+		if(!*path) return NULL;
+		return g_strdup(path);
+	}
+	return NULL;
+}
+
+static void
+drop_url(PanelWidget *panel, int pos, char *url)
+{
+	char *p = g_strdup_printf("Open URL: %s",url);
+	load_launcher_applet_from_info_url(url, p, url, "netscape.png",
+					   panel, pos, TRUE);
+	g_free(p);
+}
+
+static void
+drop_menu(PanelWidget *panel, int pos, char *dir)
+{
+	int flags = MAIN_MENU_SYSTEM|MAIN_MENU_USER;
+
+	/*guess redhat menus*/
+	if(g_file_exists("/etc/X11/wmconfig"))
+		flags |= MAIN_MENU_REDHAT_SUB;
+	/* Guess KDE menus */
+	if(g_file_exists(kde_menudir))
+		flags |= MAIN_MENU_KDE_SUB;
+	load_menu_applet(dir, flags, panel, pos, TRUE);
+}
+
+static void
+drop_urilist(PanelWidget *panel, int pos, char *urilist)
+{
+	GList *li, *files;
+	struct stat s;
+
+	files = gnome_uri_list_extract_uris(urilist);
+
+	for(li = files; li; li = g_list_next(li)) {
+		const char *mimetype;
+		char *filename;
+
+		if(strncmp(li->data,"http:",strlen("http:"))==0 ||
+		   strncmp(li->data,"https:",strlen("https:"))==0 ||
+		   strncmp(li->data,"ftp:",strlen("ftp:"))==0 ||
+		   strncmp(li->data,"gopher:",strlen("gopher:"))==0) {
+			drop_url(panel,pos,li->data);
+			continue;
+		}
+
+		filename = extract_filename(li->data);
+		if(!filename)
+			continue;
+
+		if(stat(filename, &s) != 0) {
+			g_free(filename);
+			continue;
+		}
+
+		mimetype = gnome_mime_type(filename);
+
+		if(mimetype && !strncmp(mimetype, "image", sizeof("image")-1))
+			panel_widget_set_back_pixmap (panel, filename);
+		else if(mimetype
+			&& !strcmp(mimetype, "application/x-gnome-app-info"))
+			load_launcher_applet(filename, panel, pos, TRUE);
+		else if(S_ISDIR(s.st_mode)) {
+			drop_menu(panel, pos, filename);
+		} else if(S_IEXEC & s.st_mode) /*executable?*/
+			ask_about_launcher(filename,panel,pos,TRUE);
+		g_free(filename);
+	}
+
+	gnome_uri_list_free_strings (files);
+}
+
+static void
+drop_internal_applet(PanelWidget *panel, int pos, char *applet_type)
+{
+	if(!applet_type)
+		return;
+	if(strcmp(applet_type,"MENU:MAIN")==0) {
+		drop_menu(panel, pos, NULL);
+	} else if(strcmp(applet_type,"DRAWER:NEW")==0) {
+		load_drawer_applet(-1, NULL, NULL, panel, pos, TRUE);
+	} else if(strcmp(applet_type,"LOGOUT:NEW")==0) {
+		load_logout_applet(panel, pos, TRUE);
+	} else if(strcmp(applet_type,"LOCK:NEW")==0) {
+		load_lock_applet(panel, pos, TRUE);
+	} else if(strcmp(applet_type,"SWALLOW:ASK")==0) {
+		ask_about_swallowing(panel, pos, TRUE);
+	} else if(strcmp(applet_type,"LAUNCHER:ASK")==0) {
+		ask_about_launcher(NULL, panel, pos, TRUE);
+	} else if(strcmp(applet_type,"STATUS:TRY")==0) {
+		load_status_applet(panel, pos, TRUE);
+	}
+}
+
+static void
+drop_color(PanelWidget *panel, int pos, guint16 *dropped)
+{
+	GdkColor c;
+
+	if(!dropped) return;
+
+	c.red = dropped[0];
+	c.green = dropped[1];
+	c.blue = dropped[2];
+	c.pixel = 0;
+
+	panel_widget_set_back_color(panel, &c);
+}
+
 static void
 panel_widget_dnd_drop_internal (GtkWidget	 *widget,
 				GdkDragContext   *context,
@@ -760,144 +964,28 @@ panel_widget_dnd_drop_internal (GtkWidget	 *widget,
 		pos = panel->size;
 
 	switch (info) {
-	case TARGET_URL: {
-		GList *ltmp, *files;
-		struct stat s;
-
-		files =
-		  gnome_uri_list_extract_filenames(selection_data->data);
-
-		for(ltmp = files; ltmp; ltmp = g_list_next(ltmp)) {
-		  const char *mimetype;
-
-		  mimetype = gnome_mime_type(ltmp->data);
-
-		  if(mimetype &&
-		     (!strcmp(mimetype,"x-url/http") ||
-		      !strcmp(mimetype,"x-url/ftp"))) {
-			  char *exec[3] = {
-			  	"gnome-moz-remote",
-				NULL,
-				NULL
-			  };
-			  char *p;
-
-		          exec[1] = ltmp->data;
-			  p = g_strdup_printf("Open URL: %s",
-					      (char *)ltmp->data);
-			  load_launcher_applet_from_info(ltmp->data,
-							 p,exec,2,
-							 "netscape.png",
-							 panel,pos);
-			  g_free(p);
-		  }
-
-		  if(stat(ltmp->data, &s) != 0) {
-			  continue;
-		  }
-
-		  if(mimetype && !strncmp(mimetype, "image", sizeof("image")-1))
-		    panel_widget_set_back_pixmap (panel, ltmp->data);
-		  else if(mimetype
-			  && !strcmp(mimetype, "application/x-gnome-app-info"))
-		    load_launcher_applet(ltmp->data, panel, pos);
-		  else if(S_ISDIR(s.st_mode)) {
-			  int flags = MAIN_MENU_SYSTEM|MAIN_MENU_USER;
-
-			  /*guess redhat menus*/
-			  if(g_file_exists("/etc/X11/wmconfig"))
-				  flags |= MAIN_MENU_REDHAT|MAIN_MENU_REDHAT_SUB;
-			  /* Guess KDE menus */
-			  if(g_file_exists(KDE_MENUDIR))
-				  flags |= MAIN_MENU_KDE|MAIN_MENU_KDE_SUB;
-			  load_menu_applet(ltmp->data, flags, panel, pos);
-		  } else if(S_IEXEC & s.st_mode) /*executable?*/
-		    ask_about_launcher(ltmp->data,panel,pos);
-		}
-		
-		gnome_uri_list_free_strings (files);
+	case TARGET_URL:
+		drop_urilist(panel, pos, selection_data->data);
 		break;
-	}
-	case TARGET_NETSCAPE_URL: {
-		char *exec[3] = {
-			"gnome-moz-remote",
-			NULL,
-			NULL
-		};
-		char *p;
-		
-         	exec[1] = selection_data->data;
-		p = g_strdup_printf("Open URL: %s",selection_data->data);
-		load_launcher_applet_from_info(selection_data->data,p,exec,2,
-					       "netscape.png",panel,pos);
-		g_free(p);
+	case TARGET_NETSCAPE_URL:
+		drop_url(panel, pos, selection_data->data);
 		break;
-	}
-	case TARGET_COLOR: {
-		guint16 *dropped;
-		GdkColor c;
-
-		dropped = (guint16 *)selection_data->data;
-
-		c.red = dropped[0];
-		c.green = dropped[1];
-		c.blue = dropped[2];
-		c.pixel = 0;
-
-		panel_widget_set_back_color(panel, &c);
+	case TARGET_COLOR:
+		drop_color(panel, pos, (guint16 *)selection_data->data);
 		break;
-	}
-	case TARGET_DIRECTORY: {
-		int flags = MAIN_MENU_SYSTEM|MAIN_MENU_USER;
-
-		/*guess redhat menus*/
-		if(g_file_exists("/etc/X11/wmconfig"))
-			flags |= MAIN_MENU_REDHAT|MAIN_MENU_REDHAT_SUB;
-		/* Guess KDE menus */
-		if(g_file_exists(KDE_MENUDIR))
-				  flags |= MAIN_MENU_KDE|MAIN_MENU_KDE_SUB;
-		load_menu_applet ((char *)selection_data->data,flags,
-				  panel, pos);
+	case TARGET_DIRECTORY:
+		drop_menu(panel, pos, selection_data->data);
 		break;
-	}
-	case TARGET_APPLET: {
-		char *goad_id = (char *)selection_data->data;
-		if(!goad_id)
+	case TARGET_APPLET:
+		if(!selection_data->data)
 			return;
-		load_extern_applet(goad_id,NULL,panel,pos,FALSE);
+		load_extern_applet((char *)selection_data->data, NULL,
+				   panel, pos, TRUE, FALSE);
 		break;
-	}
-	case TARGET_APPLET_INTERNAL: {
-		char *applet_type = (char *)selection_data->data;
-		if(!applet_type)
-			return;
-		if(strcmp(applet_type,"MENU:MAIN")==0) {
-			int flags = MAIN_MENU_SYSTEM|MAIN_MENU_USER;
-
-			/*guess redhat menus*/
-			if(g_file_exists("/etc/X11/wmconfig"))
-				flags |= MAIN_MENU_REDHAT|MAIN_MENU_REDHAT_SUB;
-			/* Guess KDE menus */
-			if(g_file_exists(KDE_MENUDIR))
-				flags |= MAIN_MENU_KDE|MAIN_MENU_KDE_SUB;
-
-			load_menu_applet(NULL,flags, panel, pos);
-		} else if(strcmp(applet_type,"DRAWER:NEW")==0) {
-			load_drawer_applet(-1,NULL,NULL, panel, pos);
-		} else if(strcmp(applet_type,"LOGOUT:NEW")==0) {
-			load_logout_applet(panel, pos);
-		} else if(strcmp(applet_type,"LOCK:NEW")==0) {
-			load_lock_applet(panel, pos);
-		} else if(strcmp(applet_type,"SWALLOW:ASK")==0) {
-			ask_about_swallowing(panel,pos);
-		} else if(strcmp(applet_type,"LAUNCHER:ASK")==0) {
-			ask_about_launcher(NULL,panel,pos);
-		} else if(strcmp(applet_type,"STATUS:TRY")==0) {
-			load_status_applet(panel,pos);
-		}
+	case TARGET_APPLET_INTERNAL:
+		drop_internal_applet(panel, pos, selection_data->data);
 		break;
 	}
-	}
 }
 
 static void
@@ -915,6 +1003,10 @@ panel_widget_setup(PanelWidget *panel)
 			   "applet_move",
 			   GTK_SIGNAL_FUNC(panel_applet_move),
 			   NULL);
+	gtk_signal_connect(GTK_OBJECT(panel),
+			   "applet_draw",
+			   GTK_SIGNAL_FUNC(panel_applet_draw),
+			   NULL);
 	gtk_signal_connect(GTK_OBJECT(panel),
 			   "back_change",
 			   GTK_SIGNAL_FUNC(panel_back_change),
@@ -961,6 +1053,17 @@ basep_pos_connect_signals (BasePWidget *basep)
 	}
 }
 
+static void
+floating_size_alloc(BasePWidget *basep, GtkAllocation *alloc, gpointer data)
+{
+	if(!GTK_WIDGET_REALIZED(basep))
+		return;
+
+	gtk_container_foreach(GTK_CONTAINER(basep->panel),
+			      orient_change_foreach,
+			      basep->panel);
+}
+
 void
 panel_setup(GtkWidget *panelw)
 {
@@ -1051,6 +1154,11 @@ panel_setup(GtkWidget *panelw)
 		gtk_signal_connect_after(GTK_OBJECT(panelw), "realize",
 					 GTK_SIGNAL_FUNC(panel_realize),
 					 NULL);
+
+	if(IS_FLOATING_WIDGET(panelw))
+		gtk_signal_connect_after(GTK_OBJECT(panelw), "size_allocate",
+					 GTK_SIGNAL_FUNC(floating_size_alloc),
+					 NULL);
 }
 
 /*send state change to all the panels*/
