@@ -30,6 +30,14 @@ struct _GlobalConfig {
 	int tile_depth[LAST_TILE];
 	int hide_panel_frame;
 	int tile_when_over;
+	int saturate_when_over;
+	int keys_enabled;
+	guint menu_keycode;
+	guint menu_state;
+	guint run_keycode;
+	guint run_state;
+	int menu_flags;
+	gboolean use_large_icons;
 };
 
 #endif /* PANEL_CONFIG_GLOBAL_H */
