@@ -10,10 +10,12 @@
 #include <string.h>
 #include <signal.h>
 #include <limits.h>
+#include <unistd.h>
 #include <gnome.h>
 #include <gdk/gdkx.h>
-
+#include <X11/keysym.h>
 #include "panel-include.h"
+#include "global-keys.h"
 
 /*#define PANEL_DEBUG 1*/
 
@@ -39,6 +41,10 @@ char *old_panel_cfg_path=NULL;
 /*list of all panel widgets created*/
 extern GSList *panel_list;
 
+extern char *kde_menudir;
+extern char *kde_icondir;
+extern char *kde_mini_icondir;
+
 int ss_cur_applet = 0;
 int ss_done_save = FALSE;
 gushort ss_cookie = 0;
@@ -81,6 +87,8 @@ apply_global_config(void)
 	static int small_icons_old = 0; /*same here*/
 	static int keep_bottom_old = -1;
 	static int autohide_size_old = -1;
+	static int menu_flags_old = -1;
+	static int old_use_large_icons = -1;
 	GSList *li;
 	panel_widget_change_global(global_config.explicit_hide_step_size,
 				   global_config.auto_hide_step_size,
@@ -99,7 +107,8 @@ apply_global_config(void)
 	  for rereading, hopefullly the user doesn't do this too often
 	  so that he doesn't have to reread his menus all the time:)*/
 	if(dot_buttons_old != global_config.show_dot_buttons ||
-	   small_icons_old != global_config.show_small_icons) {
+	   small_icons_old != global_config.show_small_icons ||
+	   old_use_large_icons != global_config.use_large_icons) {
 		GSList *li;
 		for(li=applets;li!=NULL;li=g_slist_next(li)) {
 			AppletInfo *info = li->data;
@@ -128,6 +137,7 @@ apply_global_config(void)
 	}
 	dot_buttons_old = global_config.show_dot_buttons;
 	small_icons_old = global_config.show_small_icons;
+	old_use_large_icons = global_config.use_large_icons;
 	send_tooltips_state(global_config.tooltips_enabled);
 
 	if(keep_bottom_old == -1 ||
@@ -160,9 +170,20 @@ apply_global_config(void)
 				gtk_widget_queue_resize (GTK_WIDGET (pd->panel));
 			}
 			basep_update_frame (BASEP_WIDGET (pd->panel));
+
+			if ((menu_flags_old != global_config.menu_flags) &&
+			    pd->menu) {
+				gtk_widget_destroy (pd->menu);
+				pd->menu = NULL;
+				pd->menu_age = 0;
+			}
+				
 		}
 	}
 	autohide_size_old = global_config.minimized_size;
+	menu_flags_old = global_config.menu_flags;
+
+	panel_global_keys_setup();
 }
 
 static int
@@ -223,7 +244,7 @@ send_applet_session_save (AppletInfo *info,
 	gtk_timeout_add(ss_timeout,session_save_timeout,GINT_TO_POINTER((int)ss_cookie));
 
 	CORBA_exception_init(&ev);
-	GNOME_Applet_session_save(obj,
+	GNOME_Applet_save_session(obj,
 				  (CORBA_char *)cfgpath,
 				  (CORBA_char *)globcfgpath,
 				  ss_cookie, &ev);
@@ -285,20 +306,36 @@ save_applet_configuration(AppletInfo *info)
 	case APPLET_EXTERN:
 		{
 			char *globalcfg;
+			char *s;
 			Extern *ext = info->data;
 
+			/*just in case the applet times out*/
+			gnome_config_set_string("id", EMPTY_ID);
+			gnome_config_pop_prefix();
+
 			globalcfg = g_concat_dir_and_file(PANEL_CONFIG_PATH,
 							  "Applet_All_Extern/");
 
+			/*this is the file path we must kill first */
+			g_string_sprintf(buf, "%sApplet_%d_Extern",
+					 PANEL_CONFIG_PATH, info->applet_id+1);
+			
+			/* this should really be: */
+			/*gnome_config_push_prefix("");
+			gnome_config_clean_file(buf->str);
+			gnome_config_pop_prefix();
+			gnome_config_sync();*/
+			/* but gnome-config.[ch] is broken ! */
+			s = gnome_config_get_real_path(buf->str);
+			unlink(s);
+			g_free(s);
+
+			gnome_config_sync();
+
 			/*this is the file path we pass to the applet for it's
 			  own config, this is a separate file, so that we */
 			g_string_sprintf(buf, "%sApplet_%d_Extern/",
 					 PANEL_CONFIG_PATH, info->applet_id+1);
-			gnome_config_clean_file(buf->str);
-			/*just in case the applet times out*/
-			gnome_config_set_string("id", EMPTY_ID);
-			gnome_config_pop_prefix();
-			gnome_config_sync();
 			/*have the applet do it's own session saving*/
 			send_applet_session_save(info,ext->applet,
 						 buf->str, globalcfg);
@@ -344,6 +381,7 @@ save_applet_configuration(AppletInfo *info)
 						menu->path);
 			gnome_config_set_int("main_menu_flags",
 					     menu->main_menu_flags);
+			gnome_config_set_bool("old_style_main",FALSE);
 			break;
 		}
 	case APPLET_LAUNCHER:
@@ -421,6 +459,8 @@ save_panel_configuration(gpointer data, gpointer user_data)
 	gnome_config_set_int("sz", panel->sz);
 
 	gnome_config_set_bool("fit_pixmap_bg", panel->fit_pixmap_bg);
+	gnome_config_set_bool("strech_pixmap_bg", panel->strech_pixmap_bg);
+	gnome_config_set_bool("rotate_pixmap_bg", panel->rotate_pixmap_bg);
 
 	gnome_config_set_string("backpixmap",
 				panel->back_pixmap ? panel->back_pixmap : "");
@@ -489,6 +529,9 @@ save_next_applet(void)
 	gnome_config_drop_all();
 }
 
+
+
+
 static void
 do_session_save(GnomeClient *client,
 		int complete_sync,
@@ -542,8 +585,11 @@ do_session_save(GnomeClient *client,
 #ifdef PANEL_DEBUG
 	printf(" 4\n"); fflush(stdout);
 #endif
-
 	gnome_config_pop_prefix ();
+
+	if(complete_sync)
+		save_tornoff();
+
 	gnome_config_sync();
 	
 	if(complete_sync || sync_applets) {
@@ -552,6 +598,7 @@ do_session_save(GnomeClient *client,
 		save_next_applet();
 	}
 
+
 #if 0 /*PANEL_DEBUG*/
 	puts("");
 #endif
@@ -662,11 +709,8 @@ panel_session_die (GnomeClient *client,
 void
 panel_quit(void)
 {
-	if(GNOME_CLIENT_CONNECTED(client))
-		gnome_client_request_save (client, GNOME_SAVE_BOTH, 1,
-					   GNOME_INTERACT_ANY, 0, 1);
-	else
-		gtk_main_quit();
+	gnome_client_request_save (client, GNOME_SAVE_BOTH, 1,
+				   GNOME_INTERACT_ANY, 0, 1);
 }
 
 static void
@@ -679,58 +723,53 @@ load_default_applets1(PanelWidget *panel)
 		"gnome/apps/Applications/Netscape.desktop",
 		NULL };
 	int i;
-	int flags = MAIN_MENU_SYSTEM|MAIN_MENU_USER|
-		MAIN_MENU_SYSTEM_SUB|MAIN_MENU_USER_SUB|
-		MAIN_MENU_APPLETS|MAIN_MENU_APPLETS_SUB;
 	char *p;
 
-	/*guess redhat menus*/
-	if(g_file_exists(REDHAT_MENUDIR))
-		flags |= MAIN_MENU_REDHAT|MAIN_MENU_REDHAT_SUB;
+	load_menu_applet(NULL, get_default_menu_flags (), 
+			 panels->data, 0, FALSE);
 
-	/*guess KDE menus*/
-	if(g_file_exists(KDE_MENUDIR))
-		flags |= MAIN_MENU_KDE|MAIN_MENU_KDE_SUB;
-
-	/*guess debian menus*/
-	if (g_file_exists(DEBIAN_MENUDIR))
-		flags |= MAIN_MENU_DEBIAN|MAIN_MENU_DEBIAN_SUB;
+	/* if bigger then 640+50+50 add a logout button */
+	if(gdk_screen_width()>640+50)
+		load_logout_applet(panel, 48, FALSE);
+	/* if bigger then 640+100+50 add a lock button */
+	if(gdk_screen_width()>640+100+50)
+		load_lock_applet(panel, 48*2, FALSE);
 	
-	load_menu_applet(NULL,flags, panels->data, 0);
-
-	/*load a Programs menu*/
-	p = gnome_datadir_file ("gnome/apps");
-	/*it really should exist*/
-	if(g_file_exists(p))
-		load_menu_applet(p,flags, panel, 1);
-	g_free(p);
-	
-	if(gdk_screen_width()>800) {
-		/*load up some launchers, but only if screen larger then 800*/
+	if(gdk_screen_width()>639) {
+		/*load up some launchers, but only if screen larger then 639*/
 		for(i=0;def_launchers[i]!=NULL;i++) {
 			p = gnome_datadir_file (def_launchers[i]);
 			/*int center = gdk_screen_width()/2;*/
 			if(p) {
-				load_launcher_applet(p,panel,48*3+i*48);
+				load_launcher_applet(p, panel,
+						     48*4+i*48, FALSE);
 				g_free(p);
 			}
 		}
 	}
 
-	load_extern_applet("tasklist_applet",NULL,
-			   panel,INT_MAX/2/*flush right*/,TRUE);
-	load_status_applet(panel,INT_MAX/2 + 1000/*flush right*/);
-	load_extern_applet("gen_util_clock",NULL,
-			   panel,INT_MAX/2 + 2000/*flush right*/,TRUE);
-	load_extern_applet("gen_util_mailcheck",NULL,
-			   panel,INT_MAX/2 + 3000/*flush right*/,TRUE);
+	load_extern_applet("tasklist_applet", NULL,
+			   panel, INT_MAX/2/*flush right*/, FALSE, TRUE);
+	/* if we are larger then 640, put the status dock on the main
+	   panel, otherwise on the auxiliaray below */
+	if(gdk_screen_width()>640)
+		load_status_applet(panel,
+				   INT_MAX/2 + 1000/*flush right*/, FALSE);
 }
 
 static void
 load_default_applets2(PanelWidget *panel)
 {
-	load_extern_applet("deskguide_applet",NULL,
-			   panel,0,TRUE);
+	load_extern_applet("deskguide_applet", NULL,
+			   panel, 0, FALSE, TRUE);
+	load_extern_applet("gen_util_clock", NULL,
+			   panel, 1000, FALSE, TRUE);
+	load_extern_applet("gen_util_mailcheck",NULL,
+			   panel, 2000, FALSE, TRUE);
+	/* if we are smaller or equal to 640, put the status dock on the
+	   auxiliary and not the main panel */
+	if(gdk_screen_width()<=640)
+		load_status_applet(panel, 3000, FALSE);
 }
 
 void
@@ -781,68 +820,106 @@ init_user_applets(void)
 		 INT_MAX/2 should allways be large enough */
 		pos += gnome_config_get_bool("right_stick=false")?INT_MAX/2:0;
 		
-		if(strcmp(applet_name,EXTERN_ID) == 0) {
+		if(strcmp(applet_name, EXTERN_ID) == 0) {
 			char *goad_id = gnome_config_get_string("goad_id");
 			if(goad_id && *goad_id) {
 				/*this is the config path to be passed to the
 				  applet when it loads*/
 				g_string_sprintf(buf,"%sApplet_%d_Extern/",
 						 PANEL_CONFIG_PATH,num);
-				load_extern_applet(goad_id,buf->str,panel,pos,TRUE);
+				load_extern_applet(goad_id, buf->str, 
+						   panel, pos, FALSE, TRUE);
 			}
 			g_free(goad_id);
-		} else if(strcmp(applet_name,LAUNCHER_ID) == 0) { 
+		} else if(strcmp(applet_name, LAUNCHER_ID) == 0) { 
 			char *params = gnome_config_get_string("parameters=");
-			load_launcher_applet(params,panel,pos);
+			load_launcher_applet(params, panel, pos, FALSE);
 			g_free(params);
-		} else if(strcmp(applet_name,LOGOUT_ID) == 0) { 
-			load_logout_applet(panel,pos);
-		} else if(strcmp(applet_name,LOCK_ID) == 0) {
-			load_lock_applet(panel,pos);
-		} else if(strcmp(applet_name,STATUS_ID) == 0) {
-			load_status_applet(panel,pos);
-		} else if(strcmp(applet_name,SWALLOW_ID) == 0) {
+		} else if(strcmp(applet_name, LOGOUT_ID) == 0) { 
+			load_logout_applet(panel, pos, FALSE);
+		} else if(strcmp(applet_name, LOCK_ID) == 0) {
+			load_lock_applet(panel, pos, FALSE);
+		} else if(strcmp(applet_name, STATUS_ID) == 0) {
+			load_status_applet(panel, pos, FALSE);
+		} else if(strcmp(applet_name, SWALLOW_ID) == 0) {
 			char *path = gnome_config_get_string("execpath=");
 			char *params = gnome_config_get_string("parameters=");
 			int width = gnome_config_get_int("width=0");
 			int height = gnome_config_get_int("height=0");
-			load_swallow_applet(path,params,width,height,panel,pos);
+			load_swallow_applet(path, params, width, height,
+					    panel, pos, FALSE);
 			g_free(path);
 			g_free(params);
-		} else if(strcmp(applet_name,MENU_ID) == 0) {
+		} else if(strcmp(applet_name, MENU_ID) == 0) {
 			char *params = gnome_config_get_string("parameters=");
+			char *s;
 			int type =
 				gnome_config_get_int("main_menu_type=-1");
-			int flags =
-				gnome_config_get_int("main_menu_flags=5");
+			int flags;
+			gboolean old_style = 
+				gnome_config_get_bool("old_style_main=true");
+
+			s = g_strdup_printf("main_menu_flags=%d",
+					    (int)(MAIN_MENU_SYSTEM_SUB |
+						  MAIN_MENU_USER_SUB |
+						  MAIN_MENU_APPLETS_SUB |
+						  MAIN_MENU_PANEL_SUB |
+						  MAIN_MENU_DESKTOP_SUB));
+			flags = gnome_config_get_int(s);
+			g_free(s);
+
 			if(type>=0) {
 				flags = 0;
 				if(type == X_MAIN_MENU_BOTH) {
 					flags |= MAIN_MENU_SYSTEM|MAIN_MENU_USER;
 				} else if(type == X_MAIN_MENU_SYSTEM) {
-					flags |= MAIN_MENU_SYSTEM|MAIN_MENU_USER|
+					flags |= MAIN_MENU_SYSTEM|
 						MAIN_MENU_USER_SUB;
 				} else {
-					flags |= MAIN_MENU_SYSTEM|MAIN_MENU_SYSTEM_SUB|
+					flags |= MAIN_MENU_SYSTEM_SUB|
 						MAIN_MENU_USER;
 				}
 				/*guess redhat menus*/
 				if(g_file_exists("/etc/X11/wmconfig"))
-					flags |= MAIN_MENU_REDHAT|MAIN_MENU_REDHAT_SUB;
+					flags |= MAIN_MENU_REDHAT_SUB;
 				/*guess KDE menus */
-				if(g_file_exists(KDE_MENUDIR))
-					flags |= MAIN_MENU_KDE|MAIN_MENU_KDE_SUB;
+				if(g_file_exists(kde_menudir))
+					flags |= MAIN_MENU_KDE_SUB;
 				/*guess debian menus*/
-				if (g_file_exists("/etc/menu-methods/gnome"))
-					flags |= MAIN_MENU_DEBIAN|MAIN_MENU_DEBIAN_SUB;
+				if (g_file_exists(DEBIAN_MENUDIR))
+					flags |= MAIN_MENU_DEBIAN_SUB;
 			}
-			load_menu_applet(params,flags,panel,pos);
+			if(old_style) {
+				/*this is needed to make panel properly
+				  read older style configs */
+				if(flags&MAIN_MENU_SYSTEM &&
+				   flags&MAIN_MENU_SYSTEM_SUB)
+					flags &=~ MAIN_MENU_SYSTEM;
+				if(flags&MAIN_MENU_USER &&
+				   flags&MAIN_MENU_USER_SUB)
+					flags &=~ MAIN_MENU_USER;
+				if(flags&MAIN_MENU_REDHAT &&
+				   flags&MAIN_MENU_REDHAT_SUB)
+					flags &=~ MAIN_MENU_USER;
+				if(flags&MAIN_MENU_DEBIAN &&
+				   flags&MAIN_MENU_DEBIAN_SUB)
+					flags &=~ MAIN_MENU_DEBIAN;
+				if(flags&MAIN_MENU_KDE &&
+				   flags&MAIN_MENU_KDE_SUB)
+					flags &=~ MAIN_MENU_KDE;
+				flags |= MAIN_MENU_APPLETS_SUB |
+					MAIN_MENU_PANEL_SUB |
+					MAIN_MENU_DESKTOP_SUB;
+			}
+
+			load_menu_applet(params, flags, panel, pos, FALSE);
 			g_free(params);
-		} else if(strcmp(applet_name,DRAWER_ID) == 0) {
+		} else if(strcmp(applet_name, DRAWER_ID) == 0) {
 			int mypanel = gnome_config_get_int("parameters=-1");
 			char *pixmap = gnome_config_get_string("pixmap=");
 			char *tooltip = gnome_config_get_string("tooltip=");
-			load_drawer_applet(mypanel,pixmap,tooltip,panel,pos);
+			load_drawer_applet(mypanel, pixmap, tooltip,
+					   panel, pos, FALSE);
 			g_free(pixmap);
 			g_free(tooltip);
 		} else
@@ -850,7 +927,7 @@ init_user_applets(void)
 		gnome_config_pop_prefix();
 		g_free(applet_name);
 	}
-	g_string_free(buf,TRUE);
+	g_string_free(buf, TRUE);
 }
 
 void
@@ -879,6 +956,8 @@ init_user_panels(void)
 					PANEL_BACK_NONE,
 					NULL,
 					TRUE,
+					FALSE,
+					TRUE,
 					NULL);
 		panel_setup(panel);
 		gtk_widget_show(panel);
@@ -896,6 +975,8 @@ init_user_panels(void)
 					   PANEL_BACK_NONE,
 					   NULL,
 					   TRUE,
+					   FALSE,
+					   TRUE,
 					   NULL);
 		panel_setup(panel);
 		gtk_widget_show(panel);
@@ -915,13 +996,15 @@ init_user_panels(void)
 	for(num=1;num<=count;num++) {
 		PanelType type;
 		PanelBackType back_type;
-		PanelSizeType sz;
+		int sz;
 		BasePState state;
 		BasePMode mode;
 		BorderEdge edge;
 		char *back_pixmap, *color;
 		GdkColor back_color = {0,0,0,1};
-		int fit_pixmap_bg;
+		gboolean fit_pixmap_bg;
+		gboolean strech_pixmap_bg;
+		gboolean rotate_pixmap_bg;
 		int hidebuttons_enabled;
 		int hidebutton_pixmaps_enabled;
 		
@@ -942,9 +1025,24 @@ init_user_panels(void)
 		g_string_sprintf(buf,"back_type=%d",PANEL_BACK_NONE);
 		back_type=gnome_config_get_int(buf->str);
 		fit_pixmap_bg = gnome_config_get_bool ("fit_pixmap_bg=TRUE");
+		strech_pixmap_bg = gnome_config_get_bool ("strech_pixmap_bg=FALSE");
+		rotate_pixmap_bg = gnome_config_get_bool ("rotate_pixmap_bg=FALSE");
 
 		g_string_sprintf(buf,"sz=%d", SIZE_STANDARD);
 		sz=gnome_config_get_int(buf->str);
+		if(sz<0) sz = 0;
+
+		/*a hack to allow for old config files to be read correctly*/
+		/*don't update this if new sizes (SIZE_SMALL) are added*/
+		if(sz<4) {
+			switch(sz) {
+			case 0: sz = SIZE_TINY; break;
+			case 1: sz = SIZE_STANDARD; break;
+			case 2: sz = SIZE_LARGE; break;
+			case 3: sz = SIZE_HUGE; break;
+			default: break;
+			}
+		}
 		
 		/*now for type specific config*/
 
@@ -971,7 +1069,10 @@ init_user_panels(void)
 						 hidebuttons_enabled,
 						 hidebutton_pixmaps_enabled,
 						 back_type, back_pixmap,
-						 fit_pixmap_bg, &back_color);
+						 fit_pixmap_bg,
+						 strech_pixmap_bg,
+						 rotate_pixmap_bg,
+						 &back_color);
 			break;
 		case ALIGNED_PANEL: {
 			AlignedAlignment align;
@@ -986,7 +1087,10 @@ init_user_panels(void)
 						    hidebuttons_enabled,
 						    hidebutton_pixmaps_enabled,
 						    back_type, back_pixmap,
-						    fit_pixmap_bg, &back_color);
+						    fit_pixmap_bg,
+						    strech_pixmap_bg,
+						    rotate_pixmap_bg,
+						    &back_color);
 			break;
 		}
 		case SLIDING_PANEL: {
@@ -1005,7 +1109,10 @@ init_user_panels(void)
 						    hidebuttons_enabled,
 						    hidebutton_pixmaps_enabled,
 						    back_type, back_pixmap,
-						    fit_pixmap_bg, &back_color);
+						    fit_pixmap_bg,
+						    strech_pixmap_bg,
+						    rotate_pixmap_bg,
+						    &back_color);
 			break;
 		}
 		case DRAWER_PANEL: {
@@ -1022,7 +1129,10 @@ init_user_panels(void)
 						   hidebuttons_enabled,
 						   hidebutton_pixmaps_enabled,
 						   back_type, back_pixmap,
-						   fit_pixmap_bg, &back_color);
+						   fit_pixmap_bg,
+						   strech_pixmap_bg,
+						   rotate_pixmap_bg,
+						   &back_color);
 #if 0
 			g_string_sprintf (buf, "temp_state=%d", BASEP_SHOWN);
 			temp_state = gnome_config_get_int (buf->str);
@@ -1045,7 +1155,10 @@ init_user_panels(void)
 						     hidebuttons_enabled,
 						     hidebutton_pixmaps_enabled,
 						     back_type, back_pixmap,
-						     fit_pixmap_bg, &back_color);
+						     fit_pixmap_bg,
+						     strech_pixmap_bg,
+						     rotate_pixmap_bg,
+						     &back_color);
 			break;
 		}
 		default:
@@ -1066,6 +1179,7 @@ init_user_panels(void)
 	g_string_free(buf,TRUE);
 }
 
+
 void
 load_up_globals(void)
 {
@@ -1090,6 +1204,11 @@ load_up_globals(void)
 	global_config.hungry_menus =
 		gnome_config_get_bool("hungry_menus=TRUE");
 
+	global_config.use_large_icons =
+		gnome_config_get_bool(gdk_screen_height () > 768 
+				      ? "use_large_icons=TRUE"
+				      : "use_large_icons=FALSE");
+
 	global_config.off_panel_popups =
 		gnome_config_get_bool("off_panel_popups=TRUE");
 		
@@ -1117,6 +1236,18 @@ load_up_globals(void)
 	g_string_sprintf(buf,"movement_type=%d", PANEL_SWITCH_MOVE);
 	global_config.movement_type=gnome_config_get_int(buf->str);
 
+	global_config.keys_enabled=gnome_config_get_bool("keys_enabled=TRUE");
+
+	g_string_sprintf(buf,"menu_keycode=%d", XK_RIGHT_MENU);
+	global_config.menu_keycode=gnome_config_get_int(buf->str);
+
+	global_config.menu_state=gnome_config_get_int("menu_state=0");
+
+	g_string_sprintf(buf,"run_keycode=%d", XK_r);
+	global_config.run_keycode=gnome_config_get_int(buf->str);
+
+	global_config.run_state=gnome_config_get_int("run_state=0");
+	
 	global_config.applet_padding=gnome_config_get_int("applet_padding=3");
 
 	global_config.autoraise = gnome_config_get_bool("autoraise=TRUE");
@@ -1127,8 +1258,13 @@ load_up_globals(void)
 	global_config.simple_movement = gnome_config_get_bool("simple_movement=FALSE");
 	global_config.hide_panel_frame = gnome_config_get_bool("hide_panel_frame=FALSE");
 	global_config.tile_when_over = gnome_config_get_bool("tile_when_over=FALSE");
+	global_config.saturate_when_over = gnome_config_get_bool("saturate_when_over=TRUE");
+	
+	g_string_sprintf (buf, "menu_flags=%d", get_default_menu_flags ());
+	global_config.menu_flags = gnome_config_get_int (buf->str);
+
 	for(i=0;i<LAST_TILE;i++) {
-		g_string_sprintf(buf,"tiles_enabled_%d=TRUE",i);
+		g_string_sprintf(buf,"tiles_enabled_%d=FALSE",i);
 		global_config.tiles_enabled[i] =
 			gnome_config_get_bool(buf->str);
 
@@ -1209,6 +1345,8 @@ convert_write_config(void)
 			      global_config.hide_panel_frame);
 	gnome_config_set_bool("tile_when_over",
 			      global_config.tile_when_over);
+	gnome_config_set_bool("saturate_when_over",
+			      global_config.saturate_when_over);
 	buf = g_string_new(NULL);
 	for(i=0;i<LAST_TILE;i++) {
 		g_string_sprintf(buf,"tiles_enabled_%d",i);
