@@ -19,6 +19,8 @@ BEGIN_GNOME_DECLS
 
 #define DEFAULT_PANEL_NUM 0
 
+#define PANEL_CONFIG_PATH "/panel.d/default/"
+
 int panel_session_save (GnomeClient *client,
 			int phase,
 			GnomeSaveStyle save_style,
@@ -27,6 +29,9 @@ int panel_session_save (GnomeClient *client,
 			int fast,
 			gpointer client_data);
 
+void save_next_applet(void);
+
+
 int panel_session_die (GnomeClient *client,
 			gpointer client_data);
 
@@ -41,6 +46,9 @@ void init_user_panels(void);
 
 void apply_global_config(void);
 
+/* just temporary so that we can bridge the gap between old and new config */
+void convert_old_config(void);
+
 END_GNOME_DECLS
 
 #endif
